@@ -236,6 +236,23 @@ REST_API_VERSION_HISTORY = """REST API Version History:
              ``PUT /flavors/{flavor_id}/os-extra_specs/{id}`` APIs.
     * 2.87 - Adds support for rescuing boot from volume instances when the
              compute host reports the COMPUTE_BFV_RESCUE capability trait.
+    * 2.88 - Drop statistics-style fields from the ``/os-hypervisors/detail``
+             and ``/os-hypervisors/{hypervisor_id}`` APIs, and remove the
+             ``/os-hypervisors/statistics`` and
+             ``/os-hypervisors/{hypervisor_id}/uptime`` APIs entirely.
+    * 2.89 - Add ``attachment_id``, ``bdm_uuid`` and remove ``id`` from the
+             responses of ``GET  /servers/{server_id}/os-volume_attachments``
+             and ``GET /servers/{server_id}/os-volume_attachments/{volume_id}``
+    * 2.90 - Add support for requesting a specific hostname when creating,
+             updating or rebuilding an instance. The
+             ``OS-EXT-SRV-ATTR:hostname`` attribute is now returned in various
+             server responses regardless of policy configuration.
+    * 2.91 - Add support to unshelve instance to a specific host and
+             to pin/unpin AZ.
+    * 2.92 - Drop generation of keypair, add keypair name validation on
+             ``POST /os-keypairs`` and allow including @ and dot (.) characters
+             in keypair name.
+    * 2.93 - Add support for volume backed server rebuild.
 """
 
 # The minimum and maximum versions of the API supported
@@ -244,7 +261,7 @@ REST_API_VERSION_HISTORY = """REST API Version History:
 # Note(cyeoh): This only applies for the v2.1 API once microversions
 # support is fully merged. It does not affect the V2 API.
 _MIN_API_VERSION = '2.1'
-_MAX_API_VERSION = '2.87'
+_MAX_API_VERSION = '2.93'
 DEFAULT_API_VERSION = _MIN_API_VERSION
 
 # Almost all proxy APIs which are related to network, images and baremetal
