@@ -175,11 +175,18 @@ class InterfaceAttachmentController(wsgi.Controller):
                 exception.SecurityGroupCannotBeApplied,
                 exception.NetworkInterfaceTaggedAttachNotSupported,
                 exception.NetworksWithQoSPolicyNotSupported,
-                exception.InterfaceAttachPciClaimFailed) as e:
+                exception.InterfaceAttachPciClaimFailed,
+                exception.InterfaceAttachResourceAllocationFailed,
+                exception.ForbiddenPortsWithAccelerator,
+                exception.ForbiddenWithRemoteManagedPorts,
+                exception.ExtendedResourceRequestOldCompute,
+                ) as e:
             raise exc.HTTPBadRequest(explanation=e.format_message())
-        except (exception.InstanceIsLocked,
-                exception.FixedIpAlreadyInUse,
-                exception.PortInUse) as e:
+        except (
+            exception.InstanceIsLocked,
+            exception.FixedIpAlreadyInUse,
+            exception.PortInUse,
+        ) as e:
             raise exc.HTTPConflict(explanation=e.format_message())
         except (exception.PortNotFound,
                 exception.NetworkNotFound) as e:
@@ -196,7 +203,7 @@ class InterfaceAttachmentController(wsgi.Controller):
         return self.show(req, server_id, vif['id'])
 
     @wsgi.response(202)
-    @wsgi.expected_errors((404, 409, 501))
+    @wsgi.expected_errors((400, 404, 409, 501))
     def delete(self, req, server_id, id):
         """Detach an interface from an instance."""
         context = req.environ['nova.context']
@@ -220,3 +227,5 @@ class InterfaceAttachmentController(wsgi.Controller):
         except exception.InstanceInvalidState as state_error:
             common.raise_http_conflict_for_instance_invalid_state(state_error,
                     'detach_interface', server_id)
+        except exception.ForbiddenPortsWithAccelerator as e:
+            raise exc.HTTPBadRequest(explanation=e.format_message())
