@@ -16,7 +16,6 @@ from keystoneauth1 import exceptions as ks_exc
 from oslo_log import log as logging
 from oslo_utils import strutils
 from oslo_utils import uuidutils
-import six
 import webob.exc
 
 from nova.api.openstack import api_version_request
@@ -49,13 +48,10 @@ class ServiceController(wsgi.Controller):
         self.actions = {"enable": self._enable,
                         "disable": self._disable,
                         "disable-log-reason": self._disable_log_reason}
-        self._placementclient = None  # Lazy-load on first access.
 
     @property
     def placementclient(self):
-        if self._placementclient is None:
-            self._placementclient = report.SchedulerReportClient()
-        return self._placementclient
+        return report.report_client_singleton()
 
     def _get_services(self, req):
         # The API services are filtered out since they are not RPC services
@@ -190,6 +186,9 @@ class ServiceController(wsgi.Controller):
         host = body['host']
         binary = body['binary']
 
+        if binary == 'nova-compute' and forced_down is False:
+            self._check_for_evacuations(context, host)
+
         ret_value = {'service': {'host': host,
                                  'binary': binary,
                                  'forced_down': forced_down}}
@@ -226,6 +225,25 @@ class ServiceController(wsgi.Controller):
 
         return action(body, context)
 
+    def _check_for_evacuations(self, context, hostname):
+        # NOTE(lyarwood): When forcing a compute service back up ensure that
+        # there are no evacuation migration records against this host as the
+        # source that are marked as done, suggesting that the compute service
+        # hasn't restarted and moved such records to a completed state.
+        filters = {
+            'source_compute': hostname,
+            'status': 'done',
+            'migration_type': objects.fields.MigrationType.EVACUATION,
+        }
+        if any(objects.MigrationList.get_by_filters(context, filters)):
+            msg = _("Unable to force up host %(host)s as `done` evacuation "
+                    "migration records remain associated with the host. "
+                    "Ensure the compute service has been restarted, "
+                    "allowing these records to move to `completed` before "
+                    "retrying this request.") % {'host': hostname}
+            # TODO(lyarwood): Move to 409 HTTPConflict under a new microversion
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
     @wsgi.response(204)
     @wsgi.expected_errors((400, 404, 409))
     def delete(self, req, id):
@@ -285,7 +303,8 @@ class ServiceController(wsgi.Controller):
                     # get_all_host() above will raise, as there are no longer
                     # any compute node records for the host. Catch it here and
                     # continue to allow compute service deletion.
-                    pass
+                    LOG.info('Deleting compute service with no associated '
+                             'compute nodes.')
 
                 aggrs = self.aggregate_api.get_aggregates_by_host(context,
                                                                   service.host)
@@ -305,8 +324,8 @@ class ServiceController(wsgi.Controller):
                         LOG.error(
                             "Failed to delete compute node resource provider "
                             "for compute node %s: %s",
-                            compute_node.uuid, six.text_type(e))
-                # remove the host_mapping of this host.
+                            compute_node.uuid, str(e))
+                # Remove the host_mapping of this host.
                 try:
                     hm = objects.HostMapping.get_by_host(context, service.host)
                     hm.destroy()
@@ -462,6 +481,8 @@ class ServiceController(wsgi.Controller):
         if 'forced_down' in body:
             service.forced_down = strutils.bool_from_string(
                 body['forced_down'], strict=True)
+            if service.forced_down is False:
+                self._check_for_evacuations(context, service.host)
 
         # Check to see if anything was actually updated since the schema does
         # not define any required fields.
