@@ -27,7 +27,7 @@ class SuspendServerController(wsgi.Controller):
         self.compute_api = compute.API()
 
     @wsgi.response(202)
-    @wsgi.expected_errors((404, 409))
+    @wsgi.expected_errors((403, 404, 409, 400))
     @wsgi.action('suspend')
     def _suspend(self, req, id, body):
         """Permit admins to suspend the server."""
@@ -38,12 +38,13 @@ class SuspendServerController(wsgi.Controller):
                         target={'user_id': server.user_id,
                                 'project_id': server.project_id})
             self.compute_api.suspend(context, server)
-        except (exception.OperationNotSupportedForSEV,
-                exception.InstanceIsLocked) as e:
+        except exception.InstanceIsLocked as e:
             raise exc.HTTPConflict(explanation=e.format_message())
         except exception.InstanceInvalidState as state_error:
             common.raise_http_conflict_for_instance_invalid_state(state_error,
                     'suspend', id)
+        except exception.ForbiddenPortsWithAccelerator as e:
+            raise exc.HTTPBadRequest(explanation=e.format_message())
 
     @wsgi.response(202)
     @wsgi.expected_errors((404, 409))
@@ -51,8 +52,9 @@ class SuspendServerController(wsgi.Controller):
     def _resume(self, req, id, body):
         """Permit admins to resume the server from suspend."""
         context = req.environ['nova.context']
-        context.can(ss_policies.POLICY_ROOT % 'resume')
         server = common.get_instance(self.compute_api, context, id)
+        context.can(ss_policies.POLICY_ROOT % 'resume',
+                    target={'project_id': server.project_id})
         try:
             self.compute_api.resume(context, server)
         except exception.InstanceIsLocked as e:
