@@ -41,7 +41,6 @@ Possible values:
 * ``ironic.IronicDriver``
 * ``vmwareapi.VMwareVCDriver``
 * ``hyperv.HyperVDriver``
-* ``powervm.PowerVMDriver``
 * ``zvm.ZVMDriver``
 """),
     cfg.BoolOpt('allow_resize_to_same_host',
@@ -51,7 +50,8 @@ Allow destination machine to match source for resize. Useful when
 testing in single-host environments. By default it is not allowed
 to resize to the same host. Setting this option to true will add
 the same host to the destination options. Also set to true
-if you allow the ServerGroupAffinityFilter and need to resize.
+if you allow the ServerGroupAffinityFilter and need to resize. For changes to
+this option to take effect, the nova-api service needs to be restarted.
 """),
     cfg.ListOpt('non_inheritable_image_properties',
         default=['cache_in_nova', 'bittorrent'],
@@ -199,7 +199,7 @@ The template will be rendered using Jinja2 template engine, and receive a
 top-level key called ``interfaces``. This key will contain a list of
 dictionaries, one for each interface.
 
-Refer to the cloudinit documentaion for more information:
+Refer to the cloudinit documentation for more information:
 
   https://cloudinit.readthedocs.io/en/latest/topics/datasources.html
 
@@ -296,8 +296,8 @@ Generic property to specify the pointer type.
 Input devices allow interaction with a graphical framebuffer. For
 example to provide a graphic tablet for absolute cursor movement.
 
-If set, the 'hw_pointer_model' image property takes precedence over
-this configuration option.
+If set, either the ``hw_input_bus`` or ``hw_pointer_model`` image metadata
+properties will take precedence over this configuration option.
 
 Related options:
 
@@ -305,6 +305,21 @@ Related options:
   agent disabled. When used with libvirt the instance mode should be
   configured as HVM.
  """),
+    cfg.IntOpt('reimage_timeout_per_gb',
+        default=20,
+        min=1,
+        help="""
+Timeout for reimaging a volume.
+
+Number of seconds to wait for volume-reimaged events to arrive before
+continuing or failing.
+
+This is a per gigabyte time which has a default value of 20 seconds and
+will be multiplied by the GB size of image. Eg: an image of 6 GB will have
+a timeout of 20 * 6 = 120 seconds.
+Try increasing the timeout if the image copy per GB takes more time and you
+are hitting timeout failures.
+"""),
 ]
 
 resource_tracker_opts = [
@@ -425,9 +440,7 @@ allocation_ratio_opts = [
 Virtual CPU to physical CPU allocation ratio.
 
 This option is used to influence the hosts selected by the Placement API by
-configuring the allocation ratio for ``VCPU`` inventory. In addition, the
-``AggregateCoreFilter`` (deprecated) will fall back to this configuration value
-if no per-aggregate setting is found.
+configuring the allocation ratio for ``VCPU`` inventory.
 
 .. note::
 
@@ -458,9 +471,7 @@ Related options:
 Virtual RAM to physical RAM allocation ratio.
 
 This option is used to influence the hosts selected by the Placement API by
-configuring the allocation ratio for ``MEMORY_MB`` inventory. In addition, the
-``AggregateRamFilter`` (deprecated) will fall back to this configuration value
-if no per-aggregate setting is found.
+configuring the allocation ratio for ``MEMORY_MB`` inventory.
 
 .. note::
 
@@ -486,9 +497,7 @@ Related options:
 Virtual disk to physical disk allocation ratio.
 
 This option is used to influence the hosts selected by the Placement API by
-configuring the allocation ratio for ``DISK_GB`` inventory. In addition, the
-``AggregateDiskFilter`` (deprecated) will fall back to this configuration value
-if no per-aggregate setting is found.
+configuring the allocation ratio for ``DISK_GB`` inventory.
 
 When configured, a ratio greater than 1.0 will result in over-subscription of
 the available physical disk, which can be useful for more efficiently packing
@@ -986,23 +995,57 @@ Additional documentation is available here:
   https://docs.openstack.org/nova/latest/admin/managing-resource-providers.html
 
 """),
-]
+    cfg.ListOpt('image_type_exclude_list',
+                default=[],
+                help="""
+A list of image formats that should not be advertised as supported by this
+compute node.
 
-interval_opts = [
-    cfg.IntOpt('bandwidth_poll_interval',
-        default=600,
-        help="""
-Interval to pull network bandwidth usage info.
-
-Not supported on all hypervisors. If a hypervisor doesn't support bandwidth
-usage, it will not get the info in the usage events.
+In some situations, it may be desirable to have a compute node
+refuse to support an expensive or complex image format. This factors into
+the decisions made by the scheduler about which compute node to select when
+booted with a given image.
 
 Possible values:
 
-* 0: Will run at the default periodic interval.
-* Any value < 0: Disables the option.
-* Any positive integer in seconds.
+* Any glance image ``disk_format`` name (i.e. ``raw``, ``qcow2``, etc)
+
+Related options:
+
+* ``[scheduler]query_placement_for_image_type_support`` - enables
+  filtering computes based on supported image types, which is required
+  to be enabled for this to take effect.
 """),
+    cfg.ListOpt('vmdk_allowed_types',
+                default=['streamOptimized', 'monolithicSparse'],
+                help="""
+A list of strings describing allowed VMDK "create-type" subformats
+that will be allowed. This is recommended to only include
+single-file-with-sparse-header variants to avoid potential host file
+exposure due to processing named extents. If this list is empty, then no
+form of VMDK image will be allowed.
+"""),
+    cfg.BoolOpt('packing_host_numa_cells_allocation_strategy',
+        default=False,
+        help="""
+This option controls allocation strategy used to choose NUMA cells on host for
+placing VM's NUMA cells (for VMs with defined numa topology). By
+default host's NUMA cell with more resources consumed will be chosen last for
+placing attempt. When the packing_host_numa_cells_allocation_strategy variable
+is set to ``False``, host's NUMA cell with more resources available will be
+used. When set to ``True`` cells with some usage will be packed with VM's cell
+until it will be completely exhausted, before a new free host's cell will be
+used.
+
+Possible values:
+
+* ``True``: Packing VM's NUMA cell on most used host NUMA cell.
+* ``False``: Spreading VM's NUMA cell on host's NUMA cells with more resources
+  available.
+"""),
+]
+
+interval_opts = [
     cfg.IntOpt('sync_power_state_interval',
         default=600,
         help="""
