@@ -127,6 +127,20 @@ mock_attribute_re = re.compile(r"[\.\(](retrun_value)[,=\s]")
 # Regex for useless assertions
 useless_assertion_re = re.compile(
     r"\.((assertIsNone)\(None|(assertTrue)\((True|\d+|'.+'|\".+\")),")
+# Regex for misuse of assert_has_calls
+mock_assert_has_calls_re = re.compile(r"\.assert_has_calls\s?=")
+
+# Regex for catching aliasing mock.Mock class in test
+mock_class_aliasing_re = re.compile(
+    r"^[A-Za-z0-9_.]+\s*=\s*mock\.(Magic|NonCallable)?Mock$")
+# Regex for catching aliasing mock.Mock class in test
+mock_class_as_new_value_in_patching_re = re.compile(
+    r"mock\.patch(\.object)?.* new=mock\.(Magic|NonCallable)?Mock[^(]")
+# Regex for direct use of oslo.concurrency lockutils.ReaderWriterLock
+rwlock_re = re.compile(
+    r"(?P<module_part>(oslo_concurrency\.)?(lockutils|fasteners))"
+    r"\.ReaderWriterLock\(.*\)")
+six_re = re.compile(r"^(import six(\..*)?|from six(\..*)? import .*)$")
 
 
 class BaseASTChecker(ast.NodeVisitor):
@@ -182,8 +196,8 @@ def import_no_db_in_virt(logical_line, filename):
     N307
     """
     if "nova/virt" in filename and not filename.endswith("fake.py"):
-        if logical_line.startswith("from nova.db import api"):
-            yield (0, "N307: nova.db.api import not allowed in nova/virt/*")
+        if logical_line.startswith("from nova.db.main import api"):
+            yield (0, "N307: nova.db.* import not allowed in nova/virt/*")
 
 
 @core.flake8ext
@@ -922,3 +936,145 @@ def useless_assertion(logical_line, filename):
         match = useless_assertion_re.search(logical_line)
         if match:
             yield (0, msg % (match.group(2) or match.group(3)))
+
+
+@core.flake8ext
+def check_assert_has_calls(logical_line, filename):
+    """Check misuse of assert_has_calls.
+
+    Not correct: mock_method.assert_has_calls = [mock.call(0)]
+    Correct:     mock_method.assert_has_calls([mock.call(0)])
+
+    N366
+    """
+    msg = "N366: The assert_has_calls is a method rather than a variable."
+    if ('nova/tests/' in filename and
+            mock_assert_has_calls_re.search(logical_line)):
+        yield (0, msg)
+
+
+@core.flake8ext
+def do_not_alias_mock_class(logical_line, filename):
+    """Check for aliasing Mock class
+
+    Aliasing Mock class almost always a bad idea. Consider the test code
+    trying to catch the instantiation of the Rados class but instead
+    introducing a global change on the Mock object:
+    https://github.com/openstack/nova/blob/10b1dc84f47a71061340f8e0ae0fe32dca44061a/nova/tests/unit/storage/test_rbd.py#L122-L125
+    After this code every test that assumes that mock.Mock().shutdown is a new
+    auto-generated mock.Mock() object will fail a shutdown is now defined in
+    the Mock class level and therefore surviving between test cases.
+
+    N367
+    """
+    if 'nova/tests/' in filename:
+        res = mock_class_aliasing_re.match(logical_line)
+        if res:
+            yield (
+                0,
+                "N367: Aliasing mock.Mock class is dangerous as it easy to "
+                "introduce class level changes in Mock that survives "
+                "between test cases. If you want to mock object creation "
+                "then mock the class under test with a mock _instance_ and "
+                "set the return_value of the mock to return mock instances. "
+                "See for example: "
+                "https://review.opendev.org/c/openstack/nova/+/805657"
+            )
+
+
+@core.flake8ext
+def do_not_use_mock_class_as_new_mock_value(logical_line, filename):
+    """Check if mock.Mock class is used during set up of a patcher as new
+    kwargs.
+
+    The mock.patch and mock.patch.object takes a `new` kwargs and use that
+    value as the replacement during the patching. Using new=mock.Mock
+    (instead of new=mock.Mock() or new_callable=mock.Mock) results in code
+    under test pointing to the Mock class. This is almost always a wrong thing
+    as any changes on that class will leak between test cases uncontrollably.
+
+    N368
+    """
+    if 'nova/tests/' in filename:
+        res = mock_class_as_new_value_in_patching_re.search(logical_line)
+        if res:
+            yield (
+                0,
+                "N368: Using mock.patch(..., new=mock.Mock) causes that the "
+                "patching will introduce the Mock class as replacement value "
+                "instead of a mock object. Any change on the Mock calls will "
+                "leak out from the test and can cause interference. "
+                "Use new=mock.Mock() or new_callable=mock.Mock instead."
+            )
+
+
+@core.flake8ext
+def check_lockutils_rwlocks(logical_line):
+    """Check for direct use of oslo.concurrency lockutils.ReaderWriterLock()
+
+    oslo.concurrency lockutils uses fasteners.ReaderWriterLock to provide
+    read/write locks and fasteners calls threading.current_thread() to track
+    and identify lock holders and waiters. The eventlet implementation of
+    current_thread() only supports greenlets of type GreenThread, else it falls
+    back on the native threading.current_thread() method.
+
+    See https://github.com/eventlet/eventlet/issues/731 for details.
+
+    N369
+    """
+    msg = ("N369: %(module)s.ReaderWriterLock() does not "
+           "function correctly with eventlet patched code. "
+           "Use nova.utils.ReaderWriterLock() instead.")
+    match = re.match(rwlock_re, logical_line)
+    if match:
+        yield (
+            0,
+            msg % {'module': match.group('module_part')}
+        )
+
+
+@core.flake8ext
+def check_six(logical_line):
+    """Check for use of six
+
+    nova is now Python 3-only so we don't want six. However, people might use
+    it out of habit and it will likely work since six is a transitive
+    dependency.
+
+    N370
+    """
+    match = re.match(six_re, logical_line)
+    if match:
+        yield (0, "N370: Don't use or import six")
+
+
+@core.flake8ext
+def import_stock_mock(logical_line):
+    """Use python's mock, not the mock library.
+
+    Since we `dropped support for python 2`__, we no longer need to use the
+    mock library, which existed to backport py3 functionality into py2. Change
+    Ib44b5bff657c8e76c4f701e14d51a4efda3f6d32 cut over to importing the stock
+    mock, which must be done by saying::
+
+        from unittest import mock
+
+    ...because if you say::
+
+        import mock
+
+    ...you may be getting the stock mock; or, due to transitive dependencies in
+    the environment, the library mock. This check can be removed in the future
+    (and we can start saying ``import mock`` again) if we manage to purge these
+    transitive dependencies.
+
+    .. __: https://review.opendev.org/#/c/687954/
+
+    N371
+    """
+    if logical_line == 'import mock' or logical_line.startswith('from mock'):
+        yield (
+            0,
+            "N371: You must explicitly import python's mock: "
+            "``from unittest import mock``"
+        )
