@@ -18,7 +18,6 @@ import re
 from cursive import signature_utils
 from oslo_serialization import jsonutils
 from oslo_versionedobjects import fields
-import six
 
 from nova import exception
 from nova.i18n import _
@@ -98,7 +97,7 @@ class ResourceClass(fields.StringPattern):
 
     @staticmethod
     def coerce(obj, attr, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             uppered = value.upper()
             if ResourceClass._REGEX.match(uppered):
                 return uppered
@@ -178,7 +177,7 @@ class Architecture(BaseNovaEnum):
         :returns: the canonicalized host architecture
         """
 
-        return cls.canonicalize(os.uname()[4])
+        return cls.canonicalize(os.uname().machine)
 
     @classmethod
     def is_valid(cls, name):
@@ -261,6 +260,14 @@ class BlockDeviceType(BaseNovaEnum):
     ALL = (CDROM, DISK, FLOPPY, FS, LUN)
 
 
+class BlockDeviceEncryptionFormatType(BaseNovaEnum):
+    PLAIN = 'plain'
+    LUKS = 'luks'
+    LUKSv2 = 'luksv2'
+
+    ALL = (PLAIN, LUKS, LUKSv2)
+
+
 class ConfigDrivePolicy(BaseNovaEnum):
     OPTIONAL = "optional"
     MANDATORY = "mandatory"
@@ -466,6 +473,14 @@ class ImageSignatureKeyType(BaseNovaEnum):
     )
 
 
+class InputBus(BaseNovaEnum):
+
+    USB = 'usb'
+    VIRTIO = 'virtio'
+
+    ALL = (USB, VIRTIO)
+
+
 class MigrationType(BaseNovaEnum):
 
     MIGRATION = 'migration'  # cold migration
@@ -570,8 +585,9 @@ class VideoModel(BaseNovaEnum):
     VIRTIO = 'virtio'
     GOP = 'gop'
     NONE = 'none'
+    BOCHS = 'bochs'
 
-    ALL = (CIRRUS, QXL, VGA, VMVGA, XEN, VIRTIO, GOP, NONE)
+    ALL = (CIRRUS, QXL, VGA, VMVGA, XEN, VIRTIO, GOP, NONE, BOCHS)
 
 
 class VIFModel(BaseNovaEnum):
@@ -600,6 +616,16 @@ class VIFModel(BaseNovaEnum):
         return super(VIFModel, self).coerce(obj, attr, value)
 
 
+class VIOMMUModel(BaseNovaEnum):
+
+    INTEL = 'intel'
+    SMMUV3 = 'smmuv3'
+    VIRTIO = 'virtio'
+    AUTO = 'auto'
+
+    ALL = (INTEL, SMMUV3, VIRTIO, AUTO)
+
+
 class VMMode(BaseNovaEnum):
     """Represents possible vm modes for instances.
 
@@ -750,8 +776,12 @@ class PciDeviceType(BaseNovaEnum):
     STANDARD = "type-PCI"
     SRIOV_PF = "type-PF"
     SRIOV_VF = "type-VF"
+    # NOTE(sean-k-mooney): The DB field is Column(String(8), nullable=False)
+    # type-vdpa is 9 long...and as Jay notes above the prefix is silly so
+    # for the new vdpa value we drop the prefix to avoid a DB migration
+    VDPA = "vdpa"
 
-    ALL = (STANDARD, SRIOV_PF, SRIOV_VF)
+    ALL = (STANDARD, SRIOV_PF, SRIOV_VF, VDPA)
 
 
 class PCINUMAAffinityPolicy(BaseNovaEnum):
@@ -759,8 +789,9 @@ class PCINUMAAffinityPolicy(BaseNovaEnum):
     REQUIRED = "required"
     LEGACY = "legacy"
     PREFERRED = "preferred"
+    SOCKET = "socket"
 
-    ALL = (REQUIRED, LEGACY, PREFERRED)
+    ALL = (REQUIRED, LEGACY, PREFERRED, SOCKET)
 
 
 class DiskFormat(BaseNovaEnum):
@@ -793,7 +824,7 @@ class PointerModelType(BaseNovaEnum):
 
     USBTABLET = "usbtablet"
 
-    ALL = (USBTABLET)
+    ALL = (USBTABLET,)
 
 
 class NotificationPriority(BaseNovaEnum):
@@ -1060,7 +1091,7 @@ class NetworkModel(FieldType):
     def coerce(obj, attr, value):
         if isinstance(value, network_model.NetworkInfo):
             return value
-        elif isinstance(value, six.string_types):
+        elif isinstance(value, str):
             # Hmm, do we need this?
             return network_model.NetworkInfo.hydrate(value)
         else:
@@ -1090,7 +1121,7 @@ class NetworkVIFModel(FieldType):
     def coerce(obj, attr, value):
         if isinstance(value, network_model.VIF):
             return value
-        elif isinstance(value, six.string_types):
+        elif isinstance(value, str):
             return NetworkVIFModel.from_primitive(obj, attr, value)
         else:
             raise ValueError(_('A nova.network.model.VIF object is required '
@@ -1184,6 +1215,10 @@ class BlockDeviceTypeField(BaseEnumField):
     AUTO_TYPE = BlockDeviceType()
 
 
+class BlockDeviceEncryptionFormatTypeField(BaseEnumField):
+    AUTO_TYPE = BlockDeviceEncryptionFormatType()
+
+
 class ConfigDrivePolicyField(BaseEnumField):
     AUTO_TYPE = ConfigDrivePolicy()
 
@@ -1236,6 +1271,10 @@ class ImageSignatureKeyTypeField(BaseEnumField):
     AUTO_TYPE = ImageSignatureKeyType()
 
 
+class InputBusField(BaseEnumField):
+    AUTO_TYPE = InputBus()
+
+
 class MigrationTypeField(BaseEnumField):
     AUTO_TYPE = MigrationType()
 
@@ -1272,6 +1311,10 @@ class VIFModelField(BaseEnumField):
     AUTO_TYPE = VIFModel()
 
 
+class VIOMMUModelField(BaseEnumField):
+    AUTO_TYPE = VIOMMUModel()
+
+
 class VMModeField(BaseEnumField):
     AUTO_TYPE = VMMode()
 
@@ -1336,6 +1379,14 @@ class InstancePowerStateField(BaseEnumField):
     AUTO_TYPE = InstancePowerState()
 
 
+class NetworkModelField(AutoTypedField):
+    AUTO_TYPE = NetworkModel()
+
+
+class NetworkVIFModelField(AutoTypedField):
+    AUTO_TYPE = NetworkVIFModel()
+
+
 class ListOfListsOfStringsField(AutoTypedField):
     AUTO_TYPE = List(List(fields.String()))
 
