@@ -14,30 +14,33 @@
 
 from oslo_db import exception as db_exc
 from oslo_db.sqlalchemy import utils as sqlalchemyutils
-from oslo_log import log as logging
-from sqlalchemy import or_
-from sqlalchemy.orm import joinedload
-from sqlalchemy.sql.expression import asc
-from sqlalchemy.sql import func
-from sqlalchemy.sql import text
-from sqlalchemy.sql import true
+from oslo_utils import versionutils
+import sqlalchemy as sa
+from sqlalchemy import orm
+from sqlalchemy import sql
+from sqlalchemy.sql import expression
 
-from nova import db
-from nova.db.sqlalchemy import api as db_api
-from nova.db.sqlalchemy.api import require_context
-from nova.db.sqlalchemy import api_models
-from nova.db.sqlalchemy import models as main_models
+import nova.conf
+from nova.db.api import api as api_db_api
+from nova.db.api import models as api_models
+from nova.db import utils as db_utils
 from nova import exception
-from nova.i18n import _LW
+from nova.notifications.objects import base as notification
+from nova.notifications.objects import flavor as flavor_notification
 from nova import objects
 from nova.objects import base
 from nova.objects import fields
 
 
-LOG = logging.getLogger(__name__)
 OPTIONAL_FIELDS = ['extra_specs', 'projects']
+# Remove these fields in version 2.0 of the object.
 DEPRECATED_FIELDS = ['deleted', 'deleted_at']
 
+# Non-joined fields which can be updated.
+MUTABLE_FIELDS = set(['description'])
+
+CONF = nova.conf.CONF
+
 
 def _dict_with_extra_specs(flavor_model):
     extra_specs = {x['key']: x['value']
@@ -49,18 +52,19 @@ def _dict_with_extra_specs(flavor_model):
 # decorators with static methods. We pull these out for now and can
 # move them back into the actual staticmethods on the object when those
 # issues are resolved.
-@db_api.api_context_manager.reader
+@api_db_api.context_manager.reader
 def _get_projects_from_db(context, flavorid):
-    db_flavor = context.session.query(api_models.Flavors).\
-                filter_by(flavorid=flavorid).\
-                options(joinedload('projects')).\
-                first()
+    db_flavor = context.session.query(api_models.Flavors).filter_by(
+        flavorid=flavorid
+    ).options(
+        orm.joinedload(api_models.Flavors.projects)
+    ).first()
     if not db_flavor:
         raise exception.FlavorNotFound(flavor_id=flavorid)
     return [x['project_id'] for x in db_flavor['projects']]
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _flavor_add_project(context, flavor_id, project_id):
     project = api_models.FlavorProjects()
     project.update({'flavor_id': flavor_id,
@@ -72,7 +76,7 @@ def _flavor_add_project(context, flavor_id, project_id):
                                            project_id=project_id)
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _flavor_del_project(context, flavor_id, project_id):
     result = context.session.query(api_models.FlavorProjects).\
              filter_by(project_id=project_id).\
@@ -83,9 +87,9 @@ def _flavor_del_project(context, flavor_id, project_id):
                                              project_id=project_id)
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _flavor_extra_specs_add(context, flavor_id, specs, max_retries=10):
-    writer = db_api.api_context_manager.writer
+    writer = api_db_api.context_manager.writer
     for attempt in range(max_retries):
         try:
             spec_refs = context.session.query(
@@ -120,7 +124,7 @@ def _flavor_extra_specs_add(context, flavor_id, specs, max_retries=10):
                     id=flavor_id, retries=max_retries)
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _flavor_extra_specs_del(context, flavor_id, key):
     result = context.session.query(api_models.FlavorExtraSpecs).\
              filter_by(flavor_id=flavor_id).\
@@ -131,7 +135,7 @@ def _flavor_extra_specs_del(context, flavor_id, key):
             extra_specs_key=key, flavor_id=flavor_id)
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _flavor_create(context, values):
     specs = values.get('extra_specs')
     db_specs = []
@@ -167,7 +171,7 @@ def _flavor_create(context, values):
     return _dict_with_extra_specs(db_flavor)
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _flavor_destroy(context, flavor_id=None, flavorid=None):
     query = context.session.query(api_models.Flavors)
 
@@ -185,26 +189,20 @@ def _flavor_destroy(context, flavor_id=None, flavorid=None):
     context.session.query(api_models.FlavorExtraSpecs).\
         filter_by(flavor_id=result.id).delete()
     context.session.delete(result)
-
-
-@db_api.main_context_manager.reader
-def _ensure_migrated(context):
-    result = context.session.query(main_models.InstanceTypes).\
-             filter_by(deleted=0).count()
-    if result:
-        LOG.warning(_LW('Main database contains %(count)i unmigrated flavors'),
-                    {'count': result})
-    return result == 0
+    return result
 
 
 # TODO(berrange): Remove NovaObjectDictCompat
+# TODO(mriedem): Remove NovaPersistentObject in version 2.0
 @base.NovaObjectRegistry.register
 class Flavor(base.NovaPersistentObject, base.NovaObject,
              base.NovaObjectDictCompat):
     # Version 1.0: Initial version
     # Version 1.1: Added save_projects(), save_extra_specs(), removed
-    #              remoteable from save()
-    VERSION = '1.1'
+    #              remotable from save()
+    # Version 1.2: Added description field. Note: this field should not be
+    #              persisted with the embedded instance.flavor.
+    VERSION = '1.2'
 
     fields = {
         'id': fields.IntegerField(),
@@ -221,32 +219,19 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
         'is_public': fields.BooleanField(),
         'extra_specs': fields.DictOfStringsField(),
         'projects': fields.ListOfStringsField(),
+        'description': fields.StringField(nullable=True)
         }
 
     def __init__(self, *args, **kwargs):
         super(Flavor, self).__init__(*args, **kwargs)
         self._orig_extra_specs = {}
         self._orig_projects = []
-        self._in_api = False
 
-    @property
-    def in_api(self):
-        if self._in_api:
-            return True
-        else:
-            try:
-                if 'id' in self:
-                    self._flavor_get_from_db(self._context, self.id)
-                else:
-                    flavor = self._flavor_get_by_flavor_id_from_db(
-                        self._context,
-                        self.flavorid)
-                    # Fix us up so we can use our real id
-                    self.id = flavor['id']
-                self._in_api = True
-            except exception.FlavorNotFound:
-                pass
-            return self._in_api
+    def obj_make_compatible(self, primitive, target_version):
+        super(Flavor, self).obj_make_compatible(primitive, target_version)
+        target_version = versionutils.convert_version_to_tuple(target_version)
+        if target_version < (1, 2) and 'description' in primitive:
+            del primitive['description']
 
     @staticmethod
     def _from_db_object(context, flavor, db_flavor, expected_attrs=None):
@@ -285,20 +270,23 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
         return flavor
 
     @staticmethod
-    @db_api.api_context_manager.reader
     def _flavor_get_query_from_db(context):
-        query = context.session.query(api_models.Flavors).\
-                options(joinedload('extra_specs'))
+        # We don't use a database context decorator on this method because this
+        # method is not executing a query, it's only building one.
+        query = context.session.query(api_models.Flavors).options(
+            orm.joinedload(api_models.Flavors.extra_specs)
+        )
         if not context.is_admin:
-            the_filter = [api_models.Flavors.is_public == true()]
+            the_filter = [api_models.Flavors.is_public == sql.true()]
             the_filter.extend([
                 api_models.Flavors.projects.any(project_id=context.project_id)
             ])
-            query = query.filter(or_(*the_filter))
+            query = query.filter(sa.or_(*the_filter))
         return query
 
     @staticmethod
-    @require_context
+    @db_utils.require_context
+    @api_db_api.context_manager.reader
     def _flavor_get_from_db(context, id):
         """Returns a dict describing specific flavor."""
         result = Flavor._flavor_get_query_from_db(context).\
@@ -306,10 +294,11 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
                         first()
         if not result:
             raise exception.FlavorNotFound(flavor_id=id)
-        return db_api._dict_with_extra_specs(result)
+        return _dict_with_extra_specs(result)
 
     @staticmethod
-    @require_context
+    @db_utils.require_context
+    @api_db_api.context_manager.reader
     def _flavor_get_by_name_from_db(context, name):
         """Returns a dict describing specific flavor."""
         result = Flavor._flavor_get_query_from_db(context).\
@@ -317,19 +306,20 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
                             first()
         if not result:
             raise exception.FlavorNotFoundByName(flavor_name=name)
-        return db_api._dict_with_extra_specs(result)
+        return _dict_with_extra_specs(result)
 
     @staticmethod
-    @require_context
+    @db_utils.require_context
+    @api_db_api.context_manager.reader
     def _flavor_get_by_flavor_id_from_db(context, flavor_id):
         """Returns a dict describing specific flavor_id."""
         result = Flavor._flavor_get_query_from_db(context).\
                         filter_by(flavorid=flavor_id).\
-                        order_by(asc(api_models.Flavors.id)).\
+                        order_by(expression.asc(api_models.Flavors.id)).\
                         first()
         if not result:
             raise exception.FlavorNotFound(flavor_id=flavor_id)
-        return db_api._dict_with_extra_specs(result)
+        return _dict_with_extra_specs(result)
 
     @staticmethod
     def _get_projects_from_db(context, flavorid):
@@ -337,13 +327,8 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
 
     @base.remotable
     def _load_projects(self):
-        try:
-            self.projects = self._get_projects_from_db(self._context,
-                                                       self.flavorid)
-        except exception.FlavorNotFound:
-            self.projects = [x['project_id'] for x in
-                             db.flavor_access_get_by_flavor_id(self._context,
-                                                               self.flavorid)]
+        self.projects = self._get_projects_from_db(self._context,
+                                                   self.flavorid)
         self.obj_reset_changes(['projects'])
 
     def obj_load_attr(self, attrname):
@@ -394,30 +379,20 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
 
     @base.remotable_classmethod
     def get_by_id(cls, context, id):
-        try:
-            db_flavor = cls._flavor_get_from_db(context, id)
-        except exception.FlavorNotFound:
-            db_flavor = db.flavor_get(context, id)
+        db_flavor = cls._flavor_get_from_db(context, id)
         return cls._from_db_object(context, cls(context), db_flavor,
                                    expected_attrs=['extra_specs'])
 
     @base.remotable_classmethod
     def get_by_name(cls, context, name):
-        try:
-            db_flavor = cls._flavor_get_by_name_from_db(context, name)
-        except exception.FlavorNotFoundByName:
-            db_flavor = db.flavor_get_by_name(context, name)
+        db_flavor = cls._flavor_get_by_name_from_db(context, name)
         return cls._from_db_object(context, cls(context), db_flavor,
                                    expected_attrs=['extra_specs'])
 
     @base.remotable_classmethod
     def get_by_flavor_id(cls, context, flavor_id, read_deleted=None):
-        try:
-            db_flavor = cls._flavor_get_by_flavor_id_from_db(context,
-                                                             flavor_id)
-        except exception.FlavorNotFound:
-            db_flavor = db.flavor_get_by_flavor_id(context, flavor_id,
-                                                   read_deleted)
+        db_flavor = cls._flavor_get_by_flavor_id_from_db(context,
+                                                         flavor_id)
         return cls._from_db_object(context, cls(context), db_flavor,
                                    expected_attrs=['extra_specs'])
 
@@ -430,10 +405,7 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
         return _flavor_del_project(context, flavor_id, project_id)
 
     def _add_access(self, project_id):
-        if self.in_api:
-            self._flavor_add_project(self._context, self.id, project_id)
-        else:
-            db.flavor_access_add(self._context, self.flavorid, project_id)
+        self._flavor_add_project(self._context, self.id, project_id)
 
     @base.remotable
     def add_access(self, project_id):
@@ -442,12 +414,10 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
                                               reason='projects modified')
         self._add_access(project_id)
         self._load_projects()
+        self._send_notification(fields.NotificationAction.UPDATE)
 
     def _remove_access(self, project_id):
-        if self.in_api:
-            self._flavor_del_project(self._context, self.id, project_id)
-        else:
-            db.flavor_access_remove(self._context, self.flavorid, project_id)
+        self._flavor_del_project(self._context, self.id, project_id)
 
     @base.remotable
     def remove_access(self, project_id):
@@ -456,29 +426,18 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
                                               reason='projects modified')
         self._remove_access(project_id)
         self._load_projects()
+        self._send_notification(fields.NotificationAction.UPDATE)
 
     @staticmethod
     def _flavor_create(context, updates):
         return _flavor_create(context, updates)
 
-    @staticmethod
-    def _ensure_migrated(context):
-        return _ensure_migrated(context)
-
     @base.remotable
     def create(self):
         if self.obj_attr_is_set('id'):
             raise exception.ObjectActionError(action='create',
                                               reason='already created')
 
-        # NOTE(danms): Once we have made it past a point where we know
-        # all flavors have been migrated, we can remove this. Ideally
-        # in Ocata with a blocker migration to be sure.
-        if not self._ensure_migrated(self._context):
-            raise exception.ObjectActionError(
-                action='create',
-                reason='main database still contains flavors')
-
         updates = self.obj_get_changes()
         expected_attrs = []
         for attr in OPTIONAL_FIELDS:
@@ -487,6 +446,7 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
         db_flavor = self._flavor_create(self._context, updates)
         self._from_db_object(self._context, self, db_flavor,
                              expected_attrs=expected_attrs)
+        self._send_notification(fields.NotificationAction.CREATE)
 
     @base.remotable
     def save_projects(self, to_add=None, to_delete=None):
@@ -520,33 +480,40 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
         :param:to_add: A dict of new keys to add/update
         :param:to_delete: A list of keys to remove
         """
-
-        if self.in_api:
-            add_fn = self._flavor_extra_specs_add
-            del_fn = self._flavor_extra_specs_del
-            ident = self.id
-        else:
-            add_fn = db.flavor_extra_specs_update_or_create
-            del_fn = db.flavor_extra_specs_delete
-            ident = self.flavorid
-
         to_add = to_add if to_add is not None else {}
         to_delete = to_delete if to_delete is not None else []
 
         if to_add:
-            add_fn(self._context, ident, to_add)
+            self._flavor_extra_specs_add(self._context, self.id, to_add)
 
         for key in to_delete:
-            del_fn(self._context, ident, key)
+            self._flavor_extra_specs_del(self._context, self.id, key)
         self.obj_reset_changes(['extra_specs'])
 
+    # NOTE(mriedem): This method is not remotable since we only expect the API
+    # to be able to make updates to a flavor.
+    @api_db_api.context_manager.writer
+    def _save(self, context, values):
+        db_flavor = context.session.query(api_models.Flavors).\
+            filter_by(id=self.id).first()
+        if not db_flavor:
+            raise exception.FlavorNotFound(flavor_id=self.id)
+        db_flavor.update(values)
+        db_flavor.save(context.session)
+        # Refresh ourselves from the DB object so we get the new updated_at.
+        self._from_db_object(context, self, db_flavor)
+        self.obj_reset_changes()
+
     def save(self):
         updates = self.obj_get_changes()
         projects = updates.pop('projects', None)
         extra_specs = updates.pop('extra_specs', None)
         if updates:
-            raise exception.ObjectActionError(
-                action='save', reason='read-only fields were changed')
+            # Only allowed to update from the whitelist of mutable fields.
+            if set(updates.keys()) - MUTABLE_FIELDS:
+                raise exception.ObjectActionError(
+                    action='save', reason='read-only fields were changed')
+            self._save(self._context, updates)
 
         if extra_specs is not None:
             deleted_keys = (set(self._orig_extra_specs.keys()) -
@@ -572,6 +539,10 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
         if added_projects or deleted_projects:
             self.save_projects(added_projects, deleted_projects)
 
+        if (added_keys or deleted_keys or added_projects or deleted_projects or
+                updates):
+            self._send_notification(fields.NotificationAction.UPDATE)
+
     @staticmethod
     def _flavor_destroy(context, flavor_id=None, flavorid=None):
         return _flavor_destroy(context, flavor_id=flavor_id, flavorid=flavorid)
@@ -584,16 +555,40 @@ class Flavor(base.NovaPersistentObject, base.NovaObject,
         # delete request with only our name filled out. However, if we have
         # our id property, we should instead delete with that since it's
         # far more specific.
-        try:
-            if 'id' in self:
-                self._flavor_destroy(self._context, flavor_id=self.id)
-            else:
-                self._flavor_destroy(self._context, flavorid=self.flavorid)
-        except exception.FlavorNotFound:
-            db.flavor_destroy(self._context, self.flavorid)
+        if 'id' in self:
+            db_flavor = self._flavor_destroy(self._context,
+                                             flavor_id=self.id)
+        else:
+            db_flavor = self._flavor_destroy(self._context,
+                                             flavorid=self.flavorid)
+        self._from_db_object(self._context, self, db_flavor)
+        self._send_notification(fields.NotificationAction.DELETE)
+
+    def _send_notification(self, action):
+        # NOTE(danms): Instead of making the below notification
+        # lazy-load projects (which is a problem for instance-bound
+        # flavors and compute-cell operations), just load them here.
+        if 'projects' not in self:
+            # If the flavor is deleted we can't lazy-load projects.
+            # FlavorPayload will orphan the flavor which will make the
+            # NotificationPayloadBase set projects=None in the notification
+            # payload.
+            if action != fields.NotificationAction.DELETE:
+                self._load_projects()
+        notification_type = flavor_notification.FlavorNotification
+        payload_type = flavor_notification.FlavorPayload
+
+        payload = payload_type(self)
+        notification_type(
+            publisher=notification.NotificationPublisher(
+                host=CONF.host, source=fields.NotificationSource.API),
+            event_type=notification.EventType(object="flavor",
+                                              action=action),
+            priority=fields.NotificationPriority.INFO,
+            payload=payload).emit(self._context)
 
 
-@db_api.api_context_manager.reader
+@api_db_api.context_manager.reader
 def _flavor_get_all_from_db(context, inactive, filters, sort_key, sort_dir,
                             limit, marker):
     """Returns all flavors.
@@ -620,7 +615,7 @@ def _flavor_get_all_from_db(context, inactive, filters, sort_key, sort_dir,
             the_filter.extend([api_models.Flavors.projects.any(
                 project_id=context.project_id)])
         if len(the_filter) > 1:
-            query = query.filter(or_(*the_filter))
+            query = query.filter(sa.or_(*the_filter))
         else:
             query = query.filter(the_filter[0])
     marker_row = None
@@ -650,103 +645,13 @@ class FlavorList(base.ObjectListBase, base.NovaObject):
     @base.remotable_classmethod
     def get_all(cls, context, inactive=False, filters=None,
                 sort_key='flavorid', sort_dir='asc', limit=None, marker=None):
-        try:
-            api_db_flavors = _flavor_get_all_from_db(context,
-                                                     inactive=inactive,
-                                                     filters=filters,
-                                                     sort_key=sort_key,
-                                                     sort_dir=sort_dir,
-                                                     limit=limit,
-                                                     marker=marker)
-            # NOTE(danms): If we were asked for a marker and found it in
-            # results from the API DB, we must continue our pagination with
-            # just the limit (if any) to the main DB.
-            marker = None
-        except exception.MarkerNotFound:
-            api_db_flavors = []
-
-        if limit is not None:
-            limit_more = limit - len(api_db_flavors)
-        else:
-            limit_more = None
-
-        if limit_more is None or limit_more > 0:
-            db_flavors = db.flavor_get_all(context, inactive=inactive,
-                                           filters=filters, sort_key=sort_key,
-                                           sort_dir=sort_dir, limit=limit_more,
-                                           marker=marker)
-        else:
-            db_flavors = []
+        api_db_flavors = _flavor_get_all_from_db(context,
+                                                 inactive=inactive,
+                                                 filters=filters,
+                                                 sort_key=sort_key,
+                                                 sort_dir=sort_dir,
+                                                 limit=limit,
+                                                 marker=marker)
         return base.obj_make_list(context, cls(context), objects.Flavor,
-                                  api_db_flavors + db_flavors,
+                                  api_db_flavors,
                                   expected_attrs=['extra_specs'])
-
-
-@db_api.main_context_manager.reader
-def _get_main_db_flavor_ids(context, limit):
-    # NOTE(danms): We don't need this imported at runtime, so
-    # keep it separate here
-    from nova.db.sqlalchemy import models
-    return [x[0] for x in context.session.query(models.InstanceTypes.id).
-            filter_by(deleted=0).
-            limit(limit)]
-
-
-@db_api.main_context_manager.writer
-def _destroy_flavor_hard(context, name):
-    # NOTE(danms): We don't need this imported at runtime, so
-    # keep it separate here
-    from nova.db.sqlalchemy import models
-    context.session.query(models.InstanceTypes).filter_by(name=name).delete()
-
-
-def migrate_flavors(ctxt, count, hard_delete=False):
-    main_db_ids = _get_main_db_flavor_ids(ctxt, count)
-    if not main_db_ids:
-        return 0, 0
-
-    count_all = len(main_db_ids)
-    count_hit = 0
-
-    for flavor_id in main_db_ids:
-        try:
-            flavor = Flavor.get_by_id(ctxt, flavor_id)
-            flavor_values = {field: getattr(flavor, field)
-                             for field in flavor.fields}
-            flavor._flavor_create(ctxt, flavor_values)
-            count_hit += 1
-            if hard_delete:
-                _destroy_flavor_hard(ctxt, flavor.name)
-            else:
-                db.flavor_destroy(ctxt, flavor.flavorid)
-        except exception.FlavorNotFound:
-            LOG.warning(_LW('Flavor id %(id)i disappeared during migration'),
-                        {'id': flavor_id})
-        except (exception.FlavorExists, exception.FlavorIdExists) as e:
-            LOG.error(str(e))
-
-    return count_all, count_hit
-
-
-def _adjust_autoincrement(context, value):
-    engine = db_api.get_api_engine()
-    if engine.name == 'postgresql':
-        # NOTE(danms): If we migrated some flavors in the above function,
-        # then we will have confused postgres' sequence for the autoincrement
-        # primary key. MySQL does not care about this, but since postgres does,
-        # we need to reset this to avoid a failure on the next flavor creation.
-        engine.execute(
-            text('ALTER SEQUENCE flavors_id_seq RESTART WITH %i;' % (
-                value)))
-
-
-@db_api.api_context_manager.reader
-def _get_max_flavor_id(context):
-    max_id = context.session.query(func.max(api_models.Flavors.id)).one()[0]
-    return max_id or 0
-
-
-def migrate_flavor_reset_autoincrement(ctxt, count):
-    max_id = _get_max_flavor_id(ctxt)
-    _adjust_autoincrement(ctxt, max_id + 1)
-    return 0, 0
