@@ -18,7 +18,7 @@ from oslo_utils import versionutils
 
 from nova import availability_zones
 from nova import context as nova_context
-from nova.db import api as db
+from nova.db.main import api as db
 from nova import exception
 from nova.notifications.objects import base as notification
 from nova.notifications.objects import service as service_notification
@@ -31,12 +31,12 @@ LOG = logging.getLogger(__name__)
 
 
 # NOTE(danms): This is the global service version counter
-SERVICE_VERSION = 53
+SERVICE_VERSION = 64
 
 
 # NOTE(danms): This is our SERVICE_VERSION history. The idea is that any
 # time we bump the version, we will put an entry here to record the change,
-# along with any pertinent data. For things that we can programatically
+# along with any pertinent data. For things that we can programmatically
 # detect that need a bump, we put something in _collect_things() below to
 # assemble a dict of things we can check. For example, we pretty much always
 # want to consider the compute RPC API version a thing that requires a service
@@ -190,13 +190,52 @@ SERVICE_VERSION_HISTORY = (
     # Version 53: Compute RPC v5.12:
     # Add accel_uuids (accelerator requests) param to rebuild_instance
     {'compute_rpc': '5.12'},
+    # Version 54: Compute RPC v5.13:
+    # Add accel_uuids (accelerator requests) param to shelve_instance and
+    # shelve_offload_instance and unshelve_instance
+    {'compute_rpc': '5.13'},
+    # Version 55: Compute RPC v5.13:
+    # Add support for qos interface attach
+    {'compute_rpc': '5.13'},
+    # Version 56: Compute RPC v6.0:
+    {'compute_rpc': '6.0'},
+    # Version 57: Compute RPC v6.0:
+    # Add support for vnic 'accelerator-direct'.
+    {'compute_rpc': '6.0'},
+    # Version 58: Compute RPC v6.0:
+    # Add support for booting with neutron extended resource request
+    {'compute_rpc': '6.0'},
+    # Version 59: Compute RPC v6.0:
+    # Add support for server move operations with neutron extended resource
+    # request
+    {'compute_rpc': '6.0'},
+    # Version 60: Compute RPC v6.0:
+    # Add support for interface attach operation with neutron extended resource
+    # request
+    {'compute_rpc': '6.0'},
+    # Version 61: Compute RPC v6.0:
+    # Add support for remotely-managed ports (vnic-type 'remote-managed')
+    {'compute_rpc': '6.0'},
+    # Version 62: Compute RPC v6.0:
+    # Add support for VDPA port attach/detach
+    {'compute_rpc': '6.0'},
+    # Version 63: Compute RPC v6.0:
+    # Add support for VDPA hotplug live migration and suspend/resume
+    {'compute_rpc': '6.0'},
+    # Version 64: Compute RPC v6.1:
+    # Add reimage_boot_volume parameter to rebuild_instance()
+    {'compute_rpc': '6.1'},
 )
 
 # This is used to raise an error at service startup if older than N-1 computes
-# are detected. Update this at the beginning of every release cycle
-OLDEST_SUPPORTED_SERVICE_VERSION = 'Ussuri'
+# are detected. Update this at the beginning of every release cycle to point to
+# the smallest service version that was added in N-1.
+OLDEST_SUPPORTED_SERVICE_VERSION = 'Yoga'
 SERVICE_VERSION_ALIASES = {
-    'Ussuri': 41
+    'Victoria': 52,
+    'Wallaby': 54,
+    'Xena': 57,
+    'Yoga': 61,
 }
 
 
