@@ -15,11 +15,13 @@
 #    under the License.
 
 import collections
+import typing as ty
 
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 
+from nova import context as ctx
 from nova import exception
 from nova import objects
 from nova.objects import fields
@@ -29,6 +31,9 @@ from nova.pci import whitelist
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
+MappingType = ty.Dict[str, ty.List['objects.PciDevice']]
+PCIInvType = ty.DefaultDict[str, ty.List['objects.PciDevice']]
+
 
 class PciDevTracker(object):
     """Manage pci devices in a compute node.
@@ -51,31 +56,37 @@ class PciDevTracker(object):
     are saved.
     """
 
-    def __init__(self, context, node_id=None):
+    def __init__(
+        self,
+        context: ctx.RequestContext,
+        compute_node: 'objects.ComputeNode',
+    ):
         """Create a pci device tracker.
 
-        If a node_id is passed in, it will fetch pci devices information
-        from database, otherwise, it will create an empty devices list
-        and the resource tracker will update the node_id information later.
+        :param context: The request context.
+        :param compute_node: The object.ComputeNode whose PCI devices we're
+                             tracking.
         """
-
-        super(PciDevTracker, self).__init__()
-        self.stale = {}
-        self.node_id = node_id
-        self.dev_filter = whitelist.Whitelist(CONF.pci.passthrough_whitelist)
-        self.stats = stats.PciDeviceStats(dev_filter=self.dev_filter)
+        self.stale: ty.Dict[str, objects.PciDevice] = {}
+        self.node_id: str = compute_node.id
+        self.dev_filter = whitelist.Whitelist(CONF.pci.device_spec)
+        numa_topology = compute_node.numa_topology
+        if numa_topology:
+            # For legacy reasons, the NUMATopology is stored as a JSON blob.
+            # Deserialize it into a real object.
+            numa_topology = objects.NUMATopology.obj_from_db_obj(numa_topology)
+        self.stats = stats.PciDeviceStats(
+            numa_topology, dev_filter=self.dev_filter)
         self._context = context
-        if node_id:
-            self.pci_devs = objects.PciDeviceList.get_by_compute_node(
-                    context, node_id)
-        else:
-            self.pci_devs = objects.PciDeviceList(objects=[])
+        self.pci_devs = objects.PciDeviceList.get_by_compute_node(
+            context, self.node_id)
         self._build_device_tree(self.pci_devs)
         self._initial_instance_usage()
 
-    def _initial_instance_usage(self):
-        self.allocations = collections.defaultdict(list)
-        self.claims = collections.defaultdict(list)
+    def _initial_instance_usage(self) -> None:
+        self.allocations: PCIInvType = collections.defaultdict(list)
+        self.claims: PCIInvType = collections.defaultdict(list)
+
         for dev in self.pci_devs:
             uuid = dev.instance_uuid
             if dev.status == fields.PciDeviceStatus.CLAIMED:
@@ -85,7 +96,7 @@ class PciDevTracker(object):
             elif dev.status == fields.PciDeviceStatus.AVAILABLE:
                 self.stats.add_device(dev)
 
-    def save(self, context):
+    def save(self, context: ctx.RequestContext) -> None:
         for dev in self.pci_devs:
             if dev.obj_what_changed():
                 with dev.obj_alternate_context(context):
@@ -94,10 +105,12 @@ class PciDevTracker(object):
                         self.pci_devs.objects.remove(dev)
 
     @property
-    def pci_stats(self):
+    def pci_stats(self) -> stats.PciDeviceStats:
         return self.stats
 
-    def update_devices_from_hypervisor_resources(self, devices_json):
+    def update_devices_from_hypervisor_resources(
+        self, devices_json: str,
+    ) -> None:
         """Sync the pci device tracker with hypervisor information.
 
         To support pci device hot plug, we sync with the hypervisor
@@ -120,7 +133,7 @@ class PciDevTracker(object):
             try:
                 if self.dev_filter.device_assignable(dev):
                     devices.append(dev)
-            except exception.PciConfigInvalidWhitelist as e:
+            except exception.PciConfigInvalidSpec as e:
                 # The raised exception is misleading as the problem is not with
                 # the whitelist config but with the host PCI device reported by
                 # libvirt. The code that matches the host PCI device to the
@@ -151,12 +164,12 @@ class PciDevTracker(object):
                     # parse whitelist config with
                     # devspec.PciAddressSpec._set_pci_dev_info()
                     str(e).replace(
-                        'Invalid PCI devices Whitelist config:', 'The'))
+                        'Invalid [pci]device_spec config:', 'The'))
 
         self._set_hvdevs(devices)
 
     @staticmethod
-    def _build_device_tree(all_devs):
+    def _build_device_tree(all_devs: ty.List['objects.PciDevice']) -> None:
         """Build a tree of devices that represents parent-child relationships.
 
         We need to have the relationships set up so that we can easily make
@@ -186,12 +199,14 @@ class PciDevTracker(object):
             if dev.dev_type == fields.PciDeviceType.SRIOV_PF:
                 dev.child_devices = []
                 parents[dev.address] = dev
-            elif dev.dev_type == fields.PciDeviceType.SRIOV_VF:
+            elif dev.dev_type in (
+                fields.PciDeviceType.SRIOV_VF, fields.PciDeviceType.VDPA
+            ):
                 dev.parent_device = parents.get(dev.parent_addr)
                 if dev.parent_device:
                     parents[dev.parent_addr].child_devices.append(dev)
 
-    def _set_hvdevs(self, devices):
+    def _set_hvdevs(self, devices: ty.List[ty.Dict[str, ty.Any]]) -> None:
         exist_addrs = set([dev.address for dev in self.pci_devs])
         new_addrs = set([dev['address'] for dev in devices])
 
@@ -202,11 +217,14 @@ class PciDevTracker(object):
                 # from the pci whitelist.
                 try:
                     existed.remove()
-                except exception.PciDeviceInvalidStatus as e:
-                    LOG.warning("Unable to remove device with %(status)s "
-                                "ownership %(instance_uuid)s because of "
-                                "%(pci_exception)s. "
-                                "Check your [pci]passthrough_whitelist "
+                except (
+                        exception.PciDeviceInvalidStatus,
+                        exception.PciDeviceInvalidOwner,
+                ) as e:
+                    LOG.warning("Unable to remove device with status "
+                                "'%(status)s' and ownership %(instance_uuid)s "
+                                "because of %(pci_exception)s. "
+                                "Check your [pci]device_spec "
                                 "configuration to make sure this allocated "
                                 "device is whitelisted. If you have removed "
                                 "the device from the whitelist intentionally "
@@ -235,9 +253,13 @@ class PciDevTracker(object):
                 else:
                     # Note(yjiang5): no need to update stats if an assigned
                     # device is hot removed.
-                    self.stats.remove_device(existed)
+                    # NOTE(gibi): only remove the device from the pools if it
+                    # is not already removed
+                    if existed in self.stats.get_free_devs():
+                        self.stats.remove_device(existed)
             else:
                 # Update tracked devices.
+                new_value: ty.Dict[str, ty.Any]
                 new_value = next((dev for dev in devices if
                     dev['address'] == existed.address))
                 new_value['compute_node_id'] = self.node_id
@@ -271,7 +293,12 @@ class PciDevTracker(object):
 
         self._build_device_tree(self.pci_devs)
 
-    def _claim_instance(self, context, pci_requests, instance_numa_topology):
+    def _claim_instance(
+        self,
+        context: ctx.RequestContext,
+        pci_requests: 'objects.InstancePCIRequests',
+        instance_numa_topology: 'objects.InstanceNUMATopology',
+    ) -> ty.List['objects.PciDevice']:
         instance_cells = None
         if instance_numa_topology:
             instance_cells = instance_numa_topology.cells
@@ -279,7 +306,7 @@ class PciDevTracker(object):
         devs = self.stats.consume_requests(pci_requests.requests,
                                            instance_cells)
         if not devs:
-            return None
+            return []
 
         instance_uuid = pci_requests.instance_uuid
         for dev in devs:
@@ -291,18 +318,15 @@ class PciDevTracker(object):
                         {'instance': instance_uuid})
         return devs
 
-    def _allocate_instance(self, instance, devs):
-        for dev in devs:
-            dev.allocate(instance)
+    def claim_instance(
+        self,
+        context: ctx.RequestContext,
+        pci_requests: 'objects.InstancePCIRequests',
+        instance_numa_topology: 'objects.InstanceNUMATopology',
+    ) -> ty.List['objects.PciDevice']:
 
-    def allocate_instance(self, instance):
-        devs = self.claims.pop(instance['uuid'], [])
-        self._allocate_instance(instance, devs)
-        if devs:
-            self.allocations[instance['uuid']] += devs
-
-    def claim_instance(self, context, pci_requests, instance_numa_topology):
         devs = []
+
         if self.pci_devs and pci_requests.requests:
             instance_uuid = pci_requests.instance_uuid
             devs = self._claim_instance(context, pci_requests,
@@ -311,7 +335,21 @@ class PciDevTracker(object):
                 self.claims[instance_uuid] = devs
         return devs
 
-    def free_device(self, dev, instance):
+    def _allocate_instance(
+        self, instance: 'objects.Instance', devs: ty.List['objects.PciDevice'],
+    ) -> None:
+        for dev in devs:
+            dev.allocate(instance)
+
+    def allocate_instance(self, instance: 'objects.Instance') -> None:
+        devs = self.claims.pop(instance['uuid'], [])
+        self._allocate_instance(instance, devs)
+        if devs:
+            self.allocations[instance['uuid']] += devs
+
+    def free_device(
+        self, dev: 'objects.PciDevice', instance: 'objects.Instance'
+    ) -> None:
         """Free device from pci resource tracker
 
         :param dev: cloned pci device object that needs to be free
@@ -330,7 +368,11 @@ class PciDevTracker(object):
                 break
 
     def _remove_device_from_pci_mapping(
-            self, instance_uuid, pci_device, pci_mapping):
+        self,
+        instance_uuid: str,
+        pci_device: 'objects.PciDevice',
+        pci_mapping: MappingType,
+    ) -> None:
         """Remove a PCI device from allocations or claims.
 
         If there are no more PCI devices, pop the uuid.
@@ -341,7 +383,9 @@ class PciDevTracker(object):
             if len(pci_devices) == 0:
                 pci_mapping.pop(instance_uuid, None)
 
-    def _free_device(self, dev, instance=None):
+    def _free_device(
+        self, dev: 'objects.PciDevice', instance: 'objects.Instance' = None,
+    ) -> None:
         freed_devs = dev.free(instance)
         stale = self.stale.pop(dev.address, None)
         if stale:
@@ -349,31 +393,41 @@ class PciDevTracker(object):
         for dev in freed_devs:
             self.stats.add_device(dev)
 
-    def free_instance_allocations(self, context, instance):
+    def free_instance_allocations(
+        self, context: ctx.RequestContext, instance: 'objects.Instance',
+    ) -> None:
         """Free devices that are in ALLOCATED state for instance.
 
-        :param context: user request context (nova.context.RequestContext)
+        :param context: user request context
         :param instance: instance object
         """
-        if self.allocations.pop(instance['uuid'], None):
-            for dev in self.pci_devs:
-                if (dev.status == fields.PciDeviceStatus.ALLOCATED and
-                        dev.instance_uuid == instance['uuid']):
-                    self._free_device(dev)
+        if not self.allocations.pop(instance['uuid'], None):
+            return
 
-    def free_instance_claims(self, context, instance):
+        for dev in self.pci_devs:
+            if (dev.status == fields.PciDeviceStatus.ALLOCATED and
+                    dev.instance_uuid == instance['uuid']):
+                self._free_device(dev)
+
+    def free_instance_claims(
+        self, context: ctx.RequestContext, instance: 'objects.Instance',
+    ) -> None:
         """Free devices that are in CLAIMED state for instance.
 
         :param context: user request context (nova.context.RequestContext)
         :param instance: instance object
         """
-        if self.claims.pop(instance['uuid'], None):
-            for dev in self.pci_devs:
-                if (dev.status == fields.PciDeviceStatus.CLAIMED and
-                        dev.instance_uuid == instance['uuid']):
-                    self._free_device(dev)
+        if not self.claims.pop(instance['uuid'], None):
+            return
 
-    def free_instance(self, context, instance):
+        for dev in self.pci_devs:
+            if (dev.status == fields.PciDeviceStatus.CLAIMED and
+                    dev.instance_uuid == instance['uuid']):
+                self._free_device(dev)
+
+    def free_instance(
+        self, context: ctx.RequestContext, instance: 'objects.Instance',
+    ) -> None:
         """Free devices that are in CLAIMED or ALLOCATED state for instance.
 
         :param context: user request context (nova.context.RequestContext)
@@ -387,9 +441,13 @@ class PciDevTracker(object):
         self.free_instance_allocations(context, instance)
         self.free_instance_claims(context, instance)
 
-    def update_pci_for_instance(self, context, instance, sign):
-        """Update PCI usage information if devices are de/allocated.
-        """
+    def update_pci_for_instance(
+        self,
+        context: ctx.RequestContext,
+        instance: 'objects.Instance',
+        sign: int,
+    ) -> None:
+        """Update PCI usage information if devices are de/allocated."""
         if not self.pci_devs:
             return
 
@@ -398,14 +456,17 @@ class PciDevTracker(object):
         if sign == 1:
             self.allocate_instance(instance)
 
-    def clean_usage(self, instances, migrations, orphans):
+    def clean_usage(
+        self,
+        instances: 'objects.InstanceList',
+        migrations: 'objects.MigrationList',
+    ) -> None:
         """Remove all usages for instances not passed in the parameter.
 
         The caller should hold the COMPUTE_RESOURCE_SEMAPHORE lock
         """
-        existed = set(inst['uuid'] for inst in instances)
-        existed |= set(mig['instance_uuid'] for mig in migrations)
-        existed |= set(inst['uuid'] for inst in orphans)
+        existed = set(inst.uuid for inst in instances)
+        existed |= set(mig.instance_uuid for mig in migrations)
 
         # need to copy keys, because the dict is modified in the loop body
         for uuid in list(self.claims):
@@ -419,19 +480,3 @@ class PciDevTracker(object):
                 devs = self.allocations.pop(uuid, [])
                 for dev in devs:
                     self._free_device(dev)
-
-
-def get_instance_pci_devs(inst, request_id=None):
-    """Get the devices allocated to one or all requests for an instance.
-
-    - For generic PCI request, the request id is None.
-    - For sr-iov networking, the request id is a valid uuid
-    - There are a couple of cases where all the PCI devices allocated to an
-      instance need to be returned. Refer to libvirt driver that handles
-      soft_reboot and hard_boot of 'xen' instances.
-    """
-    pci_devices = inst.pci_devices
-    if pci_devices is None:
-        return []
-    return [device for device in pci_devices if
-                   device.request_id == request_id or request_id == 'all']
