@@ -22,15 +22,28 @@ POLICY_ROOT = 'os_compute_api:os-migrate-server:%s'
 
 
 migrate_server_policies = [
-    policy.RuleDefault(
+    policy.DocumentedRuleDefault(
         name=POLICY_ROOT % 'migrate',
-        check_str=base.RULE_ADMIN_API),
-    policy.RuleDefault(
-        name=POLICY_ROOT % 'discoverable',
-        check_str=base.RULE_ANY),
-    policy.RuleDefault(
+        check_str=base.ADMIN,
+        description="Cold migrate a server to a host",
+        operations=[
+            {
+                'method': 'POST',
+                'path': '/servers/{server_id}/action (migrate)'
+            }
+        ],
+        scope_types=['project']),
+    policy.DocumentedRuleDefault(
         name=POLICY_ROOT % 'migrate_live',
-        check_str=base.RULE_ADMIN_API),
+        check_str=base.ADMIN,
+        description="Live migrate a server to a new host without a reboot",
+        operations=[
+            {
+                'method': 'POST',
+                'path': '/servers/{server_id}/action (os-migrateLive)'
+            }
+        ],
+        scope_types=['project']),
 ]
 
 
