@@ -22,12 +22,17 @@ POLICY_ROOT = 'os_compute_api:os-migrations:%s'
 
 
 migrations_policies = [
-    policy.RuleDefault(
+    policy.DocumentedRuleDefault(
         name=POLICY_ROOT % 'index',
-        check_str=base.RULE_ADMIN_API),
-    policy.RuleDefault(
-        name=POLICY_ROOT % 'discoverable',
-        check_str=base.RULE_ANY),
+        check_str=base.ADMIN,
+        description="List migrations",
+        operations=[
+            {
+                'method': 'GET',
+                'path': '/os-migrations'
+            }
+        ],
+        scope_types=['project']),
 ]
 
 
