@@ -20,16 +20,10 @@ Manage hosts in the current zone.
 import collections
 import functools
 import time
-try:
-    from collections import UserDict as IterableUserDict   # Python 3
-except ImportError:
-    from UserDict import IterableUserDict                  # Python 2
-
 
 import iso8601
 from oslo_log import log as logging
 from oslo_utils import timeutils
-import six
 
 import nova.conf
 from nova import context as context_module
@@ -48,8 +42,9 @@ LOG = logging.getLogger(__name__)
 HOST_INSTANCE_SEMAPHORE = "host_instance"
 
 
-class ReadOnlyDict(IterableUserDict):
+class ReadOnlyDict(collections.UserDict):
     """A read-only dict."""
+
     def __init__(self, source=None):
         self.data = {}
         if source:
@@ -228,6 +223,7 @@ class HostState(object):
         self.numa_topology = objects.NUMATopology.obj_from_db_obj(
             compute.numa_topology) if compute.numa_topology else None
         self.pci_stats = pci_stats.PciDeviceStats(
+            self.numa_topology,
             stats=compute.pci_device_pools)
 
         # All virt drivers report host_ip
@@ -600,7 +596,7 @@ class HostManager(object):
                     return name_to_cls_map.values()
                 else:
                     return []
-            hosts = six.itervalues(name_to_cls_map)
+            hosts = name_to_cls_map.values()
 
         return self.filter_handler.get_filtered_objects(self.enabled_filters,
                 hosts, spec_obj, index)
@@ -610,16 +606,16 @@ class HostManager(object):
         return self.weight_handler.get_weighed_objects(self.weighers,
                 hosts, spec_obj)
 
-    def _get_computes_for_cells(self, context, cells, compute_uuids=None):
+    def _get_computes_for_cells(self, context, cells, compute_uuids):
         """Get a tuple of compute node and service information.
 
         :param context: request context
         :param cells: list of CellMapping objects
-        :param compute_uuids: list of ComputeNode UUIDs. If this is None, all
-            compute nodes from each specified cell will be returned, otherwise
-            only the ComputeNode objects with a UUID in the list of UUIDs in
-            any given cell is returned. If this is an empty list, the returned
-            compute_nodes tuple item will be an empty dict.
+        :param compute_uuids: Optional list of ComputeNode UUIDs. If this is
+            None, all compute nodes from each specified cell will be returned,
+            otherwise only the ComputeNode objects with a UUID in the list of
+            UUIDs in any given cell is returned. If this is an empty list, the
+            returned compute_nodes tuple item will be an empty dict.
 
         Returns a tuple (compute_nodes, services) where:
          - compute_nodes is cell-uuid keyed dict of compute node lists
