@@ -35,9 +35,9 @@ import os
 import os.path
 import pprint
 import sys
+from unittest import mock
 
 import fixtures
-import mock
 from oslo_cache import core as cache
 from oslo_concurrency import lockutils
 from oslo_config import cfg
@@ -286,6 +286,7 @@ class TestCase(base.BaseTestCase):
         quota.UID_QFD_POPULATED_CACHE_ALL = False
 
         self.useFixture(nova_fixtures.GenericPoisonFixture())
+        self.useFixture(nova_fixtures.SysFsPoisonFixture())
 
         # make sure that the wsgi app is fully initialized for all testcase
         # instead of only once initialized for test worker
