@@ -18,10 +18,10 @@ import sys
 import textwrap
 import time
 import typing as ty
+from unittest import mock
 
 import fixtures
 from lxml import etree
-import mock
 from oslo_log import log as logging
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import versionutils
@@ -31,6 +31,7 @@ from nova.objects import fields as obj_fields
 from nova.tests.fixtures import libvirt_data as fake_libvirt_data
 from nova.virt.libvirt import config as vconfig
 from nova.virt.libvirt import driver as libvirt_driver
+from nova.virt.libvirt import host
 
 
 # Allow passing None to the various connect methods
@@ -275,6 +276,7 @@ class FakePCIDevice(object):
             <product id='0x%(prod_id)s'>%(prod_name)s</product>
             <vendor id='0x%(vend_id)s'>%(vend_name)s</vendor>
         %(capability)s
+        %(vpd_capability)s
             <iommuGroup number='%(iommu_group)d'>
               <address domain='0x0000' bus='%(bus)#02x' slot='%(slot)#02x' function='0x%(function)d'/>
             </iommuGroup>
@@ -293,13 +295,22 @@ class FakePCIDevice(object):
         <availableInstances>%(instances)s</availableInstances>
         </type>""".strip())  # noqa
 
+    vpd_cap_templ = textwrap.dedent("""
+        <capability type='vpd'>
+        <name>%(name)s</name>
+        %(fields)s
+        </capability>""".strip())
+    vpd_fields_templ = textwrap.dedent("""
+        <fields access='%(access)s'>%(section_fields)s</fields>""".strip())
+    vpd_field_templ = """<%(field_name)s>%(field_value)s</%(field_name)s>"""
+
     is_capable_of_mdevs = False
 
     def __init__(
         self, dev_type, bus, slot, function, iommu_group, numa_node, *,
         vf_ratio=None, multiple_gpu_types=False, generic_types=False,
         parent=None, vend_id=None, vend_name=None, prod_id=None,
-        prod_name=None, driver_name=None, mac_address=None,
+        prod_name=None, driver_name=None, vpd_fields=None, mac_address=None,
     ):
         """Populate pci devices
 
@@ -343,6 +354,8 @@ class FakePCIDevice(object):
         self.driver_name = driver_name
         self.mac_address = mac_address
 
+        self.vpd_fields = vpd_fields
+
         self.generate_xml()
 
     def generate_xml(self, skip_capability=False):
@@ -452,6 +465,7 @@ class FakePCIDevice(object):
             'prod_name': prod_name,
             'driver': driver,
             'capability': capability,
+            'vpd_capability': self.format_vpd_cap(),
             'iommu_group': self.iommu_group,
             'numa_node': self.numa_node,
             'parent': parent,
@@ -462,6 +476,30 @@ class FakePCIDevice(object):
         if self.numa_node == -1:
             self.pci_device = self.pci_device.replace("<numa node='-1'/>", "")
 
+    def format_vpd_cap(self):
+        if not self.vpd_fields:
+            return ''
+        fields = []
+        for access_type in ('readonly', 'readwrite'):
+            section_fields = []
+            for field_name, field_value in self.vpd_fields.get(
+                    access_type, {}).items():
+                section_fields.append(self.vpd_field_templ % {
+                    'field_name': field_name,
+                    'field_value': field_value,
+                })
+            if section_fields:
+                fields.append(
+                    self.vpd_fields_templ % {
+                        'access': access_type,
+                        'section_fields': '\n'.join(section_fields),
+                    }
+                )
+        return self.vpd_cap_templ % {
+            'name': self.vpd_fields.get('name', ''),
+            'fields': '\n'.join(fields)
+        }
+
     def XMLDesc(self, flags):
         return self.pci_device
 
@@ -496,7 +534,7 @@ class HostPCIDevicesInfo(object):
         """
         self.devices = {}
 
-        if not (num_vfs or num_pfs) and not num_mdevcap:
+        if not (num_vfs or num_pfs or num_pci) and not num_mdevcap:
             return
 
         if num_vfs and not num_pfs:
@@ -581,7 +619,7 @@ class HostPCIDevicesInfo(object):
         self, dev_type, bus, slot, function, iommu_group, numa_node,
         vf_ratio=None, multiple_gpu_types=False, generic_types=False,
         parent=None, vend_id=None, vend_name=None, prod_id=None,
-        prod_name=None, driver_name=None, mac_address=None,
+        prod_name=None, driver_name=None, vpd_fields=None, mac_address=None,
     ):
         pci_dev_name = _get_libvirt_nodedev_name(bus, slot, function)
 
@@ -603,6 +641,7 @@ class HostPCIDevicesInfo(object):
             prod_id=prod_id,
             prod_name=prod_name,
             driver_name=driver_name,
+            vpd_fields=vpd_fields,
             mac_address=mac_address,
         )
         self.devices[pci_dev_name] = dev
@@ -925,6 +964,7 @@ class libvirtError(Exception):
     Alternatively, you can use the `make_libvirtError` convenience function to
     allow you to specify these attributes in one shot.
     """
+
     def __init__(self, defmsg, conn=None, dom=None, net=None, pool=None,
                  vol=None):
         Exception.__init__(self, defmsg)
@@ -1394,20 +1434,28 @@ class Domain(object):
                     'Test attempts to add more than 8 PCI devices. This is '
                     'not supported by the fake libvirt implementation.')
             nic['func'] = func
-            # this branch covers most interface types with a source
-            # such as linux bridge interfaces.
-            if 'source' in nic:
+            if nic['type'] in ('ethernet',):
+                # this branch covers kernel ovs interfaces
                 nics += '''<interface type='%(type)s'>
           <mac address='%(mac)s'/>
-          <source %(type)s='%(source)s'/>
           <target dev='tap274487d1-6%(func)s'/>
           <address type='pci' domain='0x0000' bus='0x00' slot='0x03'
                    function='0x%(func)s'/>
         </interface>''' % nic
-            elif nic['type'] in ('ethernet',):
-                # this branch covers kernel ovs interfaces
+            elif nic['type'] in ('vdpa',):
+                # this branch covers hardware offloaded ovs with vdpa
                 nics += '''<interface type='%(type)s'>
           <mac address='%(mac)s'/>
+          <source dev='%(source)s'/>
+          <address type='pci' domain='0x0000' bus='0x00' slot='0x03'
+                   function='0x%(func)s'/>
+        </interface>''' % nic
+            # this branch covers most interface types with a source
+            # such as linux bridge interfaces.
+            elif 'source' in nic:
+                nics += '''<interface type='%(type)s'>
+          <mac address='%(mac)s'/>
+          <source %(type)s='%(source)s'/>
           <target dev='tap274487d1-6%(func)s'/>
           <address type='pci' domain='0x0000' bus='0x00' slot='0x03'
                    function='0x%(func)s'/>
@@ -2157,6 +2205,7 @@ _EventAddHandleFunc = FakeHandler
 class LibvirtFixture(fixtures.Fixture):
     """Performs global setup/stubbing for all libvirt tests.
     """
+
     def __init__(self, stub_os_vif=True):
         self.stub_os_vif = stub_os_vif
         self.pci_address_to_mac_map = collections.defaultdict(
@@ -2230,6 +2279,15 @@ class LibvirtFixture(fixtures.Fixture):
         self.mock_uname = self.useFixture(
             fixtures.MockPatch('os.uname', return_value=fake_uname)).mock
 
+        real_exists = os.path.exists
+
+        def fake_exists(path):
+            if path == host.SEV_KERNEL_PARAM_FILE:
+                return False
+            return real_exists(path)
+
+        self.useFixture(fixtures.MonkeyPatch('os.path.exists', fake_exists))
+
         # ...and on all machine types
         fake_loaders = [
             {
