@@ -17,21 +17,24 @@
 """Fixtures for Nova tests."""
 
 import collections
+import contextlib
 from contextlib import contextmanager
 import functools
 import logging as std_logging
 import os
 import time
+from unittest import mock
 import warnings
 
 import eventlet
 import fixtures
 import futurist
-import mock
 from openstack import service_description
 from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_db import exception as db_exc
+from oslo_db.sqlalchemy import enginefacade
+from oslo_db.sqlalchemy import test_fixtures as db_fixtures
 from oslo_log import log as logging
 import oslo_messaging as messaging
 from oslo_messaging import conffixture as messaging_conffixture
@@ -65,7 +68,6 @@ CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 DB_SCHEMA = collections.defaultdict(str)
-SESSION_CONFIGURED = False
 PROJECT_ID = '6f70656e737461636b20342065766572'
 
 
@@ -103,6 +105,7 @@ class NullHandler(std_logging.Handler):
     log_fixture.get_logging_handle_error_fixture to detect formatting errors in
     debug level logs without saving the logs.
     """
+
     def handle(self, record):
         self.format(record)
 
@@ -173,6 +176,9 @@ class StandardLogging(fixtures.Fixture):
                 'migrate.versioning.api').setLevel(std_logging.WARNING)
             # Or alembic for model comparisons.
             std_logging.getLogger('alembic').setLevel(std_logging.WARNING)
+            # Or oslo_db provisioning steps
+            std_logging.getLogger('oslo_db.sqlalchemy').setLevel(
+                std_logging.WARNING)
 
         # At times we end up calling back into main() functions in
         # testing. This has the possibility of calling logging.setup
@@ -200,6 +206,18 @@ class DatabasePoisonFixture(fixtures.Fixture):
             '_create_session',
             self._poison_configure))
 
+        # NOTE(gibi): not just _create_session indicates a manipulation on the
+        # DB but actually any operation that actually initializes (starts) a
+        # transaction factory. If a test does this without using the Database
+        # fixture then that test i) actually a database test and should declare
+        # it so ii) actually manipulates a global state without proper cleanup
+        # and test isolation. This could lead that later tests are failing with
+        # the error: oslo_db.sqlalchemy.enginefacade.AlreadyStartedError: this
+        # TransactionFactory is already started
+        self.useFixture(fixtures.MonkeyPatch(
+           'oslo_db.sqlalchemy.enginefacade._TransactionFactory._start',
+           self._poison_configure))
+
     def _poison_configure(self, *a, **k):
         # If you encounter this error, you might be tempted to just not
         # inherit from NoDBTestCase. Bug #1568414 fixed a few hundred of these
@@ -350,6 +368,7 @@ class CheatingSerializer(rpc.RequestContextSerializer):
     Unless we had per-service config and database layer state for
     the fake services we start, this is a reasonable cheat.
     """
+
     def serialize_context(self, context):
         """Serialize context with the db_connection inside."""
         values = super(CheatingSerializer, self).serialize_context(context)
@@ -378,6 +397,7 @@ class CellDatabases(fixtures.Fixture):
     Passing default=True tells the fixture which database should
     be given to code that doesn't target a specific cell.
     """
+
     def __init__(self):
         self._ctxt_mgrs = {}
         self._last_ctxt_mgr = None
@@ -388,7 +408,7 @@ class CellDatabases(fixtures.Fixture):
         # to point to a cell, we need to take an exclusive lock to
         # prevent any other calls to get_context_manager() until we
         # reset to the default.
-        self._cell_lock = lockutils.ReaderWriterLock()
+        self._cell_lock = ReaderWriterLock()
 
     def _cache_schema(self, connection_str):
         # NOTE(melwitt): See the regular Database fixture for why
@@ -634,58 +654,70 @@ class Database(fixtures.Fixture):
         """
         super().__init__()
 
-        # NOTE(pkholkin): oslo_db.enginefacade is configured in tests the
-        # same way as it is done for any other service that uses DB
-        global SESSION_CONFIGURED
-        if not SESSION_CONFIGURED:
-            main_db_api.configure(CONF)
-            api_db_api.configure(CONF)
-            SESSION_CONFIGURED = True
-
         assert database in {'main', 'api'}, f'Unrecognized database {database}'
+        if database == 'api':
+            assert connection is None, 'Not supported for the API database'
 
         self.database = database
         self.version = version
+        self.connection = connection
 
-        if database == 'main':
-            if connection is not None:
+    def setUp(self):
+        super().setUp()
+
+        if self.database == 'main':
+
+            if self.connection is not None:
                 ctxt_mgr = main_db_api.create_context_manager(
-                    connection=connection)
+                    connection=self.connection)
                 self.get_engine = ctxt_mgr.writer.get_engine
             else:
+                # NOTE(gibi): this injects a new factory for each test and
+                # cleans it up at then end of the test case. This way we can
+                # let each test configure the factory so we can avoid having a
+                # global flag guarding against factory re-configuration
+                new_engine = enginefacade.transaction_context()
+                self.useFixture(
+                    db_fixtures.ReplaceEngineFacadeFixture(
+                        main_db_api.context_manager, new_engine))
+                main_db_api.configure(CONF)
+
                 self.get_engine = main_db_api.get_engine
-        elif database == 'api':
-            assert connection is None, 'Not supported for the API database'
+        elif self.database == 'api':
+            # NOTE(gibi): similar note applies here as for the main_db_api
+            # above
+            new_engine = enginefacade.transaction_context()
+            self.useFixture(
+                db_fixtures.ReplaceEngineFacadeFixture(
+                    api_db_api.context_manager, new_engine))
+            api_db_api.configure(CONF)
 
             self.get_engine = api_db_api.get_engine
 
-    def setUp(self):
-        super(Database, self).setUp()
-        self.reset()
+        self._apply_schema()
+
         self.addCleanup(self.cleanup)
 
-    def _cache_schema(self):
+    def _apply_schema(self):
         global DB_SCHEMA
         if not DB_SCHEMA[(self.database, self.version)]:
+            # apply and cache schema
             engine = self.get_engine()
             conn = engine.connect()
             migration.db_sync(database=self.database, version=self.version)
             DB_SCHEMA[(self.database, self.version)] = "".join(
                 line for line in conn.connection.iterdump())
-            engine.dispose()
+        else:
+            # apply the cached schema
+            engine = self.get_engine()
+            conn = engine.connect()
+            conn.connection.executescript(
+                DB_SCHEMA[(self.database, self.version)])
 
     def cleanup(self):
         engine = self.get_engine()
         engine.dispose()
 
-    def reset(self):
-        engine = self.get_engine()
-        engine.dispose()
-        self._cache_schema()
-        conn = engine.connect()
-        conn.connection.executescript(
-            DB_SCHEMA[(self.database, self.version)])
-
 
 class DefaultFlavorsFixture(fixtures.Fixture):
     def setUp(self):
@@ -774,7 +806,10 @@ class WarningsFixture(fixtures.Fixture):
     """Filters out warnings during test runs."""
 
     def setUp(self):
-        super(WarningsFixture, self).setUp()
+        super().setUp()
+
+        self._original_warning_filters = warnings.filters[:]
+
         # NOTE(sdague): Make deprecation warnings only happen once. Otherwise
         # this gets kind of crazy given the way that upstream python libs use
         # this.
@@ -784,15 +819,19 @@ class WarningsFixture(fixtures.Fixture):
         # forward on is_admin, the deprecation is definitely really premature.
         warnings.filterwarnings(
             'ignore',
-            message='Policy enforcement is depending on the value of is_admin.'
-                    ' This key is deprecated. Please update your policy '
-                    'file to use the standard policy values.')
+            message=(
+                'Policy enforcement is depending on the value of is_admin. '
+                'This key is deprecated. Please update your policy '
+                'file to use the standard policy values.'
+            ),
+        )
 
         # NOTE(mriedem): Ignore scope check UserWarnings from oslo.policy.
         warnings.filterwarnings(
             'ignore',
             message="Policy .* failed scope check",
-            category=UserWarning)
+            category=UserWarning,
+        )
 
         # NOTE(gibi): The UUIDFields emits a warning if the value is not a
         # valid UUID. Let's escalate that to an exception in the test to
@@ -804,34 +843,26 @@ class WarningsFixture(fixtures.Fixture):
         # how to handle (or isn't given a fallback callback).
         warnings.filterwarnings(
             'error',
-            message="Cannot convert <oslo_db.sqlalchemy.enginefacade"
-                    "._Default object at ",
-            category=UserWarning)
+            message=(
+                'Cannot convert <oslo_db.sqlalchemy.enginefacade._Default '
+                'object at '
+            ),
+            category=UserWarning,
+        )
+
+        # Enable deprecation warnings for nova itself to capture upcoming
+        # SQLAlchemy changes
 
         warnings.filterwarnings(
-            'error', message='Evaluating non-mapped column expression',
-            category=sqla_exc.SAWarning)
-
-        # Enable deprecation warnings to capture upcoming SQLAlchemy changes
+            'ignore',
+            category=sqla_exc.SADeprecationWarning,
+        )
 
         warnings.filterwarnings(
             'error',
             module='nova',
-            category=sqla_exc.SADeprecationWarning)
-
-        # TODO(stephenfin): Remove once we fix this is oslo.db 10.0.1 or so
-        warnings.filterwarnings(
-            'ignore',
-            message=r'Invoking and_\(\) without arguments is deprecated, .*',
-            category=sqla_exc.SADeprecationWarning)
-
-        # TODO(stephenfin): Remove once we fix this in placement 5.0.2 or 6.0.0
-        warnings.filterwarnings(
-            'ignore',
-            message='Implicit coercion of SELECT and textual SELECT .*',
-            category=sqla_exc.SADeprecationWarning)
-
-        self.addCleanup(warnings.resetwarnings)
+            category=sqla_exc.SADeprecationWarning,
+        )
 
         # Enable general SQLAlchemy warnings also to ensure we're not doing
         # silly stuff. It's possible that we'll need to filter things out here
@@ -843,6 +874,11 @@ class WarningsFixture(fixtures.Fixture):
             category=sqla_exc.SAWarning,
         )
 
+        self.addCleanup(self._reset_warning_filters)
+
+    def _reset_warning_filters(self):
+        warnings.filters[:] = self._original_warning_filters
+
 
 class ConfPatcher(fixtures.Fixture):
     """Fixture to patch and restore global CONF.
@@ -896,6 +932,11 @@ class OSAPIFixture(fixtures.Fixture):
     - resp.content - the body of the response
     - resp.headers - dictionary of HTTP headers returned
 
+    This fixture also has the following clients with various differences:
+
+        self.admin_api - Project user with is_admin=True and the "admin" role
+        self.reader_api - Project user with only the "reader" role
+        self.other_api - Project user with only the "other" role
     """
 
     def __init__(
@@ -960,17 +1001,22 @@ class OSAPIFixture(fixtures.Fixture):
 
         self.api = client.TestOpenStackClient(
             'fake', base_url, project_id=self.project_id,
-        )
+            roles=['reader', 'member'])
         self.alternative_api = client.TestOpenStackClient(
             'fake', base_url, project_id=self.project_id,
-        )
+            roles=['reader', 'member'])
         self.admin_api = client.TestOpenStackClient(
             'admin', base_url, project_id=self.project_id,
-        )
+            roles=['reader', 'member', 'admin'])
         self.alternative_admin_api = client.TestOpenStackClient(
             'admin', base_url, project_id=self.project_id,
-        )
-
+            roles=['reader', 'member', 'admin'])
+        self.reader_api = client.TestOpenStackClient(
+            'reader', base_url, project_id=self.project_id,
+            roles=['reader'])
+        self.other_api = client.TestOpenStackClient(
+            'other', base_url, project_id=self.project_id,
+            roles=['other'])
         # Provide a way to access the wsgi application to tests using
         # the fixture.
         self.app = app
@@ -987,8 +1033,9 @@ class OSAPIFixture(fixtures.Fixture):
             user_id = env['HTTP_X_AUTH_USER']
             project_id = env['HTTP_X_AUTH_PROJECT_ID']
             is_admin = user_id == 'admin'
+            roles = env['HTTP_X_ROLES'].split(',')
             return context.RequestContext(
-                user_id, project_id, is_admin=is_admin, **kwargs)
+                user_id, project_id, is_admin=is_admin, roles=roles, **kwargs)
 
         self.useFixture(fixtures.MonkeyPatch(
             'nova.api.auth.NovaKeystoneContext._create_context', fake_ctx))
@@ -1005,6 +1052,7 @@ class OSMetadataServer(fixtures.Fixture):
     interactions needed.
 
     """
+
     def setUp(self):
         super(OSMetadataServer, self).setUp()
         # in order to run these in tests we need to bind only to local
@@ -1134,6 +1182,7 @@ class SynchronousThreadPoolExecutorFixture(fixtures.Fixture):
 
     Replace the GreenThreadPoolExecutor with the SynchronousExecutor.
     """
+
     def setUp(self):
         super(SynchronousThreadPoolExecutorFixture, self).setUp()
         self.useFixture(fixtures.MonkeyPatch(
@@ -1142,6 +1191,7 @@ class SynchronousThreadPoolExecutorFixture(fixtures.Fixture):
 
 class BannedDBSchemaOperations(fixtures.Fixture):
     """Ban some operations for migrations"""
+
     def __init__(self, banned_resources=None):
         super(BannedDBSchemaOperations, self).__init__()
         self._banned_resources = banned_resources or []
@@ -1165,6 +1215,7 @@ class BannedDBSchemaOperations(fixtures.Fixture):
 
 class ForbidNewLegacyNotificationFixture(fixtures.Fixture):
     """Make sure the test fails if new legacy notification is added"""
+
     def __init__(self):
         super(ForbidNewLegacyNotificationFixture, self).__init__()
         self.notifier = rpc.LegacyValidatingNotifier
@@ -1258,12 +1309,84 @@ class PrivsepFixture(fixtures.Fixture):
     """Disable real privsep checking so we can test the guts of methods
     decorated with sys_admin_pctxt.
     """
+
     def setUp(self):
         super(PrivsepFixture, self).setUp()
         self.useFixture(fixtures.MockPatchObject(
             nova.privsep.sys_admin_pctxt, 'client_mode', False))
 
 
+class CGroupsFixture(fixtures.Fixture):
+    """Mocks checks made for available subsystems on the host's control group.
+
+    The fixture mocks all calls made on the host to verify the capabilities
+    provided by its kernel. Through this, one can simulate the underlying
+    system hosts work on top of and have tests react to expected outcomes from
+    such.
+
+    Use sample:
+    >>> cgroups = self.useFixture(CGroupsFixture())
+    >>> cgroups = self.useFixture(CGroupsFixture(version=2))
+    >>> cgroups = self.useFixture(CGroupsFixture())
+    ... cgroups.version = 2
+
+    :attr version: Arranges mocks to simulate the host interact with nova
+                   following the given version of cgroups.
+                   Available values are:
+                        - 0: All checks related to cgroups will return False.
+                        - 1: Checks related to cgroups v1 will return True.
+                        - 2: Checks related to cgroups v2 will return True.
+                   Defaults to 1.
+    """
+
+    def __init__(self, version=1):
+        self._cpuv1 = None
+        self._cpuv2 = None
+
+        self._version = version
+
+    @property
+    def version(self):
+        return self._version
+
+    @version.setter
+    def version(self, value):
+        self._version = value
+        self._update_mocks()
+
+    def setUp(self):
+        super().setUp()
+        self._cpuv1 = self.useFixture(fixtures.MockPatch(
+            'nova.virt.libvirt.host.Host._has_cgroupsv1_cpu_controller')).mock
+        self._cpuv2 = self.useFixture(fixtures.MockPatch(
+            'nova.virt.libvirt.host.Host._has_cgroupsv2_cpu_controller')).mock
+        self._update_mocks()
+
+    def _update_mocks(self):
+        if not self._cpuv1:
+            return
+
+        if not self._cpuv2:
+            return
+
+        if self.version == 0:
+            self._cpuv1.return_value = False
+            self._cpuv2.return_value = False
+            return
+
+        if self.version == 1:
+            self._cpuv1.return_value = True
+            self._cpuv2.return_value = False
+            return
+
+        if self.version == 2:
+            self._cpuv1.return_value = False
+            self._cpuv2.return_value = True
+            return
+
+        raise ValueError(f"Unknown cgroups version: '{self.version}'.")
+
+
 class NoopQuotaDriverFixture(fixtures.Fixture):
     """A fixture to run tests using the NoopQuotaDriver.
 
@@ -1310,6 +1433,7 @@ class DownCellFixture(fixtures.Fixture):
             # List services with down cells.
             self.admin_api.api_get('/os-services')
     """
+
     def __init__(self, down_cell_mappings=None):
         self.down_cell_mappings = down_cell_mappings
 
@@ -1408,9 +1532,10 @@ class AvailabilityZoneFixture(fixtures.Fixture):
     ``get_availability_zones``.
 
     ``get_instance_availability_zone`` will return the availability_zone
-    requested when creating a server otherwise the instance.availabilty_zone
+    requested when creating a server otherwise the instance.availability_zone
     or default_availability_zone is returned.
     """
+
     def __init__(self, zones):
         self.zones = zones
 
@@ -1447,6 +1572,7 @@ class KSAFixture(fixtures.Fixture):
     """Lets us initialize an openstack.connection.Connection by stubbing the
     auth plugin.
     """
+
     def setUp(self):
         super(KSAFixture, self).setUp()
         self.mock_load_auth = self.useFixture(fixtures.MockPatch(
@@ -1558,7 +1684,11 @@ class GenericPoisonFixture(fixtures.Fixture):
                 current = __import__(components[0], {}, {})
                 for component in components[1:]:
                     current = getattr(current, component)
-                if not isinstance(getattr(current, attribute), mock.Mock):
+
+                # NOTE(stephenfin): There are a couple of mock libraries in use
+                # (including mocked versions of mock from oslotest) so we can't
+                # use isinstance checks here
+                if 'mock' not in str(type(getattr(current, attribute))):
                     self.useFixture(fixtures.MonkeyPatch(
                         meth, poison_configure(meth, why)))
             except ImportError:
@@ -1575,6 +1705,7 @@ class PropagateTestCaseIdToChildEventlets(fixtures.Fixture):
     https://bugs.launchpad.net/nova/+bug/1946339
 
     """
+
     def __init__(self, test_case_id):
         self.test_case_id = test_case_id
 
@@ -1645,3 +1776,96 @@ class PropagateTestCaseIdToChildEventlets(fixtures.Fixture):
         # our initialization to the child eventlet
         self.useFixture(
             fixtures.MonkeyPatch('nova.utils.spawn_n', wrapped_spawn_n))
+
+
+class ReaderWriterLock(lockutils.ReaderWriterLock):
+    """Wrap oslo.concurrency lockutils.ReaderWriterLock to support eventlet.
+
+    As of fasteners >= 0.15, the workaround code to use eventlet.getcurrent()
+    if eventlet patching is detected has been removed and
+    threading.current_thread is being used instead. Although we are running in
+    a greenlet in our test environment, we are not running in a greenlet of
+    type GreenThread. A GreenThread is created by calling eventlet.spawn() and
+    spawn() is not used to run our tests. At the time of this writing, the
+    eventlet patched threading.current_thread() method falls back to the
+    original unpatched current_thread() method if it is not called from a
+    GreenThead [1] and that breaks our tests involving this fixture.
+
+    We can work around this by patching threading.current_thread() with
+    eventlet.getcurrent() during creation of the lock object, if we detect we
+    are eventlet patched. If we are not eventlet patched, we use a no-op
+    context manager.
+
+    Note: this wrapper should be used for any ReaderWriterLock because any lock
+    may possibly be running inside a plain greenlet created by spawn_n().
+
+    See https://github.com/eventlet/eventlet/issues/731 for details.
+
+    [1] https://github.com/eventlet/eventlet/blob/v0.32.0/eventlet/green/threading.py#L128  # noqa
+    """
+
+    def __init__(self, *a, **kw):
+        eventlet_patched = eventlet.patcher.is_monkey_patched('thread')
+        mpatch = fixtures.MonkeyPatch(
+            'threading.current_thread', eventlet.getcurrent)
+        with mpatch if eventlet_patched else contextlib.ExitStack():
+            super().__init__(*a, **kw)
+
+
+class SysFsPoisonFixture(fixtures.Fixture):
+
+    def inject_poison(self, module_name, function_name):
+        import importlib
+        mod = importlib.import_module(module_name)
+        orig_f = getattr(mod, function_name)
+        if (
+            isinstance(orig_f, mock.Mock) or
+            # FIXME(gibi): Is this a bug in unittest.mock? If I remove this
+            # then LibvirtReportSevTraitsTests fails as builtins.open is mocked
+            # there at import time via @test.patch_open. That injects a
+            # MagicMock instance to builtins.open which we check here against
+            # Mock (or even MagicMock) via isinstance and that check says it is
+            # not a mock. More interestingly I cannot reproduce the same
+            # issue with @test.patch_open and isinstance in a simple python
+            # interpreter. So to make progress I'm checking the class name
+            # here instead as that works.
+            orig_f.__class__.__name__ == "MagicMock"
+        ):
+            # the target is already mocked, probably via a decorator run at
+            # import time, so we don't need to inject our poison
+            return
+
+        full_name = module_name + "." + function_name
+
+        def toxic_wrapper(*args, **kwargs):
+            path = args[0]
+            if isinstance(path, bytes):
+                pattern = b'/sys'
+            elif isinstance(path, str):
+                pattern = '/sys'
+            else:
+                # we ignore the rest of the potential pathlike types for now
+                pattern = None
+
+            if pattern and path.startswith(pattern):
+                raise Exception(
+                    'This test invokes %s on %s. It is bad, you '
+                    'should mock it.'
+                    % (full_name, path)
+                )
+            else:
+                return orig_f(*args, **kwargs)
+
+        self.useFixture(fixtures.MonkeyPatch(full_name, toxic_wrapper))
+
+    def setUp(self):
+        super().setUp()
+        self.inject_poison("os.path", "isdir")
+        self.inject_poison("builtins", "open")
+        self.inject_poison("glob", "iglob")
+        self.inject_poison("os", "listdir")
+        self.inject_poison("glob", "glob")
+        # TODO(gibi): Would be good to poison these too but that makes
+        # a bunch of test to fail
+        # self.inject_poison("os.path", "exists")
+        # self.inject_poison("os", "stat")
