@@ -13,11 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import nova.conf
 from nova.tests.functional.api_sample_tests import test_servers
 
-CONF = nova.conf.CONF
-
 
 class AdminPasswordJsonTest(test_servers.ServersSampleBase):
     sample_dir = 'os-admin-password'
@@ -29,4 +26,4 @@ class AdminPasswordJsonTest(test_servers.ServersSampleBase):
                                  'admin-password-change-password',
                                  subs)
         self.assertEqual(202, response.status_code)
-        self.assertEqual("", response.content)
+        self.assertEqual("", response.text)
