@@ -13,10 +13,10 @@
 # under the License.
 
 import collections
-import fixtures
-import mock
 import os.path
+from unittest import mock
 
+import fixtures
 from oslo_utils import fileutils
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import units
@@ -27,11 +27,10 @@ from nova import exception
 from nova import objects
 from nova import test
 from nova.tests import fixtures as nova_fixtures
+from nova.tests.fixtures import libvirt as fakelibvirt
 from nova.tests.functional import fixtures as func_fixtures
 from nova.tests.functional import integrated_helpers
 from nova.tests.unit import fake_network
-from nova.tests.unit import fake_notifier
-from nova.tests.unit.virt.libvirt import fakelibvirt
 from nova.virt.libvirt import config as libvirt_config
 
 CONF = conf.CONF
@@ -108,6 +107,7 @@ SERVER_DISKS = {
 
 class _FileTest(object):
     """A base class for the _FlatTest and _Qcow2Test mixin test classes"""
+
     def setUp(self):
         super(_FileTest, self).setUp()
 
@@ -149,6 +149,7 @@ class _FlatTest(_FileTest):
     mock create_image to touch a file so we can assert its existence/removal in
     tests.
     """
+
     def setUp(self):
         super(_FlatTest, self).setUp()
 
@@ -173,6 +174,7 @@ class _Qcow2Test(_FileTest):
     mock create_image to touch a file so we can assert its existence/removal in
     tests.
     """
+
     def setUp(self):
         super(_Qcow2Test, self).setUp()
 
@@ -194,6 +196,7 @@ class _RbdTest(object):
     create_image to store which rbd volumes would have been created, and exists
     to reference that store.
     """
+
     def setUp(self):
         super(_RbdTest, self).setUp()
 
@@ -288,6 +291,7 @@ class _LVMTest(object):
     the nova.virt.libvirt.storage.lvm module immediately before starting a new
     compute.
     """
+
     def setUp(self):
         super(_LVMTest, self).setUp()
 
@@ -396,6 +400,7 @@ class _LibvirtEvacuateTest(integrated_helpers.InstanceHelperMixin):
     with these mixins we get test coverage of all combinations of
     shared/nonshared instanace directories and block storage.
     """
+
     def _start_compute(self, name):
         # NOTE(mdbooth): fakelibvirt's getHostname currently returns a
         # hardcoded 'compute1', which is undesirable if we want multiple fake
@@ -422,6 +427,7 @@ class _LibvirtEvacuateTest(integrated_helpers.InstanceHelperMixin):
         self.useFixture(nova_fixtures.NeutronFixture(self))
         self.useFixture(nova_fixtures.GlanceFixture(self))
         self.useFixture(func_fixtures.PlacementFixture())
+        self.useFixture(nova_fixtures.CGroupsFixture())
         fake_network.set_stub_network_methods(self)
 
         api_fixture = self.useFixture(
@@ -431,10 +437,10 @@ class _LibvirtEvacuateTest(integrated_helpers.InstanceHelperMixin):
         # force_down and evacuate without onSharedStorage
         self.api.microversion = '2.14'
 
-        fake_notifier.stub_notifier(self)
-        self.addCleanup(fake_notifier.reset)
+        self.notifier = self.useFixture(
+            nova_fixtures.NotificationFixture(self))
 
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
 
         # Fake out all the details of volume connection
         self.useFixture(fixtures.MockPatch(
@@ -514,20 +520,17 @@ class _LibvirtEvacuateTest(integrated_helpers.InstanceHelperMixin):
     def _evacuate_with_failure(self, server, compute1):
         # Perform an evacuation during which we experience a failure on the
         # destination host
-        instance_uuid = server['id']
 
         with mock.patch.object(compute1.driver, 'plug_vifs') as plug_vifs:
             plug_vifs.side_effect = test.TestingException
 
-            self.api.post_server_action(instance_uuid,
-                                        {'evacuate': {'host': 'compute1'}})
+            server = self._evacuate_server(
+                server, {'host': 'compute1'}, expected_state='ERROR',
+                expected_task_state=None, expected_migration_status='failed')
 
             # Wait for the rebuild to start, then complete
-            fake_notifier.wait_for_versioned_notifications(
+            self.notifier.wait_for_versioned_notifications(
                     'instance.rebuild.start')
-            self._wait_for_migration_status(server, ['failed'])
-            server = self._wait_for_server_parameter(
-                server, {'OS-EXT-STS:task_state': None})
 
             # Meta-test
             plug_vifs.assert_called()
