@@ -13,12 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
-import six
-import testtools
+from unittest import mock
 
 from oslo_config import cfg
 from oslo_log import log as logging
+import testtools
 
 import nova
 from nova.compute import manager
@@ -26,10 +25,9 @@ from nova.conf import neutron as neutron_conf
 from nova import context as nova_context
 from nova import objects
 from nova.tests import fixtures as nova_fixtures
+from nova.tests.fixtures import libvirt as fakelibvirt
 from nova.tests.functional.api import client
 from nova.tests.functional.libvirt import base
-from nova.tests.unit import fake_notifier
-from nova.tests.unit.virt.libvirt import fakelibvirt
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
@@ -45,7 +43,7 @@ class NUMAServersTestBase(base.ServersTestBase):
 
         # Mock the 'NUMATopologyFilter' filter, as most tests need to inspect
         # this
-        host_manager = self.scheduler.manager.driver.host_manager
+        host_manager = self.scheduler.manager.host_manager
         numa_filter_class = host_manager.filter_cls_map['NUMATopologyFilter']
         host_pass_mock = mock.Mock(wraps=numa_filter_class().host_passes)
         _p = mock.patch('nova.scheduler.filters'
@@ -71,10 +69,14 @@ class NUMAServersTest(NUMAServersTestBase):
             expected_state=end_status)
 
         # Validate the quota usage
-        if filter_called_on_error and end_status == 'ACTIVE':
+        if (
+            filter_called_on_error and expected_usage and
+            end_status == 'ACTIVE'
+        ):
             quota_details = self.api.get_quota_detail()
-            expected_core_usages = expected_usage.get(
-                'VCPU', expected_usage.get('PCPU', 0))
+            expected_core_usages = (
+                expected_usage.get('VCPU', 0) + expected_usage.get('PCPU', 0)
+            )
             self.assertEqual(expected_core_usages,
                              quota_details['cores']['in_use'])
 
@@ -102,9 +104,8 @@ class NUMAServersTest(NUMAServersTestBase):
         nodes.
         """
 
-        host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
+        host_info = fakelibvirt.HostInfo(
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=2, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'hw:numa_nodes': '2'}
@@ -119,6 +120,35 @@ class NUMAServersTest(NUMAServersTestBase):
         self.assertNotIn('cpu_topology', inst.numa_topology.cells[0])
         self.assertNotIn('cpu_topology', inst.numa_topology.cells[1])
 
+    def test_create_server_with_numa_topology_and_cpu_topology_and_pinning(
+            self):
+        """Create a server with two NUMA nodes.
+
+        This should pass and result in a guest NUMA topology with two NUMA
+        nodes, pinned cpus and numa affined memory.
+        """
+
+        host_info = fakelibvirt.HostInfo(
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=4, cpu_threads=1,
+            kB_mem=(1024 * 1024 * 16))  # 16 GB
+        self.start_compute(host_info=host_info, hostname='compute1')
+
+        extra_spec = {
+            'hw:numa_nodes': '2',
+            'hw:cpu_max_sockets': '2',
+            'hw:cpu_max_cores': '2',
+            'hw:cpu_max_threads': '8',
+            'hw:cpu_policy': 'dedicated'}
+        flavor_id = self._create_flavor(vcpu=8, extra_spec=extra_spec)
+        server = self._run_build_test(flavor_id)
+
+        ctx = nova_context.get_admin_context()
+        inst = objects.Instance.get_by_uuid(ctx, server['id'])
+        self.assertEqual(2, len(inst.numa_topology.cells))
+        self.assertLessEqual(inst.vcpu_model.topology.sockets, 2)
+        self.assertLessEqual(inst.vcpu_model.topology.cores, 2)
+        self.assertLessEqual(inst.vcpu_model.topology.threads, 8)
+
     def test_create_server_with_numa_fails(self):
         """Create a two NUMA node instance on a host with only one node.
 
@@ -126,8 +156,7 @@ class NUMAServersTest(NUMAServersTestBase):
         separate host NUMA node.
         """
 
-        host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=2, kB_mem=15740000)
+        host_info = fakelibvirt.HostInfo()
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'hw:numa_nodes': '2'}
@@ -143,8 +172,7 @@ class NUMAServersTest(NUMAServersTestBase):
         """
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=2,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=(1024 * 1024 * 16))  # GB
+                                         cpu_cores=2, cpu_threads=2)
         # create 1024 * 2 MB huge pages, and allocate the rest of the 16 GB as
         # small pages
         for cell in host_info.numa_topology.cells:
@@ -175,8 +203,7 @@ class NUMAServersTest(NUMAServersTestBase):
         """
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=2,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=(1024 * 1024 * 16))  # GB
+                                         cpu_cores=2, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         # create 512 * 2 MB huge pages, and allocate the rest of the 16 GB as
@@ -194,8 +221,8 @@ class NUMAServersTest(NUMAServersTestBase):
 
         self._run_build_test(flavor_id, end_status='ERROR')
 
-    def test_create_server_with_legacy_pinning_policy(self):
-        """Create a server using the legacy 'hw:cpu_policy' extra spec.
+    def test_create_server_with_dedicated_policy(self):
+        """Create a server using the 'hw:cpu_policy=dedicated' extra spec.
 
         This should pass and result in a guest NUMA topology with pinned CPUs.
         """
@@ -205,8 +232,7 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=5, cpu_threads=2,
-                                         kB_mem=15740000)
+                                         cpu_cores=5, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
@@ -220,9 +246,107 @@ class NUMAServersTest(NUMAServersTestBase):
 
         inst = objects.Instance.get_by_uuid(self.ctxt, server['id'])
         self.assertEqual(1, len(inst.numa_topology.cells))
-        self.assertEqual(5, inst.numa_topology.cells[0].cpu_topology.cores)
+        self.assertEqual(5, inst.vcpu_model.topology.sockets)
 
-    def test_create_server_with_legacy_pinning_policy_old_configuration(self):
+    def test_create_server_with_mixed_policy(self):
+        """Create a server using the 'hw:cpu_policy=mixed' extra spec.
+
+        This should pass and result in a guest NUMA topology with a mixture of
+        pinned and unpinned CPUs.
+        """
+
+        # configure the flags so we 2 shared, 2 dedicated CPUs on one node and
+        # 1 shared and 3 dedicated on the other; the guest will request the
+        # latter so it should always land on the second NUMA node
+        self.flags(
+            cpu_dedicated_set='2-3,5-7', cpu_shared_set='0,1,4',
+            group='compute')
+        self.flags(vcpu_pin_set=None)
+
+        host_info = fakelibvirt.HostInfo(
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=4, cpu_threads=1)
+        self.start_compute(host_info=host_info, hostname='compute1')
+
+        # sanity check the created host topology object; this is really just a
+        # test of the fakelibvirt module
+        host_numa = objects.NUMATopology.obj_from_db_obj(
+            objects.ComputeNode.get_by_nodename(
+                self.ctxt, 'compute1',
+            ).numa_topology
+        )
+        self.assertEqual(2, len(host_numa.cells))
+        self.assertEqual({0, 1}, host_numa.cells[0].cpuset)
+        self.assertEqual({2, 3}, host_numa.cells[0].pcpuset)
+        self.assertEqual({4}, host_numa.cells[1].cpuset)
+        self.assertEqual({5, 6, 7}, host_numa.cells[1].pcpuset)
+
+        # create a flavor with 1 shared and 3 dedicated CPUs so that we can
+        # validate that both come from the same host NUMA node
+        extra_spec = {
+            'hw:cpu_policy': 'mixed',
+            'hw:cpu_dedicated_mask': '^0',
+        }
+        flavor_id = self._create_flavor(vcpu=4, extra_spec=extra_spec)
+        expected_usage = {
+            'DISK_GB': 20, 'MEMORY_MB': 2048, 'PCPU': 3, 'VCPU': 1,
+        }
+
+        server = self._run_build_test(flavor_id, expected_usage=expected_usage)
+
+        # sanity check the instance topology
+        inst = objects.Instance.get_by_uuid(self.ctxt, server['id'])
+        self.assertEqual(1, len(inst.numa_topology.cells))
+        self.assertEqual({0}, inst.numa_topology.cells[0].cpuset)
+        self.assertEqual({1, 2, 3}, inst.numa_topology.cells[0].pcpuset)
+        self.assertEqual(
+            {5, 6, 7}, set(inst.numa_topology.cells[0].cpu_pinning.values()),
+        )
+
+    def test_create_server_with_mixed_policy_fails(self):
+        """Create a server using the 'hw:cpu_policy=mixed' extra spec on a host
+        with insufficient shared cores on one node and dedicated cores on the
+        other.
+
+        This should fail since both shared and dedicated instance cores should
+        come from the same host node.
+        """
+        # configure the flags so we mark all cores on one node as shared and
+        # all cores on the other as dedicated; the guest shouldn't be able to
+        # schedule to this unless using a multi-node topology itself
+        self.flags(
+            cpu_dedicated_set='4-7', cpu_shared_set='0-3',
+            group='compute')
+        self.flags(vcpu_pin_set=None)
+
+        host_info = fakelibvirt.HostInfo(
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=4, cpu_threads=1)
+        self.start_compute(host_info=host_info, hostname='compute1')
+
+        # sanity check the created host topology object; this is really just a
+        # test of the fakelibvirt module
+        host_numa = objects.NUMATopology.obj_from_db_obj(
+            objects.ComputeNode.get_by_nodename(
+                self.ctxt, 'compute1',
+            ).numa_topology
+        )
+        self.assertEqual(2, len(host_numa.cells))
+        self.assertEqual({0, 1, 2, 3}, host_numa.cells[0].cpuset)
+        self.assertEqual(set(), host_numa.cells[0].pcpuset)
+        self.assertEqual(set(), host_numa.cells[1].cpuset)
+        self.assertEqual({4, 5, 6, 7}, host_numa.cells[1].pcpuset)
+
+        # create a flavor with 1 shared and 3 dedicated CPUs so that we can
+        # validate that it isn't schedulable
+        extra_spec = {
+            'hw:cpu_policy': 'mixed',
+            'hw:cpu_dedicated_mask': '^0',
+        }
+        flavor_id = self._create_flavor(vcpu=4, extra_spec=extra_spec)
+
+        # There shouldn't be any hosts available to satisfy this request
+        self._run_build_test(flavor_id, end_status='ERROR')
+
+    def test_create_server_with_dedicated_policy_old_configuration(self):
         """Create a server using the legacy extra spec and configuration.
 
         This should pass and result in a guest NUMA topology with pinned CPUs,
@@ -235,8 +359,7 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set='0-7')
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
+                                         cpu_cores=2, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
@@ -264,9 +387,7 @@ class NUMAServersTest(NUMAServersTestBase):
         # host has hyperthreads, which means we're going to end up consuming
         # $flavor.vcpu hosts cores plus the thread sibling(s) for each core
         host_info = fakelibvirt.HostInfo(
-            cpu_nodes=1, cpu_sockets=1, cpu_cores=2, cpu_threads=2,
-            kB_mem=(1024 * 1024 * 16),  # GB
-        )
+            cpu_nodes=1, cpu_sockets=1, cpu_cores=2, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
@@ -287,7 +408,7 @@ class NUMAServersTest(NUMAServersTestBase):
         )
         self.assertEqual({0, 1, 2, 3}, host_numa.cells[0].pinned_cpus)
 
-    def test_create_server_with_legacy_pinning_policy_fails(self):
+    def test_create_server_with_dedicated_policy_fails(self):
         """Create a pinned instance on a host with no PCPUs.
 
         This should fail because we're translating the extra spec and the host
@@ -299,8 +420,7 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=5, cpu_threads=2,
-                                         kB_mem=15740000)
+                                         cpu_cores=5, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
@@ -310,7 +430,7 @@ class NUMAServersTest(NUMAServersTestBase):
         flavor_id = self._create_flavor(vcpu=5, extra_spec=extra_spec)
         self._run_build_test(flavor_id, end_status='ERROR')
 
-    def test_create_server_with_legacy_pinning_policy_quota_fails(self):
+    def test_create_server_with_dedicated_policy_quota_fails(self):
         """Create a pinned instance on a host with PCPUs but not enough quota.
 
         This should fail because the quota request should fail.
@@ -320,8 +440,7 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
+                                         cpu_cores=2, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
@@ -351,9 +470,7 @@ class NUMAServersTest(NUMAServersTestBase):
 
         # host has hyperthreads, which means it should be rejected
         host_info = fakelibvirt.HostInfo(
-            cpu_nodes=2, cpu_sockets=1, cpu_cores=2, cpu_threads=2,
-            kB_mem=(1024 * 1024 * 16),  # GB
-        )
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=2, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
@@ -375,8 +492,7 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
+                                         cpu_cores=2, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'resources:PCPU': '2'}
@@ -388,8 +504,6 @@ class NUMAServersTest(NUMAServersTestBase):
         ctx = nova_context.get_admin_context()
         inst = objects.Instance.get_by_uuid(ctx, server['id'])
         self.assertEqual(1, len(inst.numa_topology.cells))
-        self.assertEqual(1, inst.numa_topology.cells[0].cpu_topology.cores)
-        self.assertEqual(2, inst.numa_topology.cells[0].cpu_topology.threads)
 
     def test_create_server_with_pcpu_fails(self):
         """Create a pinned instance on a host with no PCPUs.
@@ -403,8 +517,7 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=5, cpu_threads=2,
-                                         kB_mem=15740000)
+                                         cpu_cores=5, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'resources:PCPU': 2}
@@ -423,8 +536,7 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
+                                         cpu_cores=2, cpu_threads=2)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'resources:PCPU': '2'}
@@ -472,8 +584,7 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vif_plugging_timeout=0)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
+                                         cpu_cores=2, cpu_threads=2)
 
         # Start services
         self.start_compute(host_info=host_info, hostname='test_compute0')
@@ -620,7 +731,7 @@ class NUMAServersTest(NUMAServersTestBase):
         for host, compute_rp_uuid in self.compute_rp_uuids.items():
             if host == original_host:
                 # the host that had the instance should no longer have
-                # alocations since the resize has been confirmed
+                # allocations since the resize has been confirmed
                 expected_usage = {'VCPU': 0, 'PCPU': 0, 'DISK_GB': 0,
                                   'MEMORY_MB': 0}
             else:
@@ -826,8 +937,7 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=1,
-                                         kB_mem=15740000)
+                                         cpu_cores=2, cpu_threads=1)
         self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
@@ -1077,10 +1187,8 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
         self.flags(cpu_dedicated_set='0-7', group='compute')
         self.flags(vcpu_pin_set=None)
 
-        computes = {}
-        for host, compute in self.computes.items():
-            computes[host] = self.restart_compute_service(compute)
-        self.computes = computes
+        for host in list(self.computes.keys()):
+            self.restart_compute_service(host)
 
         # verify that the inventory, usages and allocation are correct after
         # the reshape
@@ -1313,10 +1421,10 @@ class NUMAServersWithNetworksTest(NUMAServersTestBase):
         ex = self.assertRaises(client.OpenStackApiException,
                                self.api.post_server_action, server['id'], post)
         # NOTE(danms): This wouldn't happen in a real deployment since rebuild
-        # is a cast, but since we are using CastAsCall this will bubble to the
-        # API.
+        # is a cast, but since we are using CastAsCallFixture this will bubble
+        # to the API.
         self.assertEqual(500, ex.response.status_code)
-        self.assertIn('NoValidHost', six.text_type(ex))
+        self.assertIn('NoValidHost', str(ex))
 
     def test_cold_migrate_with_physnet(self):
 
@@ -1372,9 +1480,6 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
         self.image_ref_0 = images[0]['id']
         self.image_ref_1 = images[1]['id']
 
-        fake_notifier.stub_notifier(self)
-        self.addCleanup(fake_notifier.reset)
-
     def _create_active_server(self, server_args=None):
         basic_server = {
             'flavorRef': 1,
@@ -1402,7 +1507,7 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
         # the free space to ensure the numa topology filter does not
         # eliminate the host.
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=4, kB_mem=15740000)
+                                         cpu_cores=4)
         self.start_compute(host_info=host_info)
 
         server = self._create_active_server(
@@ -1426,8 +1531,7 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
         # cpu_cores is set to 2 to ensure that we have enough space
         # to boot the vm but not enough space to rebuild
         # by doubling the resource use during scheduling.
-        host_info = fakelibvirt.HostInfo(
-            cpu_nodes=1, cpu_sockets=1, cpu_cores=2, kB_mem=15740000)
+        host_info = fakelibvirt.HostInfo()
         self.start_compute(host_info=host_info)
 
         server = self._create_active_server(
@@ -1446,7 +1550,7 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
         flavor_id = self._create_flavor(extra_spec=extra_spec)
 
         host_info = fakelibvirt.HostInfo(
-            cpu_nodes=2, cpu_sockets=1, cpu_cores=4, kB_mem=15740000)
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=4)
         self.start_compute(host_info=host_info)
 
         server = self._create_active_server(
@@ -1466,5 +1570,4 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
             client.OpenStackApiException, self._rebuild_server,
             server, self.image_ref_1)
         self.assertEqual(400, ex.response.status_code)
-        self.assertIn("An instance's NUMA topology cannot be changed",
-                      six.text_type(ex))
+        self.assertIn("An instance's NUMA topology cannot be changed", str(ex))
