@@ -13,12 +13,13 @@
 #    under the License.
 
 import itertools
-import mock
+from unittest import mock
 
 from keystoneauth1 import exceptions as ks_exc
 from requests.models import Response
 
 from oslo_serialization import jsonutils
+from oslo_utils.fixture import uuidsentinel as uuids
 
 from nova.accelerator import cyborg
 from nova import context
@@ -99,7 +100,7 @@ class CyborgTestCase(test.NoDBTestCase):
 
     @mock.patch('nova.accelerator.cyborg._CyborgClient.'
                 '_get_device_profile_list')
-    def test_get_device_profile_groups(self, mock_get_dp_list):
+    def _test_get_device_profile_groups(self, mock_get_dp_list, owner):
         mock_get_dp_list.return_value = [{
             "groups": [{
                     "resources:FPGA": "1",
@@ -108,16 +109,26 @@ class CyborgTestCase(test.NoDBTestCase):
             "name": "mydp",
             "uuid": "307076c2-5aed-4f72-81e8-1b42f9aa2ec6"
         }]
-        rg = request_spec.RequestGroup(requester_id='device_profile_0')
+        request_id = cyborg.get_device_profile_group_requester_id(
+            dp_group_id=0, owner=owner)
+        rg = request_spec.RequestGroup(requester_id=request_id)
         rg.add_resource(rclass='FPGA', amount='1')
         rg.add_trait(trait_name='CUSTOM_FPGA_CARD', trait_type='required')
         expected_groups = [rg]
 
-        actual_groups = self.client.get_device_profile_groups('mydp')
+        dp_groups = self.client.get_device_profile_groups('mydp')
+        actual_groups = self.client.get_device_request_groups(dp_groups,
+            owner=owner)
         self.assertEqual(len(expected_groups), len(actual_groups))
         self.assertEqual(expected_groups[0].__dict__,
                          actual_groups[0].__dict__)
 
+    def test_get_device_profile_groups_no_owner(self):
+        self._test_get_device_profile_groups(owner=None)
+
+    def test_get_device_profile_groups_port_owner(self):
+        self._test_get_device_profile_groups(owner=uuids.port)
+
     @mock.patch('nova.accelerator.cyborg._CyborgClient.'
                 '_get_device_profile_list')
     def test_get_device_profile_groups_no_dp(self, mock_get_dp_list):
@@ -179,7 +190,7 @@ class CyborgTestCase(test.NoDBTestCase):
             },
             "attach_handle_type": "PCI",
             "state": "Bound",
-            # Devic eprofile name is common to all bound ARQs
+            # Device profile name is common to all bound ARQs
             "device_profile_name": arqs[0]["device_profile_name"],
             **common
         }
@@ -218,6 +229,27 @@ class CyborgTestCase(test.NoDBTestCase):
         ret_rp_uuids = sorted([arq['device_rp_uuid'] for arq in ret_arqs])
         self.assertEqual(expected_rp_uuids, ret_rp_uuids)
 
+    @mock.patch('nova.accelerator.cyborg._CyborgClient.'
+                '_create_arqs')
+    def test_create_arqs(self, mock_create_arqs):
+        # Happy path
+        arqs, rg_rp_map = self._get_arqs_and_request_groups()
+        dp_name = arqs[0]["device_profile_name"]
+
+        mock_create_arqs.return_value = arqs
+
+        ret_arqs = self.client.create_arqs(dp_name)
+
+        self.assertEqual(arqs, ret_arqs)
+
+    def test_get_arq_device_rp_uuid(self):
+        arqs, rg_rp_map = self._get_arqs_and_request_groups()
+
+        rp_uuid = self.client.get_arq_device_rp_uuid(
+            arqs[0], rg_rp_map, owner=None)
+
+        self.assertEqual(rg_rp_map['device_profile_0'][0], rp_uuid)
+
     @mock.patch('nova.accelerator.cyborg._CyborgClient.'
                 '_create_arqs')
     def test_create_arq_and_match_rps_exception(self, mock_create_arqs):
@@ -335,7 +367,7 @@ class CyborgTestCase(test.NoDBTestCase):
         # If only some ARQs are resolved, return just the resolved ones
         unbound_arqs, _ = self._get_arqs_and_request_groups()
         _, bound_arqs = self._get_bound_arqs()
-        # Create a amixture of unbound and bound ARQs
+        # Create a mixture of unbound and bound ARQs
         arqs = [unbound_arqs[0], bound_arqs[0]]
         instance_uuid = bound_arqs[0]['instance_uuid']
 
@@ -396,6 +428,42 @@ class CyborgTestCase(test.NoDBTestCase):
         mock_log.assert_called_once_with('Failed to delete ARQs %s',
                                          arq_uuid_str)
 
+    @mock.patch('keystoneauth1.adapter.Adapter.get')
+    def test_get_arq_by_uuid(self, mock_cyborg_get):
+        _, bound_arqs = self._get_bound_arqs()
+        arq_uuids = [arq['uuid'] for arq in bound_arqs]
+        content = jsonutils.dumps({'arqs': bound_arqs[0]})
+        resp = fake_requests.FakeResponse(200, content)
+        mock_cyborg_get.return_value = resp
+
+        ret_arqs = self.client.get_arq_by_uuid(arq_uuids[0])
+
+        mock_cyborg_get.assert_called_once_with(
+            "%s/%s" % (self.client.ARQ_URL, arq_uuids[0]))
+        self.assertEqual(bound_arqs[0], ret_arqs['arqs'])
+
+    @mock.patch('nova.accelerator.cyborg._CyborgClient._call_cyborg')
+    def test_get_arq_by_uuid_exception(self, mock_call_cyborg):
+        mock_call_cyborg.return_value = (None, 'Some error')
+        _, bound_arqs = self._get_bound_arqs()
+        arq_uuids = [arq['uuid'] for arq in bound_arqs]
+
+        self.assertRaises(exception.AcceleratorRequestOpFailed,
+                self.client.get_arq_by_uuid,
+                arq_uuids[0])
+
+    @mock.patch('keystoneauth1.adapter.Adapter.get')
+    def test_get_arq_by_uuid_not_found(self, mock_cyborg_get):
+        _, bound_arqs = self._get_bound_arqs()
+        arq_uuids = [arq['uuid'] for arq in bound_arqs]
+        content = jsonutils.dumps({})
+        resp = fake_requests.FakeResponse(404, content)
+        mock_cyborg_get.return_value = resp
+
+        self.assertRaises(exception.AcceleratorRequestOpFailed,
+                self.client.get_arq_by_uuid,
+                arq_uuids[0])
+
     @mock.patch('keystoneauth1.adapter.Adapter.get')
     def test_get_arq_uuids_for_instance(self, mock_cyborg_get):
         # Happy path, without only_resolved=True
@@ -417,3 +485,45 @@ class CyborgTestCase(test.NoDBTestCase):
         bound_arqs.sort()
         ret_arqs.sort()
         self.assertEqual(bound_arqs, ret_arqs)
+
+    def test_get_arq_pci_device_profile(self):
+        """Test extracting arq pci device info"""
+        arq = {'uuid': uuids.arq_uuid,
+                    'device_profile_name': "smart_nic",
+                    'device_profile_group_id': '5',
+                    'state': 'Bound',
+                    'device_rp_uuid': uuids.resource_provider_uuid,
+                    'hostname': "host_nodename",
+                    'instance_uuid': uuids.instance_uuid,
+                    'attach_handle_info': {
+                        'bus': '0c', 'device': '0',
+                        'domain': '0000', 'function': '0',
+                        'physical_network': 'physicalnet1'
+                    },
+                    'attach_handle_type': 'PCI'
+                }
+        expect_info = {
+                 'physical_network': "physicalnet1",
+                 'pci_slot': "0000:0c:0.0",
+                  'arq_uuid': arq['uuid']
+                }
+        bind_info = cyborg.get_arq_pci_device_profile(arq)
+        self.assertEqual(expect_info, bind_info)
+
+    def test_get_device_amount_of_dp_groups(self):
+        group1 = {
+                    "resources:FPGA": "1",
+                    "trait:CUSTOM_FPGA_CARD": "required"
+        }
+        group2 = {
+                    "resources:FPGA": "2",
+                    "trait:CUSTOM_FPGA_CARD": "required"
+        }
+        num = cyborg.get_device_amount_of_dp_groups([group1])
+        self.assertEqual(1, num)
+
+        num = cyborg.get_device_amount_of_dp_groups([group2])
+        self.assertEqual(2, num)
+
+        num = cyborg.get_device_amount_of_dp_groups([group1, group2])
+        self.assertEqual(3, num)
