@@ -13,7 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 import webob
 
 from nova.api.openstack.compute import deferred_delete as dd_v21
