@@ -13,11 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import fixtures
-import mock
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import uuidutils
-import six
 import webob
 
 from nova.api.openstack import api_version_request
@@ -46,9 +46,12 @@ class MigrateServerTestsV21(admin_only_action_common.CommonTests):
         self.stub_out('nova.api.openstack.compute.migrate_server.'
                       'MigrateServerController',
                       lambda *a, **kw: self.controller)
-        self.mock_list_port = self.useFixture(
-            fixtures.MockPatch('nova.network.neutron.API.list_ports')).mock
-        self.mock_list_port.return_value = {'ports': []}
+        self.mock_neutron_extension_list = self.useFixture(
+            fixtures.MockPatch(
+                'nova.network.neutron.API._refresh_neutron_extensions_cache'
+            )
+        ).mock
+        self.mock_neutron_extension_list.return_value = {'extensions': []}
 
     def _get_migration_body(self, **kwargs):
         return {'os-migrateLive': self._get_params(**kwargs)}
@@ -138,6 +141,15 @@ class MigrateServerTestsV21(admin_only_action_common.CommonTests):
                                               allowed=0)
         self._test_migrate_exception(exc_info, webob.exc.HTTPForbidden)
 
+    def test_migrate_raise_badrequest_for_accelerator(self):
+        exc_info = exception.ForbiddenWithAccelerators()
+        self._test_migrate_exception(exc_info, webob.exc.HTTPBadRequest)
+
+    def test_migrate_raise_badrequest_for_vdpainterface(self):
+        exc_info = exception.OperationNotSupportedForVDPAInterface(
+                instance_uuid=uuids.instance, operation='foo')
+        self._test_migrate_exception(exc_info, webob.exc.HTTPBadRequest)
+
     def _test_migrate_live_succeeded(self, param):
         instance = self._stub_instance_get()
 
@@ -226,7 +238,7 @@ class MigrateServerTestsV21(admin_only_action_common.CommonTests):
                                    self.controller._migrate_live,
                                    self.req, instance.uuid, body=body)
             if check_response:
-                self.assertIn(six.text_type(fake_exc), ex.explanation)
+                self.assertIn(str(fake_exc), ex.explanation)
             mock_live_migrate.assert_called_once_with(
                 self.context, instance, False, self.disk_over_commit,
                 'hostname', self.force, self.async_)
@@ -284,11 +296,25 @@ class MigrateServerTestsV21(admin_only_action_common.CommonTests):
             expected_exc=webob.exc.HTTPConflict,
             check_response=False)
 
+    def test_migrate_live_sev_not_supported(self):
+        self._test_migrate_live_failed_with_exception(
+            exception.OperationNotSupportedForSEV(
+                instance_uuid=uuids.instance, operation='foo'),
+            expected_exc=webob.exc.HTTPBadRequest,
+            check_response=False)
+
     def test_migrate_live_vtpm_not_supported(self):
         self._test_migrate_live_failed_with_exception(
             exception.OperationNotSupportedForVTPM(
                 instance_uuid=uuids.instance, operation='foo'),
-            expected_exc=webob.exc.HTTPConflict,
+            expected_exc=webob.exc.HTTPBadRequest,
+            check_response=False)
+
+    def test_migrate_live_vdpa_interfaces_not_supported(self):
+        self._test_migrate_live_failed_with_exception(
+            exception.OperationNotSupportedForVDPAInterface(
+                instance_uuid=uuids.instance, operation='foo'),
+            expected_exc=webob.exc.HTTPBadRequest,
             check_response=False)
 
     def test_migrate_live_pre_check_error(self):
@@ -301,24 +327,14 @@ class MigrateServerTestsV21(admin_only_action_common.CommonTests):
             expected_exc=webob.exc.HTTPInternalServerError,
             check_response=False)
 
-    @mock.patch('nova.objects.Service.get_by_host_and_binary')
-    @mock.patch('nova.api.openstack.common.'
-                'instance_has_port_with_resource_request', return_value=True)
-    def test_migrate_with_bandwidth_from_old_compute_not_supported(
-            self, mock_has_res_req, mock_get_service):
-        instance = self._stub_instance_get()
-
-        mock_get_service.return_value = objects.Service(host=instance['host'])
-        mock_get_service.return_value.version = 38
-
-        self.assertRaises(
-            webob.exc.HTTPConflict, self.controller._migrate, self.req,
-            instance['uuid'], body={'migrate': None})
-
-        mock_has_res_req.assert_called_once_with(
-            instance['uuid'], self.controller.network_api)
-        mock_get_service.assert_called_once_with(
-            self.req.environ['nova.context'], instance['host'], 'nova-compute')
+    @mock.patch('nova.compute.api.API.live_migrate',
+                side_effect=exception.ForbiddenWithAccelerators)
+    def test_live_migration_raises_badrequest_for_accelerators(
+        self, mock_migrate):
+        body = self._get_migration_body(host='hostname')
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller._migrate_live,
+                          self.req, fakes.FAKE_UUID, body=body)
 
 
 class MigrateServerTestsV225(MigrateServerTestsV21):
@@ -515,9 +531,8 @@ class MigrateServerTestsV256(MigrateServerTestsV234):
                           self.req, fakes.FAKE_UUID, body=body)
 
     def _test_migrate_exception(self, exc_info, expected_result):
-        @mock.patch.object(self.compute_api, 'get')
         @mock.patch.object(self.compute_api, 'resize', side_effect=exc_info)
-        def _test(mock_resize, mock_get):
+        def _test(mock_resize):
             instance = objects.Instance(uuid=uuids.instance)
             self.assertRaises(expected_result,
                               self.controller._migrate,
@@ -606,22 +621,6 @@ class MigrateServerTestsV268(MigrateServerTestsV256):
                            method_translations=method_translations,
                            args_map=args_map)
 
-    @mock.patch('nova.virt.hardware.get_mem_encryption_constraint',
-                new=mock.Mock(return_value=True))
-    @mock.patch.object(
-        objects.instance.Instance, 'image_meta',
-        new=objects.ImageMeta.from_dict({}))
-    def test_live_migrate_sev_rejected(self):
-        instance = self._stub_instance_get()
-        body = {'os-migrateLive': {'host': 'hostname',
-                                   'block_migration': 'auto'}}
-        ex = self.assertRaises(webob.exc.HTTPConflict,
-                               self.controller._migrate_live,
-                               self.req, fakes.FAKE_UUID, body=body)
-        self.assertIn("Operation 'live-migration' not supported for "
-                      "SEV-enabled instance (%s)" % instance.uuid,
-                      six.text_type(ex))
-
     def test_live_migrate_with_forced_host(self):
         body = {'os-migrateLive': {'host': 'hostname',
                                    'block_migration': 'auto',
@@ -629,4 +628,4 @@ class MigrateServerTestsV268(MigrateServerTestsV256):
         ex = self.assertRaises(self.validation_error,
                                self.controller._migrate_live,
                                self.req, fakes.FAKE_UUID, body=body)
-        self.assertIn('force', six.text_type(ex))
+        self.assertIn('force', str(ex))
