@@ -14,13 +14,11 @@
 #    under the License.
 
 import datetime
-import uuid
 
 from oslo_serialization import jsonutils
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import routes
-import six
-from six.moves import range
 import webob.dec
 
 from nova.api import auth as api_auth
@@ -30,22 +28,22 @@ from nova.api.openstack import compute
 from nova.api.openstack.compute import versions
 from nova.api.openstack import urlmap
 from nova.api.openstack import wsgi as os_wsgi
-from nova.compute import api as compute_api
+from nova.api import wsgi
 from nova.compute import flavors
 from nova.compute import vm_states
+import nova.conf
 from nova import context
-from nova.db.sqlalchemy import models
+from nova.db.main import models
 from nova import exception as exc
 from nova import objects
 from nova.objects import base
 from nova import quota
 from nova.tests.unit import fake_block_device
-from nova.tests.unit import fake_network
 from nova.tests.unit.objects import test_keypair
 from nova import utils
-from nova import wsgi
 
 
+CONF = nova.conf.CONF
 QUOTAS = quota.QUOTAS
 
 
@@ -60,23 +58,34 @@ def fake_wsgi(self, req):
     return self.application
 
 
-def wsgi_app_v21(fake_auth_context=None, init_only=None, v2_compatible=False):
+def wsgi_app_v21(fake_auth_context=None, v2_compatible=False,
+                 custom_routes=None):
+    # NOTE(efried): Keep this (roughly) in sync with api-paste.ini
 
-    inner_app_v21 = compute.APIRouterV21()
+    def wrap(app, use_context=False):
+        if v2_compatible:
+            app = openstack_api.LegacyV2CompatibleWrapper(app)
 
-    if v2_compatible:
-        inner_app_v21 = openstack_api.LegacyV2CompatibleWrapper(inner_app_v21)
+        if use_context:
+            if fake_auth_context is not None:
+                ctxt = fake_auth_context
+            else:
+                ctxt = context.RequestContext(
+                    'fake', FAKE_PROJECT_ID, auth_token=True)
+            app = api_auth.InjectContext(ctxt, app)
+
+        app = openstack_api.FaultWrapper(app)
+
+        return app
+
+    inner_app_v21 = compute.APIRouterV21(custom_routes=custom_routes)
 
-    if fake_auth_context is not None:
-        ctxt = fake_auth_context
-    else:
-        ctxt = context.RequestContext('fake', 'fake', auth_token=True)
-    api_v21 = openstack_api.FaultWrapper(
-          api_auth.InjectContext(ctxt, inner_app_v21))
     mapper = urlmap.URLMap()
-    mapper['/v2'] = api_v21
-    mapper['/v2.1'] = api_v21
-    mapper['/'] = openstack_api.FaultWrapper(versions.Versions())
+    mapper['/'] = wrap(versions.Versions())
+    mapper['/v2'] = wrap(versions.VersionsV2())
+    mapper['/v2.1'] = wrap(versions.VersionsV2())
+    mapper['/v2/+'] = wrap(inner_app_v21, use_context=True)
+    mapper['/v2.1/+'] = wrap(inner_app_v21, use_context=True)
     return mapper
 
 
@@ -86,9 +95,9 @@ def stub_out_key_pair_funcs(testcase, have_key_pair=True, **kwargs):
                      name='key', public_key='public_key', **kwargs)]
 
     def one_key_pair(context, user_id, name):
-        if name == 'key':
+        if name in ['key', 'new-key']:
             return dict(test_keypair.fake_keypair,
-                        name='key', public_key='public_key', **kwargs)
+                        name=name, public_key='public_key', **kwargs)
         else:
             raise exc.KeypairNotFound(user_id=user_id, name=name)
 
@@ -96,10 +105,10 @@ def stub_out_key_pair_funcs(testcase, have_key_pair=True, **kwargs):
         return []
 
     if have_key_pair:
-        testcase.stub_out('nova.db.key_pair_get_all_by_user', key_pair)
-        testcase.stub_out('nova.db.key_pair_get', one_key_pair)
+        testcase.stub_out('nova.objects.KeyPairList._get_from_db', key_pair)
+        testcase.stub_out('nova.objects.KeyPair._get_from_db', one_key_pair)
     else:
-        testcase.stub_out('nova.db.key_pair_get_all_by_user', no_key_pair)
+        testcase.stub_out('nova.objects.KeyPairList._get_from_db', key_pair)
 
 
 def stub_out_instance_quota(test, allowed, quota, resource='instances'):
@@ -124,7 +133,7 @@ def stub_out_networking(test):
     test.stub_out('oslo_utils.netutils.get_my_ipv4', get_my_ip)
 
 
-def stub_out_compute_api_snapshot(stubs):
+def stub_out_compute_api_snapshot(test):
 
     def snapshot(self, context, instance, name, extra_properties=None):
         # emulate glance rejecting image names which are too long
@@ -133,15 +142,14 @@ def stub_out_compute_api_snapshot(stubs):
         return dict(id='123', status='ACTIVE', name=name,
                     properties=extra_properties)
 
-    stubs.Set(compute_api.API, 'snapshot', snapshot)
+    test.stub_out('nova.compute.api.API.snapshot', snapshot)
 
 
 class stub_out_compute_api_backup(object):
 
-    def __init__(self, stubs):
-        self.stubs = stubs
+    def __init__(self, test):
         self.extra_props_last_call = None
-        stubs.Set(compute_api.API, 'backup', self.backup)
+        test.stub_out('nova.compute.api.API.backup', self.backup)
 
     def backup(self, context, instance, name, backup_type, rotation,
                extra_properties=None):
@@ -152,10 +160,6 @@ class stub_out_compute_api_backup(object):
         return dict(id='123', status='ACTIVE', name=name, properties=props)
 
 
-def stub_out_nw_api_get_instance_nw_info(test, num_networks=1, func=None):
-    fake_network.stub_out_nw_api_get_instance_nw_info(test)
-
-
 def stub_out_nw_api(test, cls=None, private=None, publics=None):
     if not private:
         private = '192.168.0.3'
@@ -175,15 +179,40 @@ def stub_out_nw_api(test, cls=None, private=None, publics=None):
         def validate_networks(self, context, networks, max_count):
             return max_count
 
-        def create_pci_requests_for_sriov_ports(self, context,
-                                                system_metadata,
-                                                requested_networks):
-            pass
+        def create_resource_requests(
+                self, context, requested_networks,
+                pci_requests=None, affinity_policy=None):
+            return None, [], objects.RequestLevelParams()
 
     if cls is None:
         cls = Fake
-    test.stub_out('nova.network.api.API', cls)
-    fake_network.stub_out_nw_api_get_instance_nw_info(test)
+
+    test.stub_out('nova.network.neutron.API', cls)
+
+
+def stub_out_secgroup_api(test, security_groups=None):
+
+    def get_instances_security_groups_bindings(
+            context, servers, detailed=False):
+        instances_security_group_bindings = {}
+        if servers:
+            # we don't get security group information for down cells
+            instances_security_group_bindings = {
+                server['id']: security_groups or [] for server in servers
+                if server['status'] != 'UNKNOWN'
+            }
+        return instances_security_group_bindings
+
+    def get_instance_security_groups(context, instance, detailed=False):
+        return security_groups if security_groups is not None else []
+
+    test.stub_out(
+        'nova.network.security_group_api'
+        '.get_instances_security_groups_bindings',
+        get_instances_security_groups_bindings)
+    test.stub_out(
+        'nova.network.security_group_api.get_instance_security_groups',
+        get_instance_security_groups)
 
 
 class FakeToken(object):
@@ -195,7 +224,7 @@ class FakeToken(object):
     def __init__(self, **kwargs):
         FakeToken.id_count += 1
         self.id = FakeToken.id_count
-        for k, v in six.iteritems(kwargs):
+        for k, v in kwargs.items():
             setattr(self, k, v)
 
 
@@ -211,7 +240,7 @@ class HTTPRequest(os_wsgi.Request):
     def blank(cls, *args, **kwargs):
         defaults = {'base_url': 'http://localhost/v2'}
         use_admin_context = kwargs.pop('use_admin_context', False)
-        project_id = kwargs.pop('project_id', 'fake')
+        project_id = kwargs.pop('project_id', FAKE_PROJECT_ID)
         version = kwargs.pop('version', os_wsgi.DEFAULT_API_VERSION)
         defaults.update(kwargs)
         out = super(HTTPRequest, cls).blank(*args, **defaults)
@@ -236,15 +265,6 @@ class TestRouter(wsgi.Router):
         super(TestRouter, self).__init__(mapper)
 
 
-class TestRouterV21(wsgi.Router):
-    def __init__(self, controller, mapper=None):
-        if not mapper:
-            mapper = routes.Mapper()
-        mapper.resource("test", "tests",
-                        controller=os_wsgi.ResourceV21(controller))
-        super(TestRouterV21, self).__init__(mapper)
-
-
 class FakeAuthDatabase(object):
     data = {}
 
@@ -286,7 +306,7 @@ def create_info_cache(nw_cache):
                                       {'cidr': 'b33f::/64',
                                        'ips': [_ip(ip) for ip in pub1]}]}}]
 
-    if not isinstance(nw_cache, six.string_types):
+    if not isinstance(nw_cache, str):
         nw_cache = jsonutils.dumps(nw_cache)
 
     return {
@@ -302,7 +322,7 @@ def create_info_cache(nw_cache):
 
 def get_fake_uuid(token=0):
     if token not in FAKE_UUIDS:
-        FAKE_UUIDS[token] = str(uuid.uuid4())
+        FAKE_UUIDS[token] = uuidutils.generate_uuid()
     return FAKE_UUIDS[token]
 
 
@@ -315,7 +335,7 @@ def fake_instance_get(**kwargs):
 
 
 def fake_compute_get(**kwargs):
-    def _return_server_obj(context, uuid, expected_attrs=None):
+    def _return_server_obj(context, *a, **kw):
         return stub_instance_obj(context, **kwargs)
     return _return_server_obj
 
@@ -347,6 +367,9 @@ def fake_instance_get_all_by_filters(num_servers=5, **kwargs):
         if 'sort_dirs' in kwargs:
             kwargs.pop('sort_dirs')
 
+        if 'cell_mappings' in kwargs:
+            kwargs.pop('cell_mappings')
+
         for i in range(num_servers):
             uuid = get_fake_uuid(i)
             server = stub_instance(id=i + 1, uuid=uuid,
@@ -366,7 +389,8 @@ def fake_instance_get_all_by_filters(num_servers=5, **kwargs):
 def fake_compute_get_all(num_servers=5, **kwargs):
     def _return_servers_objs(context, search_opts=None, limit=None,
                              marker=None, expected_attrs=None, sort_keys=None,
-                             sort_dirs=None):
+                             sort_dirs=None, cell_down_support=False,
+                             all_tenants=False):
         db_insts = fake_instance_get_all_by_filters()(None,
                                                       limit=limit,
                                                       marker=marker)
@@ -380,7 +404,7 @@ def fake_compute_get_all(num_servers=5, **kwargs):
 
 def stub_instance(id=1, user_id=None, project_id=None, host=None,
                   node=None, vm_state=None, task_state=None,
-                  reservation_id="", uuid=FAKE_UUID, image_ref="10",
+                  reservation_id="", uuid=FAKE_UUID, image_ref=FAKE_UUID,
                   flavor_id="1", name=None, key_name='',
                   access_ipv4=None, access_ipv6=None, progress=0,
                   auto_disk_config=False, display_name=None,
@@ -393,9 +417,9 @@ def stub_instance(id=1, user_id=None, project_id=None, host=None,
                   terminated_at=timeutils.utcnow(),
                   availability_zone='', locked_by=None, cleaned=False,
                   memory_mb=0, vcpus=0, root_gb=0, ephemeral_gb=0,
-                  instance_type=None, launch_index=0, kernel_id="",
+                  flavor=None, launch_index=0, kernel_id="",
                   ramdisk_id="", user_data=None, system_metadata=None,
-                  services=None):
+                  services=None, trusted_certs=None, hidden=False):
     if user_id is None:
         user_id = 'fake_user'
     if project_id is None:
@@ -408,8 +432,8 @@ def stub_instance(id=1, user_id=None, project_id=None, host=None,
     else:
         metadata = []
 
-    inst_type = flavors.get_flavor_by_flavor_id(int(flavor_id))
-    sys_meta = flavors.save_flavor_info({}, inst_type)
+    sys_meta = flavors.save_flavor_info(
+        {}, flavors.get_flavor_by_flavor_id(int(flavor_id)))
     sys_meta.update(system_metadata or {})
 
     if host is not None:
@@ -433,10 +457,11 @@ def stub_instance(id=1, user_id=None, project_id=None, host=None,
 
     info_cache = create_info_cache(nw_cache)
 
-    if instance_type is None:
-        instance_type = flavors.get_default_flavor()
+    if flavor is None:
+        flavor = objects.Flavor.get_by_name(
+            context.get_admin_context(), 'm1.small')
     flavorinfo = jsonutils.dumps({
-        'cur': instance_type.obj_to_primitive(),
+        'cur': flavor.obj_to_primitive(),
         'old': None,
         'new': None,
     })
@@ -452,59 +477,61 @@ def stub_instance(id=1, user_id=None, project_id=None, host=None,
         "image_ref": image_ref,
         "kernel_id": kernel_id,
         "ramdisk_id": ramdisk_id,
+        "hostname": display_name or server_name,
         "launch_index": launch_index,
         "key_name": key_name,
         "key_data": key_data,
-        "config_drive": config_drive,
+        "power_state": power_state,
         "vm_state": vm_state or vm_states.ACTIVE,
         "task_state": task_state,
-        "power_state": power_state,
+        "services": services,
         "memory_mb": memory_mb,
         "vcpus": vcpus,
         "root_gb": root_gb,
         "ephemeral_gb": ephemeral_gb,
         "ephemeral_key_uuid": None,
-        "hostname": display_name or server_name,
         "host": host,
         "node": node,
-        "instance_type_id": 1,
-        "instance_type": inst_type,
+        "instance_type_id": flavor.id,
         "user_data": user_data,
         "reservation_id": reservation_id,
-        "mac_address": "",
         "launched_at": launched_at,
         "terminated_at": terminated_at,
         "availability_zone": availability_zone,
         "display_name": display_name or server_name,
         "display_description": display_description,
+        "launched_on": "",
         "locked": locked_by is not None,
         "locked_by": locked_by,
-        "metadata": metadata,
+        "os_type": "",
+        "architecture": "",
+        "vm_mode": "",
+        "uuid": uuid,
+        "root_device_name": root_device_name,
+        "default_ephemeral_device": "",
+        "default_swap_device": "",
+        "config_drive": config_drive,
         "access_ip_v4": access_ipv4,
         "access_ip_v6": access_ipv6,
-        "uuid": uuid,
-        "progress": progress,
         "auto_disk_config": auto_disk_config,
-        "name": "instance-%s" % id,
+        "progress": progress,
         "shutdown_terminate": True,
         "disable_terminate": False,
-        "security_groups": security_groups,
-        "root_device_name": root_device_name,
-        "system_metadata": utils.dict_to_metadata(sys_meta),
-        "pci_devices": [],
-        "vm_mode": "",
-        "default_swap_device": "",
-        "default_ephemeral_device": "",
-        "launched_on": "",
         "cell_name": "",
-        "architecture": "",
-        "os_type": "",
+        "metadata": metadata,
+        "system_metadata": utils.dict_to_metadata(sys_meta),
+        "security_groups": security_groups,
+        "cleaned": cleaned,
+        "pci_devices": [],
         "extra": {"numa_topology": None,
                   "pci_requests": None,
                   "flavor": flavorinfo,
-              },
-        "cleaned": cleaned,
-        "services": services}
+                  "trusted_certs": trusted_certs,
+                  },
+        "tags": [],
+        "hidden": hidden,
+        "name": "instance-%s" % id,
+    }
 
     instance.update(info_cache)
     instance['info_cache']['instance_uuid'] = instance['uuid']
@@ -515,7 +542,7 @@ def stub_instance(id=1, user_id=None, project_id=None, host=None,
 def stub_instance_obj(ctxt, *args, **kwargs):
     db_inst = stub_instance(*args, **kwargs)
     expected = ['metadata', 'system_metadata', 'flavor',
-                'info_cache', 'security_groups']
+                'info_cache', 'security_groups', 'tags']
     inst = objects.Instance._from_db_object(ctxt, objects.Instance(),
                                             db_inst,
                                             expected_attrs=expected)
@@ -545,7 +572,7 @@ def stub_volume(id, **kwargs):
         'volume_type_id': 'fakevoltype',
         'volume_metadata': [],
         'volume_type': {'name': 'vol_type_name'},
-        'multiattach': True,
+        'multiattach': False,
         'attachments': {'fakeuuid': {'mountpoint': '/'},
                         'fakeuuid2': {'mountpoint': '/dev/sdb'}
                         }
@@ -573,18 +600,10 @@ def stub_volume_update(self, context, *args, **param):
     pass
 
 
-def stub_volume_delete(self, context, *args, **param):
-    pass
-
-
 def stub_volume_get(self, context, volume_id):
     return stub_volume(volume_id)
 
 
-def stub_volume_notfound(self, context, volume_id):
-    raise exc.VolumeNotFound(volume_id=volume_id)
-
-
 def stub_volume_get_all(context, search_opts=None):
     return [stub_volume(100, project_id='fake'),
             stub_volume(101, project_id='superfake'),
@@ -660,10 +679,87 @@ def stub_bdm_get_all_by_instance_uuids(context, instance_uuids,
     return result
 
 
-def fake_get_available_languages():
-    existing_translations = ['en_GB', 'en_AU', 'de', 'zh_CN', 'en_US']
-    return existing_translations
-
-
 def fake_not_implemented(*args, **kwargs):
     raise NotImplementedError()
+
+
+FLAVORS = {
+    '1': objects.Flavor(
+        id=1,
+        name='flavor 1',
+        memory_mb=256,
+        vcpus=1,
+        root_gb=10,
+        ephemeral_gb=20,
+        flavorid='1',
+        swap=10,
+        rxtx_factor=1.0,
+        vcpu_weight=None,
+        disabled=False,
+        is_public=True,
+        description=None,
+        extra_specs={"key1": "value1", "key2": "value2"}
+    ),
+    '2': objects.Flavor(
+        id=2,
+        name='flavor 2',
+        memory_mb=512,
+        vcpus=1,
+        root_gb=20,
+        ephemeral_gb=10,
+        flavorid='2',
+        swap=5,
+        rxtx_factor=None,
+        vcpu_weight=None,
+        disabled=True,
+        is_public=True,
+        description='flavor 2 description',
+        extra_specs={}
+    ),
+}
+
+
+def stub_out_flavor_get_by_flavor_id(test):
+    @staticmethod
+    def fake_get_by_flavor_id(context, flavor_id, read_deleted=None):
+        return FLAVORS[flavor_id]
+
+    test.stub_out('nova.objects.Flavor.get_by_flavor_id',
+                  fake_get_by_flavor_id)
+
+
+def stub_out_flavor_get_all(test):
+    @staticmethod
+    def fake_get_all(context, inactive=False, filters=None,
+                     sort_key='flavorid', sort_dir='asc', limit=None,
+                     marker=None):
+        if marker in ['99999']:
+            raise exc.MarkerNotFound(marker)
+
+        def reject_min(db_attr, filter_attr):
+            return (filter_attr in filters and
+                    getattr(flavor, db_attr) < int(filters[filter_attr]))
+
+        filters = filters or {}
+        res = []
+        for flavor in FLAVORS.values():
+            if reject_min('memory_mb', 'min_memory_mb'):
+                continue
+            elif reject_min('root_gb', 'min_root_gb'):
+                continue
+
+            res.append(flavor)
+
+        res = sorted(res, key=lambda item: getattr(item, sort_key))
+        output = []
+        marker_found = True if marker is None else False
+        for flavor in res:
+            if not marker_found and marker == flavor.flavorid:
+                marker_found = True
+            elif marker_found:
+                if limit is None or len(output) < int(limit):
+                    output.append(flavor)
+
+        return objects.FlavorList(objects=output)
+
+    test.stub_out('nova.objects.FlavorList.get_all', fake_get_all)
