@@ -10,11 +10,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 import oslo_messaging as messaging
 from oslo_serialization import jsonutils
 from oslo_utils.fixture import uuidsentinel as uuids
-import six
 
 from nova.compute import power_state
 from nova.compute import rpcapi as compute_rpcapi
@@ -72,14 +72,14 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
         self.heal_reqspec_is_bfv_mock = _p.start()
         self.addCleanup(_p.stop)
 
-        _p = mock.patch('nova.objects.RequestSpec.ensure_network_metadata')
-        self.ensure_network_metadata_mock = _p.start()
+        _p = mock.patch('nova.objects.RequestSpec.ensure_network_information')
+        self.ensure_network_information_mock = _p.start()
         self.addCleanup(_p.stop)
 
         _p = mock.patch(
             'nova.network.neutron.API.'
             'get_requested_resource_for_instance',
-            return_value=[])
+            return_value=([], objects.RequestLevelParams()))
         self.mock_get_res_req = _p.start()
         self.addCleanup(_p.stop)
 
@@ -155,7 +155,7 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
         # When the task is executed with a destination it means the host is
         # being forced and we don't call the scheduler, so we don't need to
         # modify the request spec
-        self.ensure_network_metadata_mock.assert_not_called()
+        self.ensure_network_information_mock.assert_not_called()
 
     @mock.patch('nova.availability_zones.get_host_availability_zone',
                 return_value='nova')
@@ -346,6 +346,36 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
                           mock.call(self.destination)],
                          mock_get_info.call_args_list)
 
+    @mock.patch.object(live_migrate.LiveMigrationTask, '_get_compute_info')
+    def test_skip_hypervisor_version_check_on_lm_raise_ex(self, mock_get_info):
+        host1 = {'hypervisor_type': 'a', 'hypervisor_version': 7}
+        host2 = {'hypervisor_type': 'a', 'hypervisor_version': 6}
+        self.flags(group='workarounds',
+                   skip_hypervisor_version_check_on_lm=False)
+        mock_get_info.side_effect = [objects.ComputeNode(**host1),
+                                     objects.ComputeNode(**host2)]
+        self.assertRaises(exception.DestinationHypervisorTooOld,
+                          self.task._check_compatible_with_source_hypervisor,
+                          self.destination)
+        self.assertEqual([mock.call(self.instance_host),
+                          mock.call(self.destination)],
+                         mock_get_info.call_args_list)
+
+    @mock.patch.object(live_migrate.LiveMigrationTask, '_get_compute_info')
+    def test_skip_hypervisor_version_check_on_lm_do_not_raise_ex(
+        self, mock_get_info
+    ):
+        host1 = {'hypervisor_type': 'a', 'hypervisor_version': 7}
+        host2 = {'hypervisor_type': 'a', 'hypervisor_version': 6}
+        self.flags(group='workarounds',
+                   skip_hypervisor_version_check_on_lm=True)
+        mock_get_info.side_effect = [objects.ComputeNode(**host1),
+                                     objects.ComputeNode(**host2)]
+        self.task._check_compatible_with_source_hypervisor(self.destination)
+        self.assertEqual([mock.call(self.instance_host),
+                          mock.call(self.destination)],
+                         mock_get_info.call_args_list)
+
     @mock.patch.object(compute_rpcapi.ComputeAPI,
                        'check_can_live_migrate_destination')
     def test_check_requested_destination(self, mock_check):
@@ -354,7 +384,7 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
 
         with test.nested(
             mock.patch.object(self.task.network_api,
-                              'supports_port_binding_extension',
+                              'has_port_binding_extension',
                               return_value=False),
             mock.patch.object(self.task, '_check_can_migrate_pci')):
             self.assertIsNone(self.task._check_requested_destination())
@@ -388,12 +418,12 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
 
         with test.nested(
             mock.patch.object(self.task.network_api,
-                              'supports_port_binding_extension',
+                              'has_port_binding_extension',
                               return_value=False),
             mock.patch.object(self.task, '_check_can_migrate_pci')):
             ex = self.assertRaises(exception.MigrationPreCheckError,
                                    self.task._check_requested_destination)
-        self.assertIn('across cells', six.text_type(ex))
+        self.assertIn('across cells', str(ex))
 
     @mock.patch.object(live_migrate.LiveMigrationTask,
                        '_call_livem_checks_on_host')
@@ -416,7 +446,7 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
 
         mock_setup.assert_called_once_with(self.context, self.fake_spec)
         mock_reset.assert_called_once_with()
-        self.ensure_network_metadata_mock.assert_called_once_with(
+        self.ensure_network_information_mock.assert_called_once_with(
             self.instance)
         self.heal_reqspec_is_bfv_mock.assert_called_once_with(
             self.context, self.fake_spec, self.instance)
@@ -601,8 +631,11 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
             self.assertRaises(exception.MigrationPreCheckError,
                               self.task._call_livem_checks_on_host, {}, {})
 
+    @mock.patch('nova.network.neutron.API.get_binding_profile_allocation')
     @mock.patch('nova.network.neutron.API.bind_ports_to_host')
-    def test_bind_ports_on_destination_merges_profiles(self, mock_bind_ports):
+    def test_bind_ports_on_destination_merges_profiles(
+        self, mock_bind_ports, mock_get_binding_profile_alloc
+    ):
         """Assert that if both the migration_data and the provider mapping
         contains binding profile related information then such information is
         merged in the resulting profile.
@@ -616,6 +649,7 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
                         {'some-key': 'value'}))
             ])
         provider_mappings = {uuids.port1: [uuids.dest_bw_rp]}
+        mock_get_binding_profile_alloc.return_value = uuids.dest_bw_rp
 
         self.task._bind_ports_on_destination('dest-host', provider_mappings)
 
@@ -624,7 +658,59 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
             vnic_types=None,
             port_profiles={uuids.port1: {'allocation': uuids.dest_bw_rp,
                                          'some-key': 'value'}})
+        mock_get_binding_profile_alloc.assert_called_once_with(
+            self.context, uuids.port1, provider_mappings)
 
+    @mock.patch('nova.network.neutron.API.get_binding_profile_allocation')
+    @mock.patch('nova.network.neutron.API.bind_ports_to_host')
+    def test_bind_ports_on_destination_merges_profiles_extended_res_req(
+        self, mock_bind_ports, mock_get_binding_profile_alloc
+    ):
+        """Assert that if both the migration_data and the provider mapping
+        contains binding profile related information and the port has extended
+        resource request then such information is merged in the resulting
+        profile.
+        """
+
+        self.task.migrate_data = objects.LibvirtLiveMigrateData(
+            vifs=[
+                objects.VIFMigrateData(
+                    port_id=uuids.port1,
+                    profile_json=jsonutils.dumps(
+                        {'some-key': 'value'}))
+            ])
+        provider_mappings = {
+            uuids.bw_group: [uuids.dest_bw_rp],
+            uuids.pps_group: [uuids.dest_pps_rp],
+            uuids.accel_group: [uuids.cyborg_rp],
+        }
+        mock_get_binding_profile_alloc.return_value = {
+            uuids.bw_group: uuids.dest_bw_rp,
+            uuids.pps_group: uuids.dest_pps_rp,
+        }
+
+        self.task._bind_ports_on_destination('dest-host', provider_mappings)
+
+        mock_bind_ports.assert_called_once_with(
+            context=self.context, instance=self.instance, host='dest-host',
+            vnic_types=None,
+            port_profiles={
+                uuids.port1: {
+                    'allocation': {
+                        uuids.bw_group: uuids.dest_bw_rp,
+                        uuids.pps_group: uuids.dest_pps_rp,
+                    },
+                    'some-key': 'value'
+                }
+            }
+        )
+        mock_get_binding_profile_alloc.assert_called_once_with(
+            self.context, uuids.port1, provider_mappings)
+
+    @mock.patch(
+        'nova.network.neutron.API.get_binding_profile_allocation',
+        new=mock.Mock(return_value=None)
+    )
     @mock.patch('nova.network.neutron.API.bind_ports_to_host')
     def test_bind_ports_on_destination_migration_data(self, mock_bind_ports):
         """Assert that if only the migration_data contains binding profile
@@ -647,8 +733,11 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
             vnic_types=None,
             port_profiles={uuids.port1: {'some-key': 'value'}})
 
+    @mock.patch('nova.network.neutron.API.get_binding_profile_allocation')
     @mock.patch('nova.network.neutron.API.bind_ports_to_host')
-    def test_bind_ports_on_destination_provider_mapping(self, mock_bind_ports):
+    def test_bind_ports_on_destination_provider_mapping(
+        self, mock_bind_ports, mock_get_binding_profile_alloc
+    ):
         """Assert that if only the provider mapping contains binding
         profile related information then that is sent to neutron.
         """
@@ -659,6 +748,7 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
                     port_id=uuids.port1)
             ])
         provider_mappings = {uuids.port1: [uuids.dest_bw_rp]}
+        mock_get_binding_profile_alloc.return_value = uuids.dest_bw_rp
 
         self.task._bind_ports_on_destination('dest-host', provider_mappings)
 
@@ -666,6 +756,8 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
             context=self.context, instance=self.instance, host='dest-host',
             vnic_types=None,
             port_profiles={uuids.port1: {'allocation': uuids.dest_bw_rp}})
+        mock_get_binding_profile_alloc.assert_called_once_with(
+            self.context, uuids.port1, provider_mappings)
 
     @mock.patch(
         'nova.compute.utils.'
@@ -683,7 +775,9 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
             self, mock_setup, mock_reset, mock_select, mock_check, mock_call,
             mock_fill_provider_mapping, mock_update_pci_req):
         resource_req = [objects.RequestGroup(requester_id=uuids.port_id)]
-        self.mock_get_res_req.return_value = resource_req
+        self.mock_get_res_req.return_value = (
+            resource_req, objects.RequestLevelParams())
+        self.instance.pci_requests = objects.InstancePCIRequests(requests=[])
 
         self.assertEqual(("host1", "node1", fake_limits1),
                          self.task._find_destination())
@@ -699,7 +793,7 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
 
         mock_setup.assert_called_once_with(self.context, self.fake_spec)
         mock_reset.assert_called_once_with()
-        self.ensure_network_metadata_mock.assert_called_once_with(
+        self.ensure_network_information_mock.assert_called_once_with(
             self.instance)
         self.heal_reqspec_is_bfv_mock.assert_called_once_with(
             self.context, self.fake_spec, self.instance)
@@ -710,8 +804,7 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
         mock_fill_provider_mapping.assert_called_once_with(
             self.task.request_spec, fake_selection1)
         mock_update_pci_req.assert_called_once_with(
-            self.context, self.task.report_client, self.instance,
-            {uuids.port_id: []})
+            self.context, self.task.report_client, [], {uuids.port_id: []})
 
     @mock.patch.object(objects.InstanceMapping, 'get_by_instance_uuid',
                        side_effect=exception.InstanceMappingNotFound(
@@ -751,7 +844,7 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
         """
 
         @mock.patch.object(self.task.network_api,
-                           'supports_port_binding_extension')
+                           'has_port_binding_extension')
         @mock.patch.object(live_migrate,
                            'supports_vif_related_pci_allocations')
         def _test(instance_pci_reqs,
