@@ -44,6 +44,7 @@ policy_data = """
     "os_compute_api:servers:trigger_crash_dump": "",
     "os_compute_api:servers:show:host_status": "",
     "os_compute_api:servers:show": "",
+    "os_compute_api:servers:show:flavor-extra-specs" : "",
     "os_compute_api:servers:show:host_status:unknown-only": "",
     "os_compute_api:servers:allow_all_filters": "",
     "os_compute_api:servers:migrations:force_complete": "",
@@ -51,13 +52,8 @@ policy_data = """
     "os_compute_api:servers:migrations:show": "",
     "os_compute_api:servers:migrations:delete": "",
     "os_compute_api:os-admin-actions:inject_network_info": "",
-    "os_compute_api:os-admin-actions:reset_network": "",
     "os_compute_api:os-admin-actions:reset_state": "",
     "os_compute_api:os-admin-password": "",
-    "os_compute_api:os-agents:list": "",
-    "os_compute_api:os-agents:create": "",
-    "os_compute_api:os-agents:update": "",
-    "os_compute_api:os-agents:delete": "",
     "os_compute_api:os-aggregates:set_metadata": "",
     "os_compute_api:os-aggregates:remove_host": "",
     "os_compute_api:os-aggregates:add_host": "",
