@@ -12,11 +12,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import datetime
 import os
+from unittest import mock
 
 import iso8601
-import mock
 from oslo_serialization import jsonutils
 from oslo_versionedobjects import exception as ovo_exc
 
@@ -27,6 +28,10 @@ from nova import test
 from nova.tests.unit import fake_instance
 from nova import utils
 
+os_uname = collections.namedtuple(
+    'uname_result', ['sysname', 'nodename', 'release', 'version', 'machine'],
+)
+
 
 class FakeFieldType(fields.FieldType):
     def coerce(self, obj, attr, value):
@@ -187,7 +192,7 @@ class TestEnum(TestField):
 class TestArchitecture(TestField):
     @mock.patch.object(os, 'uname')
     def test_host(self, mock_uname):
-        mock_uname.return_value = (
+        mock_uname.return_value = os_uname(
             'Linux',
             'localhost.localdomain',
             '3.14.8-200.fc20.x86_64',
@@ -546,7 +551,7 @@ class TestNetworkModel(TestField):
     def setUp(self):
         super(TestNetworkModel, self).setUp()
         model = network_model.NetworkInfo()
-        self.field = fields.Field(fields.NetworkModel())
+        self.field = fields.NetworkModelField()
         self.coerce_good_values = [(model, model), (model.json(), model)]
         self.coerce_bad_values = [[], 'foo']
         self.to_primitive_values = [(model, model.json())]
@@ -565,7 +570,7 @@ class TestNetworkVIFModel(TestField):
         super(TestNetworkVIFModel, self).setUp()
         model = network_model.VIF('6c197bc7-820c-40d5-8aff-7116b993e793')
         primitive = jsonutils.dumps(model)
-        self.field = fields.Field(fields.NetworkVIFModel())
+        self.field = fields.NetworkVIFModelField()
         self.coerce_good_values = [(model, model), (primitive, model)]
         self.coerce_bad_values = [[], 'foo']
         self.to_primitive_values = [(model, primitive)]
