@@ -13,12 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
+from oslo_serialization import jsonutils
 
 from nova import exception
 from nova import objects
 from nova.objects import fields
+from nova.pci.request import PCI_REMOTE_MANAGED_TAG
 from nova.pci import stats
 from nova.pci import whitelist
 from nova import test
@@ -97,7 +100,7 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
 
     def setUp(self):
         super(PciDeviceStatsTestCase, self).setUp()
-        self.pci_stats = stats.PciDeviceStats()
+        self.pci_stats = stats.PciDeviceStats(objects.NUMATopology())
         # The following two calls need to be made before adding the devices.
         patcher = fakes.fake_pci_whitelist()
         self.addCleanup(patcher.stop)
@@ -123,7 +126,7 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
                           self.fake_dev_2)
 
     def test_pci_stats_equivalent(self):
-        pci_stats2 = stats.PciDeviceStats()
+        pci_stats2 = stats.PciDeviceStats(objects.NUMATopology())
         for dev in [self.fake_dev_1,
                     self.fake_dev_2,
                     self.fake_dev_3,
@@ -132,7 +135,7 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
         self.assertEqual(self.pci_stats, pci_stats2)
 
     def test_pci_stats_not_equivalent(self):
-        pci_stats2 = stats.PciDeviceStats()
+        pci_stats2 = stats.PciDeviceStats(objects.NUMATopology())
         for dev in [self.fake_dev_1,
                     self.fake_dev_2,
                     self.fake_dev_3]:
@@ -141,7 +144,7 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
 
     def test_object_create(self):
         m = self.pci_stats.to_device_pools_obj()
-        new_stats = stats.PciDeviceStats(m)
+        new_stats = stats.PciDeviceStats(objects.NUMATopology(), m)
 
         self.assertEqual(len(new_stats.pools), 3)
         self.assertEqual(set([d['count'] for d in new_stats]),
@@ -150,6 +153,7 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
                          set(['v1', 'v2', 'v3']))
 
     def test_apply_requests(self):
+        self.assertEqual(len(self.pci_stats.pools), 3)
         self.pci_stats.apply_requests(pci_requests)
         self.assertEqual(len(self.pci_stats.pools), 2)
         self.assertEqual(self.pci_stats.pools[0]['vendor_id'], 'v1')
@@ -228,6 +232,25 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
 
         self.assertFalse(self.pci_stats.support_requests(pci_requests, cells))
 
+    def test_filter_pools_for_socket_affinity_no_socket(self):
+        self.pci_stats.numa_topology = objects.NUMATopology(
+                cells=[objects.NUMACell(socket=None)])
+
+        self.assertEqual(
+            [],
+            self.pci_stats._filter_pools_for_socket_affinity(
+                self.pci_stats.pools, [objects.InstanceNUMACell()]))
+
+    def test_filter_pools_for_socket_affinity(self):
+        self.pci_stats.numa_topology = objects.NUMATopology(
+                cells=[objects.NUMACell(id=1, socket=1)])
+
+        pools = self.pci_stats._filter_pools_for_socket_affinity(
+            self.pci_stats.pools, [objects.InstanceNUMACell(id=1)])
+        self.assertEqual(1, len(pools))
+        self.assertEqual('p2', pools[0]['product_id'])
+        self.assertEqual('v2', pools[0]['vendor_id'])
+
     def test_consume_requests(self):
         devs = self.pci_stats.consume_requests(pci_requests)
         self.assertEqual(2, len(devs))
@@ -239,8 +262,11 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
         self.assertEqual(0, len(devs))
 
     def test_consume_requests_failed(self):
-        self.assertIsNone(self.pci_stats.consume_requests(
-                          pci_requests_multiple))
+        self.assertRaises(
+            exception.PciDeviceRequestFailed,
+            self.pci_stats.consume_requests,
+            pci_requests_multiple,
+        )
 
     def test_consume_requests_numa(self):
         cells = [
@@ -259,7 +285,12 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
             objects.InstanceNUMACell(
                 id=0, cpuset=set(), pcpuset=set(), memory=0),
         ]
-        self.assertIsNone(self.pci_stats.consume_requests(pci_requests, cells))
+        self.assertRaises(
+            exception.PciDeviceRequestFailed,
+            self.pci_stats.consume_requests,
+            pci_requests,
+            cells,
+        )
 
     def test_consume_requests_no_numa_info(self):
         cells = [
@@ -291,11 +322,16 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
 
         pci_requests = self._get_fake_requests(vendor_ids=[vendor_id],
             numa_policy=policy, count=count)
-        devs = self.pci_stats.consume_requests(pci_requests, cells)
 
         if expected is None:
-            self.assertIsNone(devs)
+            self.assertRaises(
+                exception.PciDeviceRequestFailed,
+                self.pci_stats.consume_requests,
+                pci_requests,
+                cells,
+            )
         else:
+            devs = self.pci_stats.consume_requests(pci_requests, cells)
             self.assertEqual(set(expected),
                              set([dev.product_id for dev in devs]))
 
@@ -422,10 +458,10 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
 
     @mock.patch(
         'nova.pci.whitelist.Whitelist._parse_white_list_from_config')
-    def test_white_list_parsing(self, mock_whitelist_parse):
-        white_list = '{"product_id":"0001", "vendor_id":"8086"}'
-        CONF.set_override('passthrough_whitelist', white_list, 'pci')
-        pci_stats = stats.PciDeviceStats()
+    def test_device_spec_parsing(self, mock_whitelist_parse):
+        device_spec = {"product_id": "0001", "vendor_id": "8086"}
+        CONF.set_override('device_spec', jsonutils.dumps(device_spec), 'pci')
+        pci_stats = stats.PciDeviceStats(objects.NUMATopology())
         pci_stats.add_device(self.fake_dev_2)
         pci_stats.remove_device(self.fake_dev_2)
         self.assertEqual(1, mock_whitelist_parse.call_count)
@@ -435,12 +471,37 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
 
     def setUp(self):
         super(PciDeviceStatsWithTagsTestCase, self).setUp()
-        white_list = ['{"vendor_id":"1137","product_id":"0071",'
-                        '"address":"*:0a:00.*","physical_network":"physnet1"}',
-                       '{"vendor_id":"1137","product_id":"0072"}']
-        self.flags(passthrough_whitelist=white_list, group='pci')
-        dev_filter = whitelist.Whitelist(white_list)
-        self.pci_stats = stats.PciDeviceStats(dev_filter=dev_filter)
+        device_spec = [
+            jsonutils.dumps(
+                {
+                    "vendor_id": "1137",
+                    "product_id": "0071",
+                    "address": "*:0a:00.*",
+                    "physical_network": "physnet1",
+                }
+            ),
+            jsonutils.dumps({"vendor_id": "1137", "product_id": "0072"}),
+            jsonutils.dumps(
+                {
+                    "vendor_id": "15b3",
+                    "product_id": "101e",
+                    "remote_managed": "true",
+                }
+            ),
+            jsonutils.dumps({"vendor_id": "15b3", "product_id": "101c"}),
+            jsonutils.dumps(
+                {
+                    "vendor_id": "15b3",
+                    "product_id": "1018",
+                    "remote_managed": "false",
+                }
+            ),
+        ]
+        self.flags(device_spec=device_spec, group="pci")
+        dev_filter = whitelist.Whitelist(device_spec)
+        self.pci_stats = stats.PciDeviceStats(
+            objects.NUMATopology(),
+            dev_filter=dev_filter)
 
     def _create_pci_devices(self):
         self.pci_tagged_devices = []
@@ -471,12 +532,64 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
             self.pci_untagged_devices.append(objects.PciDevice.create(None,
                                                                       pci_dev))
 
+        self.locally_managed_netdevs = []
+        self.remote_managed_netdevs = []
+        self.remote_managed_netdevs.append(
+            objects.PciDevice.create(
+                None, {
+                    'compute_node_id': 1,
+                    'address': '0000:0c:00.1',
+                    'vendor_id': '15b3',
+                    'product_id': '101e',
+                    'status': 'available',
+                    'request_id': None,
+                    'dev_type': fields.PciDeviceType.SRIOV_VF,
+                    'parent_addr': '0000:0c:00.0',
+                    'numa_node': 0,
+                    "capabilities": {"vpd": {
+                        "card_serial_number": "MT2113X00000"}}
+                }))
+
+        # For testing implicit remote_managed == False tagging.
+        self.locally_managed_netdevs.append(
+            objects.PciDevice.create(
+                None, {
+                    'compute_node_id': 1,
+                    'address': '0000:0d:00.1',
+                    'vendor_id': '15b3',
+                    'product_id': '101c',
+                    'status': 'available',
+                    'request_id': None,
+                    'dev_type': fields.PciDeviceType.SRIOV_VF,
+                    'parent_addr': '0000:0d:00.0',
+                    'numa_node': 0}))
+
+        # For testing explicit remote_managed == False tagging.
+        self.locally_managed_netdevs.append(
+            objects.PciDevice.create(
+                None, {
+                    'compute_node_id': 1,
+                    'address': '0000:0e:00.1',
+                    'vendor_id': '15b3',
+                    'product_id': '1018',
+                    'status': 'available',
+                    'request_id': None,
+                    'dev_type': fields.PciDeviceType.SRIOV_VF,
+                    'parent_addr': '0000:0e:00.0',
+                    'numa_node': 0}))
+
         for dev in self.pci_tagged_devices:
             self.pci_stats.add_device(dev)
 
         for dev in self.pci_untagged_devices:
             self.pci_stats.add_device(dev)
 
+        for dev in self.remote_managed_netdevs:
+            self.pci_stats.add_device(dev)
+
+        for dev in self.locally_managed_netdevs:
+            self.pci_stats.add_device(dev)
+
     def _assertPoolContent(self, pool, vendor_id, product_id, count, **tags):
         self.assertEqual(vendor_id, pool['vendor_id'])
         self.assertEqual(product_id, pool['product_id'])
@@ -489,9 +602,10 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
         # Pools are ordered based on the number of keys. 'product_id',
         # 'vendor_id' are always part of the keys. When tags are present,
         # they are also part of the keys. In this test class, we have
-        # two pools with the second one having the tag 'physical_network'
-        # and the value 'physnet1'
-        self.assertEqual(2, len(self.pci_stats.pools))
+        # 5 pools with the second one having the tag 'physical_network'
+        # and the value 'physnet1' and multiple pools for testing
+        # variations of explicit/implicit remote_managed tagging.
+        self.assertEqual(5, len(self.pci_stats.pools))
         self._assertPoolContent(self.pci_stats.pools[0], '1137', '0072',
                                 len(self.pci_untagged_devices))
         self.assertEqual(self.pci_untagged_devices,
@@ -501,6 +615,19 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
                                 physical_network='physnet1')
         self.assertEqual(self.pci_tagged_devices,
                          self.pci_stats.pools[1]['devices'])
+        self._assertPoolContent(self.pci_stats.pools[2], '15b3', '101e',
+                                len(self.remote_managed_netdevs),
+                                remote_managed='true')
+        self.assertEqual(self.remote_managed_netdevs,
+                         self.pci_stats.pools[2]['devices'])
+        self._assertPoolContent(self.pci_stats.pools[3], '15b3', '101c', 1,
+                                remote_managed='false')
+        self.assertEqual([self.locally_managed_netdevs[0]],
+                         self.pci_stats.pools[3]['devices'])
+        self._assertPoolContent(self.pci_stats.pools[4], '15b3', '1018', 1,
+                                remote_managed='false')
+        self.assertEqual([self.locally_managed_netdevs[1]],
+                         self.pci_stats.pools[4]['devices'])
 
     def test_add_devices(self):
         self._create_pci_devices()
@@ -512,14 +639,32 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
                             spec=[{'physical_network': 'physnet1'}]),
                         objects.InstancePCIRequest(count=1,
                             spec=[{'vendor_id': '1137',
-                                   'product_id': '0072'}])]
+                                   'product_id': '0072'}]),
+                        objects.InstancePCIRequest(count=1,
+                            spec=[{'vendor_id': '15b3',
+                                   'product_id': '101e',
+                                   PCI_REMOTE_MANAGED_TAG: 'True'}]),
+                        objects.InstancePCIRequest(count=1,
+                            spec=[{'vendor_id': '15b3',
+                                   'product_id': '101c',
+                                   PCI_REMOTE_MANAGED_TAG: 'False'}]),
+                        objects.InstancePCIRequest(count=1,
+                            spec=[{'vendor_id': '15b3',
+                                   'product_id': '1018',
+                                   PCI_REMOTE_MANAGED_TAG: 'False'}])]
         devs = self.pci_stats.consume_requests(pci_requests)
-        self.assertEqual(2, len(devs))
-        self.assertEqual(set(['0071', '0072']),
+        self.assertEqual(5, len(devs))
+        self.assertEqual(set(['0071', '0072', '1018', '101e', '101c']),
                          set([dev.product_id for dev in devs]))
         self._assertPoolContent(self.pci_stats.pools[0], '1137', '0072', 2)
         self._assertPoolContent(self.pci_stats.pools[1], '1137', '0071', 3,
                                 physical_network='physnet1')
+        self._assertPoolContent(self.pci_stats.pools[2], '15b3', '101e', 0,
+                                remote_managed='true')
+        self._assertPoolContent(self.pci_stats.pools[3], '15b3', '101c', 0,
+                                remote_managed='false')
+        self._assertPoolContent(self.pci_stats.pools[4], '15b3', '1018', 0,
+                                remote_managed='false')
 
     def test_add_device_no_devspec(self):
         self._create_pci_devices()
@@ -569,7 +714,7 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
         dev1 = self.pci_tagged_devices.pop()
         dev1.dev_type = 'type-PF'
         self.pci_stats.update_device(dev1)
-        self.assertEqual(3, len(self.pci_stats.pools))
+        self.assertEqual(6, len(self.pci_stats.pools))
         self._assertPoolContent(self.pci_stats.pools[0], '1137', '0072',
                                 len(self.pci_untagged_devices))
         self.assertEqual(self.pci_untagged_devices,
@@ -579,56 +724,152 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
                                 physical_network='physnet1')
         self.assertEqual(self.pci_tagged_devices,
                          self.pci_stats.pools[1]['devices'])
-        self._assertPoolContent(self.pci_stats.pools[2], '1137', '0071',
+        self._assertPoolContent(self.pci_stats.pools[5], '1137', '0071',
                                 1,
-                                physical_network='physnet1')
+                                physical_network='physnet1',
+                                remote_managed='false')
         self.assertEqual(dev1,
-                         self.pci_stats.pools[2]['devices'][0])
+                         self.pci_stats.pools[5]['devices'][0])
 
 
 class PciDeviceVFPFStatsTestCase(test.NoDBTestCase):
 
     def setUp(self):
         super(PciDeviceVFPFStatsTestCase, self).setUp()
-        white_list = ['{"vendor_id":"8086","product_id":"1528"}',
-                      '{"vendor_id":"8086","product_id":"1515"}']
-        self.flags(passthrough_whitelist=white_list, group='pci')
-        self.pci_stats = stats.PciDeviceStats()
+        device_spec = [
+            jsonutils.dumps({"vendor_id": "8086", "product_id": "1528"}),
+            jsonutils.dumps({"vendor_id": "8086", "product_id": "1515"}),
+            jsonutils.dumps(
+                {
+                    "vendor_id": "15b3",
+                    "product_id": "a2d6",
+                    "remote_managed": "false",
+                }
+            ),
+            jsonutils.dumps(
+                {
+                    "vendor_id": "15b3",
+                    "product_id": "101e",
+                    "remote_managed": "true",
+                }
+            ),
+        ]
+        self.flags(device_spec=device_spec, group='pci')
+        self.pci_stats = stats.PciDeviceStats(objects.NUMATopology())
 
     def _create_pci_devices(self, vf_product_id=1515, pf_product_id=1528):
         self.sriov_pf_devices = []
         for dev in range(2):
-            pci_dev = {'compute_node_id': 1,
-                       'address': '0000:81:00.%d' % dev,
-                       'vendor_id': '8086',
-                       'product_id': '%d' % pf_product_id,
-                       'status': 'available',
-                       'request_id': None,
-                       'dev_type': fields.PciDeviceType.SRIOV_PF,
-                       'parent_addr': None,
-                       'numa_node': 0}
+            pci_dev = {
+                'compute_node_id': 1,
+                'address': '0000:81:00.%d' % dev,
+                'vendor_id': '8086',
+                'product_id': '%d' % pf_product_id,
+                'status': 'available',
+                'request_id': None,
+                'dev_type': fields.PciDeviceType.SRIOV_PF,
+                'parent_addr': None,
+                'numa_node': 0
+            }
             dev_obj = objects.PciDevice.create(None, pci_dev)
             dev_obj.child_devices = []
             self.sriov_pf_devices.append(dev_obj)
 
+        # PF devices for remote_managed VFs.
+        self.sriov_pf_devices_remote = []
+        for dev in range(2):
+            pci_dev = {
+                'compute_node_id': 1,
+                'address': '0001:81:00.%d' % dev,
+                'vendor_id': '15b3',
+                'product_id': 'a2d6',
+                'status': 'available',
+                'request_id': None,
+                'dev_type': fields.PciDeviceType.SRIOV_PF,
+                'parent_addr': None,
+                'numa_node': 0,
+                "capabilities": {"vpd": {
+                    "card_serial_number": "MT2113X00000"}},
+            }
+            dev_obj = objects.PciDevice.create(None, pci_dev)
+            dev_obj.child_devices = []
+            self.sriov_pf_devices_remote.append(dev_obj)
+
         self.sriov_vf_devices = []
         for dev in range(8):
-            pci_dev = {'compute_node_id': 1,
-                       'address': '0000:81:10.%d' % dev,
-                       'vendor_id': '8086',
-                       'product_id': '%d' % vf_product_id,
-                       'status': 'available',
-                       'request_id': None,
-                       'dev_type': fields.PciDeviceType.SRIOV_VF,
-                       'parent_addr': '0000:81:00.%d' % int(dev / 4),
-                       'numa_node': 0}
+            pci_dev = {
+                'compute_node_id': 1,
+                'address': '0000:81:10.%d' % dev,
+                'vendor_id': '8086',
+                'product_id': '%d' % vf_product_id,
+                'status': 'available',
+                'request_id': None,
+                'dev_type': fields.PciDeviceType.SRIOV_VF,
+                'parent_addr': '0000:81:00.%d' % int(dev / 4),
+                'numa_node': 0
+            }
             dev_obj = objects.PciDevice.create(None, pci_dev)
             dev_obj.parent_device = self.sriov_pf_devices[int(dev / 4)]
             dev_obj.parent_device.child_devices.append(dev_obj)
             self.sriov_vf_devices.append(dev_obj)
 
+        self.sriov_vf_devices_remote = []
+        for dev in range(8):
+            pci_dev = {
+                'compute_node_id': 1,
+                'address': '0001:81:10.%d' % dev,
+                'vendor_id': '15b3',
+                'product_id': '101e',
+                'status': 'available',
+                'request_id': None,
+                'dev_type': fields.PciDeviceType.SRIOV_VF,
+                'parent_addr': '0001:81:00.%d' % int(dev / 4),
+                'numa_node': 0,
+                "capabilities": {"vpd": {"card_serial_number": "MT2113X00000"}}
+            }
+            dev_obj = objects.PciDevice.create(None, pci_dev)
+            dev_obj.parent_device = self.sriov_pf_devices_remote[int(dev / 4)]
+            dev_obj.parent_device.child_devices.append(dev_obj)
+            self.sriov_vf_devices_remote.append(dev_obj)
+
+        self.vdpa_devices = []
+        for dev in range(8):
+            pci_dev = {
+                'compute_node_id': 1,
+                'address': '0000:82:10.%d' % dev,
+                'vendor_id': '8086',
+                'product_id': '%d' % vf_product_id,
+                'status': 'available',
+                'request_id': None,
+                'dev_type': fields.PciDeviceType.VDPA,
+                'parent_addr': '0000:81:00.%d' % int(dev / 4),
+                'numa_node': 0
+            }
+            dev_obj = objects.PciDevice.create(None, pci_dev)
+            dev_obj.parent_device = self.sriov_pf_devices[int(dev / 4)]
+            dev_obj.parent_device.child_devices.append(dev_obj)
+            self.vdpa_devices.append(dev_obj)
+
         list(map(self.pci_stats.add_device, self.sriov_pf_devices))
         list(map(self.pci_stats.add_device, self.sriov_vf_devices))
+        list(map(self.pci_stats.add_device, self.vdpa_devices))
+        list(map(self.pci_stats.add_device, self.sriov_pf_devices_remote))
+        list(map(self.pci_stats.add_device, self.sriov_vf_devices_remote))
+
+    def test_consume_VDPA_requests(self):
+        self._create_pci_devices()
+        pci_requests = [
+            objects.InstancePCIRequest(
+                count=8, spec=[{'dev_type': 'vdpa'}])]
+        devs = self.pci_stats.consume_requests(pci_requests)
+        self.assertEqual(8, len(devs))
+        self.assertEqual('vdpa', devs[0].dev_type)
+        free_devs = self.pci_stats.get_free_devs()
+        # Validate that the parents of these devs has been removed
+        # from pools.
+        for dev in devs:
+            self.assertNotIn(dev.parent_addr,
+                             [free_dev.address for free_dev in free_devs])
 
     def test_consume_VF_requests(self):
         self._create_pci_devices()
@@ -657,7 +898,8 @@ class PciDeviceVFPFStatsTestCase(test.NoDBTestCase):
         free_devs = self.pci_stats.get_free_devs()
         # Validate that there are no free devices left, as when allocating
         # both available PFs, its VFs should not be available.
-        self.assertEqual(0, len(free_devs))
+        self.assertEqual(0, len([d for d in free_devs
+                                 if d.product_id == '1515']))
 
     def test_consume_VF_and_PF_requests(self):
         self._create_pci_devices()
@@ -678,10 +920,85 @@ class PciDeviceVFPFStatsTestCase(test.NoDBTestCase):
                         objects.InstancePCIRequest(count=1,
                             spec=[{'product_id': '1528',
                                     'dev_type': 'type-PF'}])]
-        self.assertIsNone(self.pci_stats.consume_requests(pci_requests))
+        self.assertRaises(
+            exception.PciDeviceRequestFailed,
+            self.pci_stats.consume_requests,
+            pci_requests,
+        )
 
-    def test_consume_VF_and_PF_same_prodict_id_failed(self):
+    def test_consume_VF_and_PF_same_product_id_failed(self):
         self._create_pci_devices(pf_product_id=1515)
         pci_requests = [objects.InstancePCIRequest(count=9,
                             spec=[{'product_id': '1515'}])]
-        self.assertIsNone(self.pci_stats.consume_requests(pci_requests))
+        self.assertRaises(
+            exception.PciDeviceRequestFailed,
+            self.pci_stats.consume_requests,
+            pci_requests,
+        )
+
+    def test_consume_PF_not_remote_managed(self):
+        self._create_pci_devices()
+        pci_requests = [objects.InstancePCIRequest(count=2,
+                            spec=[{'product_id': '1528',
+                                   'dev_type': 'type-PF',
+                                   PCI_REMOTE_MANAGED_TAG: 'false'}])]
+        devs = self.pci_stats.consume_requests(pci_requests)
+        self.assertEqual(2, len(devs))
+        self.assertEqual(set(['1528']),
+                            set([dev.product_id for dev in devs]))
+        free_devs = self.pci_stats.get_free_devs()
+        # Validate that there are no free devices left with the
+        # product ID under test, as when allocating both available
+        # PFs, its VFs should not be available.
+        self.assertEqual(0, len([d for d in free_devs
+                                 if d.product_id == '1528']))
+
+    def test_consume_VF_requests_remote_managed(self):
+        self._create_pci_devices()
+        pci_requests = [objects.InstancePCIRequest(count=2,
+                            spec=[{PCI_REMOTE_MANAGED_TAG: 'true'}])]
+        devs = self.pci_stats.consume_requests(pci_requests)
+        self.assertEqual(2, len(devs))
+        self.assertEqual(set(['101e']),
+                            set([dev.product_id for dev in devs]))
+        free_devs = self.pci_stats.get_free_devs()
+        # Validate that the parents of these VFs has been removed
+        # from pools.
+        for dev in devs:
+            self.assertNotIn(dev.parent_addr,
+                             [free_dev.address for free_dev in free_devs])
+
+    def test_consume_VF_requests_remote_managed_filtered(self):
+        self._create_pci_devices()
+        pci_requests = [objects.InstancePCIRequest(count=1,
+                            spec=[{'product_id': '101e',
+                                   PCI_REMOTE_MANAGED_TAG: 'false'}]),
+                        objects.InstancePCIRequest(count=1,
+                            spec=[{'product_id': '101e'}])]
+        free_devs_before = self.pci_stats.get_free_devs()
+        self.assertRaises(
+            exception.PciDeviceRequestFailed,
+            self.pci_stats.consume_requests,
+            pci_requests,
+        )
+        free_devs_after = self.pci_stats.get_free_devs()
+        self.assertEqual(free_devs_before, free_devs_after)
+
+    def test_consume_VF_requests_remote_managed_mix(self):
+        self._create_pci_devices()
+        pci_requests = [objects.InstancePCIRequest(count=1,
+                            spec=[{'product_id': '101e',
+                                   PCI_REMOTE_MANAGED_TAG: 'true'}]),
+                        objects.InstancePCIRequest(count=1,
+                            spec=[{'product_id': '1515',
+                                   PCI_REMOTE_MANAGED_TAG: 'false'}])]
+        devs = self.pci_stats.consume_requests(pci_requests)
+        self.assertEqual(2, len(devs))
+        self.assertEqual(set(['101e', '1515']),
+                            set([dev.product_id for dev in devs]))
+        free_devs = self.pci_stats.get_free_devs()
+        # Validate that the parents of these VFs has been removed
+        # from pools.
+        for dev in devs:
+            self.assertNotIn(dev.parent_addr,
+                             [free_dev.address for free_dev in free_devs])
