@@ -13,8 +13,8 @@
 #    under the License.
 
 import textwrap
+from unittest import mock
 
-import mock
 import pycodestyle
 
 from nova.hacking import checks
@@ -51,6 +51,7 @@ class HackingTestCase(test.NoDBTestCase):
     just assertTrue if the check is expected to fail and assertFalse if it
     should pass.
     """
+
     def test_virt_driver_imports(self):
 
         expect = (0, "N311: importing code from other virt drivers forbidden")
@@ -897,7 +898,7 @@ class HackingTestCase(test.NoDBTestCase):
             expected_errors=errors, filename="nova/tests/unit/test_context.py")
         # Check no errors in other than 'nova/tests' directory.
         self._assert_has_no_errors(
-            code, checks.nonexistent_assertion_methods_and_attributes,
+            code, checks.useless_assertion,
             filename="nova/compute/api.py")
         code = """
                    self.assertIsNone(None_test_var, "Fails")
@@ -909,3 +910,136 @@ class HackingTestCase(test.NoDBTestCase):
         self._assert_has_no_errors(
             code, checks.useless_assertion,
             filename="nova/tests/unit/test_context.py")
+
+    def test_check_assert_has_calls(self):
+        code = """
+                   mock_method.assert_has_calls = [mock.call(1)]
+                   mock_method2.assert_has_calls = [
+                       mock.call(1), mock.call(2)]
+               """
+        errors = [(x + 1, 0, 'N366') for x in range(2)]
+        # Check errors in 'nova/tests' directory.
+        self._assert_has_errors(
+            code, checks.check_assert_has_calls,
+            expected_errors=errors, filename="nova/tests/unit/test_context.py")
+        # Check no errors in other than 'nova/tests' directory.
+        self._assert_has_no_errors(
+            code, checks.check_assert_has_calls,
+            filename="nova/compute/api.py")
+        code = """
+                   mock_method.assert_has_calls([mock.call(1)])
+                   mock_method2.assert_has_calls([
+                       mock.call(1), mock.call(2)])
+               """
+        self._assert_has_no_errors(
+            code, checks.check_assert_has_calls,
+            filename="nova/tests/unit/test_context.py")
+
+    def test_do_not_alias_mock_class(self):
+        code = """
+                    my_var = mock.Mock
+                    self.mock_rados.Rados = mock.Mock
+                    self.mock_rados.Rados = mock.MagicMock
+                    self.mock_rados.Rados = mock.NonCallableMock
+               """
+        errors = [(x + 1, 0, 'N367') for x in range(4)]
+        # Check errors in 'nova/tests' directory.
+        self._assert_has_errors(
+            code, checks.do_not_alias_mock_class,
+            expected_errors=errors, filename="nova/tests/unit/test_context.py")
+        # Check no errors in other than 'nova/tests' directory.
+        self._assert_has_no_errors(
+            code, checks.do_not_alias_mock_class,
+            filename="nova/compute/api.py")
+        code = """
+                    my_var = mock.Mock()
+                    self.mock_rados.Rados = mock.Mock()
+                    self.mock_rados.Rados = mock.MagicMock()
+                    self.mock_rados.Rados = mock.NonCallableMock()
+               """
+        self._assert_has_no_errors(
+            code, checks.do_not_alias_mock_class,
+            filename="nova/tests/unit/test_context.py")
+
+    def test_do_not_use_mock_class_as_new_mock_value(self):
+        code = """
+                    patcher = mock.patch('Bar.foo', new=mock.Mock)
+                    patcher = mock.patch.object(Bar, 'foo', new=mock.Mock)
+                    patcher = mock.patch('Bar.foo', new=mock.MagicMock)
+                    patcher = mock.patch('Bar.foo', new=mock.NonCallableMock)
+                    @mock.patch('Bar.foo', new=mock.Mock)
+                    def a():
+                        pass
+
+                    with mock.patch('Bar.foo', new=mock.Mock) as m:
+                        pass
+               """
+        errors = [(x + 1, 0, 'N368') for x in range(5)] + [(9, 0, 'N368')]
+        # Check errors in 'nova/tests' directory.
+        self._assert_has_errors(
+            code, checks.do_not_use_mock_class_as_new_mock_value,
+            expected_errors=errors, filename="nova/tests/unit/test_context.py")
+        # Check no errors in other than 'nova/tests' directory.
+        self._assert_has_no_errors(
+            code, checks.do_not_use_mock_class_as_new_mock_value,
+            filename="nova/compute/api.py")
+        code = """
+                    patcher = mock.patch('Bar.foo', new=mock.Mock())
+                    patcher = mock.patch.object(Bar, 'foo', new=mock.Mock())
+                    patcher = mock.patch('Bar.foo', new=mock.MagicMock())
+                    patcher = mock.patch('Bar.foo', new=mock.NonCallableMock())
+                    @mock.patch('Bar.foo', new=mock.Mock())
+                    def a():
+                        pass
+
+                    with mock.patch('Bar.foo', new=mock.Mock()) as m:
+                        pass
+
+                    patcher = mock.patch('Bar.foo', new_callable=mock.Mock)
+               """
+        self._assert_has_no_errors(
+            code, checks.do_not_use_mock_class_as_new_mock_value,
+            filename="nova/tests/unit/test_context.py")
+
+    def test_check_lockutils_rwlocks(self):
+        code = """
+                    lockutils.ReaderWriterLock()
+                    lockutils.ReaderWriterLock(condition_cls=MyClass)
+                    oslo_concurrency.lockutils.ReaderWriterLock()
+                    fasteners.ReaderWriterLock()
+                    fasteners.ReaderWriterLock(condition_cls=MyClass)
+               """
+        errors = [(x + 1, 0, 'N369') for x in range(5)]
+        self._assert_has_errors(
+            code, checks.check_lockutils_rwlocks, expected_errors=errors)
+
+        code = """
+                    nova.utils.ReaderWriterLock()
+                    utils.ReaderWriterLock()
+                    utils.ReaderWriterLock(condition_cls=MyClass)
+                    nova_utils.ReaderWriterLock()
+               """
+        self._assert_has_no_errors(code, checks.check_lockutils_rwlocks)
+
+    def test_check_six(self):
+        code = """
+            import six
+            from six import moves
+            from six.moves import range
+            import six.moves.urllib.parse as urlparse
+        """
+        errors = [(x + 1, 0, 'N370') for x in range(4)]
+        self._assert_has_errors(code, checks.check_six, expected_errors=errors)
+
+    def test_import_stock_mock(self):
+        self._assert_has_errors(
+            "import mock",
+            checks.import_stock_mock, expected_errors=[(1, 0, 'N371')])
+        self._assert_has_errors(
+            "from mock import patch",
+            checks.import_stock_mock, expected_errors=[(1, 0, 'N371')])
+        code = """
+                    from unittest import mock
+                    import unittest.mock
+               """
+        self._assert_has_no_errors(code, checks.import_stock_mock)
