@@ -20,16 +20,12 @@ import copy
 import hashlib
 import hmac
 import os
+import pickle
 import re
-
-try:  # python 2
-    import pickle
-except ImportError:  # python 3
-    import cPickle as pickle
+from unittest import mock
 
 from keystoneauth1 import exceptions as ks_exceptions
 from keystoneauth1 import session
-import mock
 from oslo_config import cfg
 from oslo_serialization import base64
 from oslo_serialization import jsonutils
@@ -1204,7 +1200,7 @@ class MetadataHandlerTestCase(test.TestCase):
 
     def _fake_x_get_metadata(self, self_app, instance_id, remote_address):
         if remote_address is None:
-            raise Exception('Expected X-Forwared-For header')
+            raise Exception('Expected X-Forwarded-For header')
 
         if encodeutils.to_utf8(instance_id) == self.expected_instance_id:
             return self.mdinst
