@@ -16,22 +16,19 @@
 """Test of Policy Engine For Nova."""
 
 import os.path
+from unittest import mock
 
-import fixtures
-import mock
-from oslo_config import cfg
 from oslo_policy import policy as oslo_policy
 from oslo_serialization import jsonutils
 import requests_mock
-import yaml
 
 from nova import context
 from nova import exception
 from nova.policies import servers as servers_policy
 from nova import policy
 from nova import test
+from nova.tests import fixtures as nova_fixtures
 from nova.tests.unit import fake_policy
-from nova.tests.unit import policy_fixture
 from nova import utils
 
 
@@ -288,7 +285,7 @@ class IsAdminCheckTestCase(test.NoDBTestCase):
 class AdminRolePolicyTestCase(test.NoDBTestCase):
     def setUp(self):
         super(AdminRolePolicyTestCase, self).setUp()
-        self.policy = self.useFixture(policy_fixture.RoleBasedPolicyFixture())
+        self.policy = self.useFixture(nova_fixtures.RoleBasedPolicyFixture())
         self.context = context.RequestContext('fake', 'fake', roles=['member'])
         self.actions = policy.get_rules().keys()
         self.target = {}
@@ -305,11 +302,11 @@ class AdminRolePolicyTestCase(test.NoDBTestCase):
 class RealRolePolicyTestCase(test.NoDBTestCase):
     def setUp(self):
         super(RealRolePolicyTestCase, self).setUp()
-        self.policy = self.useFixture(policy_fixture.RealPolicyFixture())
+        self.policy = self.useFixture(nova_fixtures.RealPolicyFixture())
         self.non_admin_context = context.RequestContext('fake', 'fake',
                                                         roles=['member'])
         self.admin_context = context.RequestContext('fake', 'fake', True,
-                                                     roles=['member'])
+                                                     roles=['admin', 'member'])
         self.target = {}
         self.fake_policy = jsonutils.loads(fake_policy.policy_data)
 
@@ -326,7 +323,6 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:servers:show:host_status:unknown-only",
 "os_compute_api:servers:migrations:force_complete",
 "os_compute_api:servers:migrations:delete",
-"os_compute_api:os-admin-actions:reset_network",
 "os_compute_api:os-admin-actions:inject_network_info",
 "os_compute_api:os-admin-actions:reset_state",
 "os_compute_api:os-aggregates:index",
@@ -337,9 +333,6 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-aggregates:add_host",
 "os_compute_api:os-aggregates:remove_host",
 "os_compute_api:os-aggregates:set_metadata",
-"os_compute_api:os-agents:create",
-"os_compute_api:os-agents:update",
-"os_compute_api:os-agents:delete",
 "os_compute_api:os-evacuate",
 "os_compute_api:os-extended-server-attributes",
 "os_compute_api:os-flavor-access:remove_tenant_access",
@@ -365,6 +358,7 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-services:update",
 "os_compute_api:os-services:delete",
 "os_compute_api:os-shelve:shelve_offload",
+"os_compute_api:os-shelve:unshelve_to_host",
 "os_compute_api:os-availability-zone:detail",
 "os_compute_api:os-assisted-volume-snapshots:create",
 "os_compute_api:os-assisted-volume-snapshots:delete",
@@ -373,6 +367,26 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-server-external-events:create",
 "os_compute_api:os-volumes-attachments:swap",
 "os_compute_api:servers:create:zero_disk_flavor",
+"os_compute_api:os-baremetal-nodes:list",
+"os_compute_api:os-baremetal-nodes:show",
+"os_compute_api:servers:migrations:index",
+"os_compute_api:servers:migrations:show",
+"os_compute_api:os-simple-tenant-usage:list",
+"os_compute_api:os-migrations:index",
+"os_compute_api:os-services:list",
+"os_compute_api:os-instance-actions:events:details",
+"os_compute_api:os-instance-usage-audit-log:list",
+"os_compute_api:os-instance-usage-audit-log:show",
+"os_compute_api:os-hosts:list",
+"os_compute_api:os-hosts:show",
+"os_compute_api:os-hypervisors:list",
+"os_compute_api:os-hypervisors:list-detail",
+"os_compute_api:os-hypervisors:show",
+"os_compute_api:os-hypervisors:statistics",
+"os_compute_api:os-hypervisors:uptime",
+"os_compute_api:os-hypervisors:search",
+"os_compute_api:os-hypervisors:servers",
+"os_compute_api:limits:other_project",
 )
 
         self.admin_or_owner_rules = (
@@ -416,6 +430,7 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:servers:resize",
 "os_compute_api:servers:revert_resize",
 "os_compute_api:servers:show",
+"os_compute_api:servers:show:flavor-extra-specs",
 "os_compute_api:servers:update",
 "os_compute_api:servers:create_image:allow_volume_backed",
 "os_compute_api:os-admin-password",
@@ -461,45 +476,6 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-volumes-attachments:create",
 "os_compute_api:os-volumes-attachments:delete",
 "os_compute_api:os-volumes-attachments:update",
-)
-
-        self.allow_all_rules = (
-"os_compute_api:os-quota-sets:defaults",
-"os_compute_api:os-availability-zone:list",
-"os_compute_api:limits",
-"os_compute_api:extensions",
-"os_compute_api:os-floating-ip-pools",
-)
-
-        self.system_reader_rules = (
-"os_compute_api:os-tenant-networks:list",
-"os_compute_api:os-tenant-networks:show",
-"os_compute_api:os-baremetal-nodes:list",
-"os_compute_api:os-baremetal-nodes:show",
-"os_compute_api:servers:migrations:index",
-"os_compute_api:servers:migrations:show",
-"os_compute_api:os-simple-tenant-usage:list",
-"os_compute_api:os-migrations:index",
-"os_compute_api:os-services:list",
-"os_compute_api:os-instance-actions:events:details",
-"os_compute_api:os-instance-usage-audit-log:list",
-"os_compute_api:os-instance-usage-audit-log:show",
-"os_compute_api:os-agents:list",
-"os_compute_api:os-hosts:list",
-"os_compute_api:os-hosts:show",
-"os_compute_api:os-hypervisors:list",
-"os_compute_api:os-hypervisors:list-detail",
-"os_compute_api:os-hypervisors:show",
-"os_compute_api:os-hypervisors:statistics",
-"os_compute_api:os-hypervisors:uptime",
-"os_compute_api:os-hypervisors:search",
-"os_compute_api:os-hypervisors:servers",
-"os_compute_api:limits:other_project",
-"os_compute_api:os-networks:list",
-"os_compute_api:os-networks:show",
-)
-
-        self.system_reader_or_owner_rules = (
 "os_compute_api:os-simple-tenant-usage:show",
 "os_compute_api:os-security-groups:get",
 "os_compute_api:os-security-groups:show",
@@ -521,6 +497,18 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-volumes:snapshots:show",
 "os_compute_api:os-volumes:snapshots:list",
 "os_compute_api:os-volumes:snapshots:detail",
+"os_compute_api:os-networks:list",
+"os_compute_api:os-networks:show",
+"os_compute_api:os-tenant-networks:list",
+"os_compute_api:os-tenant-networks:show",
+)
+
+        self.allow_all_rules = (
+"os_compute_api:os-quota-sets:defaults",
+"os_compute_api:os-availability-zone:list",
+"os_compute_api:limits",
+"os_compute_api:extensions",
+"os_compute_api:os-floating-ip-pools",
 )
 
         self.allow_nobody_rules = (
@@ -565,80 +553,11 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
         # admin_only, non_admin, admin_or_user, empty_rule
         special_rules = ('admin_api', 'admin_or_owner', 'context_is_admin',
                          'os_compute_api:os-quota-class-sets:show',
-                         'system_admin_api', 'system_reader_api',
                          'project_admin_api', 'project_member_api',
-                         'project_reader_api', 'system_admin_or_owner',
-                         'system_or_project_reader')
+                         'project_reader_api', 'project_member_or_admin',
+                         'project_reader_or_admin')
         result = set(rules.keys()) - set(self.admin_only_rules +
             self.admin_or_owner_rules +
-            self.allow_all_rules + self.system_reader_rules +
-            self.system_reader_or_owner_rules +
+            self.allow_all_rules +
             self.allow_nobody_rules + special_rules)
         self.assertEqual(set([]), result)
-
-
-class PickPolicyFileTestCase(test.NoDBTestCase):
-
-    def setUp(self):
-        super(PickPolicyFileTestCase, self).setUp()
-        self.data = {
-            'rule_admin': 'True',
-            'rule_admin2': 'is_admin:True'
-        }
-        policy.CONF.clear_override('policy_file', group='oslo_policy')
-        self.tmpdir = self.useFixture(fixtures.TempDir())
-        original_search_dirs = cfg._search_dirs
-
-        def fake_search_dirs(dirs, name):
-            dirs.append(self.tmpdir.path)
-            return original_search_dirs(dirs, name)
-
-        self.stub_out('oslo_config.cfg._search_dirs', fake_search_dirs)
-
-    def test_non_config_policy_file(self):
-        tmpfilename = 'nova-policy.yaml'
-        self.flags(policy_file=tmpfilename, group='oslo_policy')
-        selected_policy_file = policy.pick_policy_file(
-            policy_file='non-config-file')
-        self.assertEqual(policy.CONF.oslo_policy.policy_file, tmpfilename)
-        self.assertEqual(selected_policy_file, 'non-config-file')
-
-    def test_overridden_policy_file(self):
-        tmpfilename = 'nova-policy.yaml'
-        self.flags(policy_file=tmpfilename, group='oslo_policy')
-        selected_policy_file = policy.pick_policy_file(policy_file=None)
-        self.assertEqual(policy.CONF.oslo_policy.policy_file, tmpfilename)
-        self.assertEqual(selected_policy_file, tmpfilename)
-
-    def test_only_new_default_policy_file_exist(self):
-        tmpfilename = os.path.join(self.tmpdir.path, 'policy.yaml')
-        with open(tmpfilename, 'w') as fh:
-            yaml.dump(self.data, fh)
-
-        selected_policy_file = policy.pick_policy_file(policy_file=None)
-        self.assertEqual(policy.CONF.oslo_policy.policy_file, 'policy.yaml')
-        self.assertEqual(selected_policy_file, 'policy.yaml')
-
-    @mock.patch.object(policy.CONF, 'get_location')
-    def test_only_old_default_policy_file_exist(self, mock_get):
-        mock_get.return_value = cfg.LocationInfo(cfg.Locations.set_default,
-                                                 'None')
-        tmpfilename = os.path.join(self.tmpdir.path, 'policy.json')
-        with open(tmpfilename, 'w') as fh:
-            jsonutils.dump(self.data, fh)
-
-        selected_policy_file = policy.pick_policy_file(policy_file=None)
-        self.assertEqual(policy.CONF.oslo_policy.policy_file, 'policy.yaml')
-        self.assertEqual(selected_policy_file, 'policy.json')
-
-    def test_both_default_policy_file_exist(self):
-        tmpfilename1 = os.path.join(self.tmpdir.path, 'policy.json')
-        with open(tmpfilename1, 'w') as fh:
-            jsonutils.dump(self.data, fh)
-        tmpfilename2 = os.path.join(self.tmpdir.path, 'policy.yaml')
-        with open(tmpfilename2, 'w') as fh:
-            yaml.dump(self.data, fh)
-
-        selected_policy_file = policy.pick_policy_file(policy_file=None)
-        self.assertEqual(policy.CONF.oslo_policy.policy_file, 'policy.yaml')
-        self.assertEqual(selected_policy_file, 'policy.yaml')
