@@ -18,12 +18,11 @@
 
 import os.path
 import tempfile
+from unittest import mock
 import uuid
 
-import mock
 from oslo_log import log as logging
 import oslo_messaging as messaging
-import six
 
 import nova.conf
 from nova import exception
@@ -43,6 +42,7 @@ class IsolationTestCase(test.TestCase):
     of other tests should fail.
 
     """
+
     def test_service_isolation(self):
         self.useFixture(fixtures.ServiceFixture('compute'))
 
@@ -125,10 +125,10 @@ class JsonTestCase(test.NoDBTestCase):
                 e.difference)
             self.assertIn(
                 "actual:\n{'top': {'l1': {'l2': ['c', 'a', 'b', 'd']}}}",
-                six.text_type(e))
+                str(e))
             self.assertIn(
                 "expected:\n{'top': {'l1': {'l2': ['a', 'b', 'c']}}}",
-                six.text_type(e))
+                str(e))
         else:
             self.fail("This should have raised a mismatch exception")
 
@@ -206,10 +206,10 @@ class JsonTestCase(test.NoDBTestCase):
                 e.difference)
             self.assertIn(
                 "actual:\n{'top': {'l1': {'l2': ['c', 'a', 'd']}}}",
-                six.text_type(e))
+                str(e))
             self.assertIn(
                 "expected:\n{'top': {'l1': {'l2': ['a', 'b', 'c']}}}",
-                six.text_type(e))
+                str(e))
         else:
             self.fail("This should have raised a mismatch exception")
 
@@ -233,11 +233,9 @@ class JsonTestCase(test.NoDBTestCase):
         except Exception as e:
             self.assertEqual(
                 "3 != 4: path: root.top.l1.l2.c", e.difference)
-            self.assertIn("actual:\n{'top': {'l1': {'l2': {", six.text_type(e))
-            self.assertIn(
-                "expected:\n{'top': {'l1': {'l2': {", six.text_type(e))
-            self.assertIn(
-                "message: test message\n", six.text_type(e))
+            self.assertIn("actual:\n{'top': {'l1': {'l2': {", str(e))
+            self.assertIn("expected:\n{'top': {'l1': {'l2': {", str(e))
+            self.assertIn("message: test message\n", str(e))
         else:
             self.fail("This should have raised a mismatch exception")
 
@@ -304,19 +302,20 @@ class ContainKeyValueTestCase(test.NoDBTestCase):
 
 class NovaExceptionReraiseFormatErrorTestCase(test.NoDBTestCase):
     """Test that format errors are reraised in tests."""
+
     def test_format_error_in_nova_exception(self):
         class FakeImageException(exception.NovaException):
             msg_fmt = 'Image %(image_id)s has wrong type %(type)s.'
         # wrong kwarg
         ex = self.assertRaises(KeyError, FakeImageException,
                                bogus='wrongkwarg')
-        self.assertIn('image_id', six.text_type(ex))
+        self.assertIn('image_id', str(ex))
         # no kwarg
         ex = self.assertRaises(KeyError, FakeImageException)
-        self.assertIn('image_id', six.text_type(ex))
+        self.assertIn('image_id', str(ex))
         # not enough kwargs
         ex = self.assertRaises(KeyError, FakeImageException, image_id='image')
-        self.assertIn('type', six.text_type(ex))
+        self.assertIn('type', str(ex))
 
 
 class PatchExistsTestCase(test.NoDBTestCase):
@@ -362,21 +361,6 @@ class PatchExistsTestCase(test.NoDBTestCase):
         self.assertTrue(os.path.exists(os.path.dirname(__file__)))
         self.assertFalse(os.path.exists('non-existent/file'))
 
-    @test.patch_exists('fake_file1', True)
-    @test.patch_exists('fake_file2', True)
-    @test.patch_exists(__file__, False)
-    def test_patch_exists_multiple_decorators(self):
-        """Test that @patch_exists can be used multiple times on the
-        same method.
-        """
-        self.assertTrue(os.path.exists('fake_file1'))
-        self.assertTrue(os.path.exists('fake_file2'))
-        self.assertFalse(os.path.exists(__file__))
-
-        # Check non-patched parameters
-        self.assertTrue(os.path.exists(os.path.dirname(__file__)))
-        self.assertFalse(os.path.exists('non-existent/file'))
-
 
 class PatchOpenTestCase(test.NoDBTestCase):
     fake_contents = "These file contents don't really exist"
