@@ -17,14 +17,12 @@ import errno
 import platform
 import socket
 import sys
-
-import mock
-from six.moves import range
+from unittest import mock
 
 from nova.compute import flavors
 import nova.conf
 import nova.context
-from nova.db import api as db
+from nova.db.main import api as db
 from nova import exception
 from nova.image import glance
 from nova.network import model as network_model
@@ -299,6 +297,7 @@ class ItemsMatcher(CustomMockCallMatcher):
     But the following will fail::
         my_mock(..., listy_kwarg=['foo', 'bar'], ...)
     """
+
     def __init__(self, iterable):
         # NOTE(gibi): we need the extra iter() call as Counter handles dicts
         # directly to initialize item count. However if a dict passed to
