@@ -14,32 +14,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import sys
+from unittest import mock
 
-import mock
 from oslo_service import fixture as service_fixture
 from oslo_utils import encodeutils
-import six
 
 from nova import context
-from nova import exception
 from nova import test
-from nova.tests.unit.virt.libvirt import fakelibvirt
+from nova.tests import fixtures as nova_fixtures
+from nova.tests.fixtures import libvirt as fakelibvirt
 from nova.virt.libvirt import config as vconfig
 from nova.virt.libvirt import guest as libvirt_guest
 from nova.virt.libvirt import host
 
 
-if sys.version_info > (3,):
-    long = int
-
-
 class GuestTestCase(test.NoDBTestCase):
 
     def setUp(self):
         super(GuestTestCase, self).setUp()
 
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
         self.host = host.Host("qemu:///system")
         self.context = context.get_admin_context()
 
@@ -80,7 +74,7 @@ class GuestTestCase(test.NoDBTestCase):
         self.domain.createWithFlags.assert_called_once_with(
             fakelibvirt.VIR_DOMAIN_START_PAUSED)
 
-    @mock.patch.object(libvirt_guest.LOG, 'error')
+    @mock.patch.object(libvirt_guest.LOG, 'exception')
     @mock.patch.object(encodeutils, 'safe_decode')
     def test_launch_exception(self, mock_safe_decode, mock_log):
         fake_xml = '<test>this is a test</test>'
@@ -214,207 +208,6 @@ class GuestTestCase(test.NoDBTestCase):
             "</xml>", flags=(fakelibvirt.VIR_DOMAIN_AFFECT_CONFIG |
                              fakelibvirt.VIR_DOMAIN_AFFECT_LIVE))
 
-    def test_detach_device_with_retry_from_transient_domain(self):
-        conf = mock.Mock(spec=vconfig.LibvirtConfigGuestDevice)
-        conf.to_xml.return_value = "</xml>"
-        get_config = mock.Mock()
-        get_config.side_effect = [conf, conf, conf, None, None]
-        dev_path = "/dev/vdb"
-        self.domain.isPersistent.return_value = False
-        retry_detach = self.guest.detach_device_with_retry(
-            get_config, dev_path, live=True, inc_sleep_time=.01)
-        self.domain.detachDeviceFlags.assert_called_once_with(
-            "</xml>", flags=fakelibvirt.VIR_DOMAIN_AFFECT_LIVE)
-        self.domain.detachDeviceFlags.reset_mock()
-        retry_detach()
-        self.assertEqual(1, self.domain.detachDeviceFlags.call_count)
-
-    def test_detach_device_with_retry_detach_success(self):
-        conf = mock.Mock(spec=vconfig.LibvirtConfigGuestDevice)
-        conf.to_xml.return_value = "</xml>"
-        get_config = mock.Mock()
-        # Force multiple retries of detach
-        get_config.side_effect = [conf, conf, conf, conf, conf, None, None]
-        dev_path = "/dev/vdb"
-        self.domain.isPersistent.return_value = True
-
-        retry_detach = self.guest.detach_device_with_retry(
-            get_config, dev_path, live=True, inc_sleep_time=.01)
-        # Ensure we've only done the initial detach call
-        self.domain.detachDeviceFlags.assert_called_once_with(
-            "</xml>", flags=(fakelibvirt.VIR_DOMAIN_AFFECT_CONFIG |
-                             fakelibvirt.VIR_DOMAIN_AFFECT_LIVE))
-
-        get_config.assert_called_with(dev_path)
-
-        # Some time later, we can do the wait/retry to ensure detach succeeds
-        self.domain.detachDeviceFlags.reset_mock()
-        retry_detach()
-        # Should have two retries before we pretend device is detached
-        self.assertEqual(2, self.domain.detachDeviceFlags.call_count)
-
-    def test_detach_device_with_retry_detach_failure(self):
-        conf = mock.Mock(spec=vconfig.LibvirtConfigGuestDevice)
-        conf.to_xml.return_value = "</xml>"
-        # Continue to return some value for the disk config
-        get_config = mock.Mock(return_value=conf)
-        self.domain.isPersistent.return_value = True
-
-        retry_detach = self.guest.detach_device_with_retry(
-            get_config, "/dev/vdb", live=True, inc_sleep_time=.01,
-            max_retry_count=3)
-        # Ensure we've only done the initial detach call
-        self.domain.detachDeviceFlags.assert_called_once_with(
-            "</xml>", flags=(fakelibvirt.VIR_DOMAIN_AFFECT_CONFIG |
-                             fakelibvirt.VIR_DOMAIN_AFFECT_LIVE))
-
-        # Some time later, we can do the wait/retry to ensure detach
-        self.domain.detachDeviceFlags.reset_mock()
-        # Should hit max # of retries
-        self.assertRaises(exception.DeviceDetachFailed, retry_detach)
-        self.assertEqual(4, self.domain.detachDeviceFlags.call_count)
-
-    def test_detach_device_with_retry_device_not_found(self):
-        get_config = mock.Mock(return_value=None)
-        self.domain.isPersistent.return_value = True
-        ex = self.assertRaises(
-            exception.DeviceNotFound, self.guest.detach_device_with_retry,
-            get_config, "/dev/vdb", live=True)
-        self.assertIn("/dev/vdb", six.text_type(ex))
-
-    def test_detach_device_with_retry_device_not_found_alt_name(self):
-        """Tests to make sure we use the alternative name in errors."""
-        get_config = mock.Mock(return_value=None)
-        self.domain.isPersistent.return_value = True
-        ex = self.assertRaises(
-            exception.DeviceNotFound, self.guest.detach_device_with_retry,
-            get_config, mock.sentinel.device, live=True,
-            alternative_device_name='foo')
-        self.assertIn('foo', six.text_type(ex))
-
-    @mock.patch.object(libvirt_guest.Guest, "detach_device")
-    def _test_detach_device_with_retry_second_detach_failure(
-            self, mock_detach, error_code=None, error_message=None,
-            supports_device_missing=False):
-        # This simulates a retry of the transient/live domain detach
-        # failing because the device is not found
-        conf = mock.Mock(spec=vconfig.LibvirtConfigGuestDevice)
-        conf.to_xml.return_value = "</xml>"
-        self.domain.isPersistent.return_value = True
-
-        get_config = mock.Mock(return_value=conf)
-        fake_device = "vdb"
-        fake_exc = fakelibvirt.make_libvirtError(
-            fakelibvirt.libvirtError, "",
-            error_message=error_message,
-            error_code=error_code,
-            error_domain=fakelibvirt.VIR_FROM_DOMAIN)
-        mock_detach.side_effect = [None, fake_exc]
-        retry_detach = self.guest.detach_device_with_retry(
-            get_config, fake_device, live=True,
-            inc_sleep_time=.01, max_retry_count=3,
-            supports_device_missing_error_code=supports_device_missing)
-        # Some time later, we can do the wait/retry to ensure detach
-        self.assertRaises(exception.DeviceNotFound, retry_detach)
-        # Check that the save_and_reraise_exception context manager didn't log
-        # a traceback when the libvirtError was caught and DeviceNotFound was
-        # raised.
-        self.assertNotIn('Original exception being dropped',
-                         self.stdlog.logger.output)
-
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_second_detach_operation_failed(self):
-        self._test_detach_device_with_retry_second_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_OPERATION_FAILED,
-            error_message="operation failed: disk vdb not found",
-            supports_device_missing=False)
-
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_second_detach_internal_error(self):
-        self._test_detach_device_with_retry_second_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_INTERNAL_ERROR,
-            error_message="operation failed: disk vdb not found",
-            supports_device_missing=False)
-
-    def test_detach_device_with_retry_second_detach_device_missing(self):
-        self._test_detach_device_with_retry_second_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_DEVICE_MISSING,
-            error_message="device not found: disk vdb not found",
-            supports_device_missing=True)
-
-    def _test_detach_device_with_retry_first_detach_failure(
-            self, error_code=None, error_message=None,
-            supports_device_missing=False):
-        # This simulates a persistent or live domain detach failing because the
-        # device is not found during the first attempt to detach the device.
-        # We should still attempt to detach the device from the live config if
-        # the detach from persistent failed OR we should retry the detach from
-        # the live config if the first detach from live config failed.
-        # Note that the side effects in this test case [fake_exc, None] could
-        # not happen in real life if the first detach failed because the detach
-        # from live raised not found. In real life, the second attempt to
-        # detach from live would raise not found again because the device is
-        # not present. The purpose of this test is to verify that we try to
-        # detach a second time if the first detach fails, so we are OK with the
-        # unrealistic side effects for detach from live failing the first time.
-        conf = mock.Mock(spec=vconfig.LibvirtConfigGuestDevice)
-        conf.to_xml.return_value = "</xml>"
-        self.domain.isPersistent.return_value = True
-
-        get_config = mock.Mock()
-        # Simulate an inactive or live detach attempt which fails (not found)
-        # followed by a live config detach attempt that is successful
-        get_config.side_effect = [conf, conf, conf, None, None]
-        fake_device = "vdb"
-        fake_exc = fakelibvirt.make_libvirtError(
-            fakelibvirt.libvirtError, "",
-            error_message=error_message,
-            error_code=error_code,
-            error_domain=fakelibvirt.VIR_FROM_DOMAIN)
-        # Detach from persistent or live raises not found, detach from live
-        # succeeds afterward
-        self.domain.detachDeviceFlags.side_effect = [fake_exc, None]
-        retry_detach = self.guest.detach_device_with_retry(get_config,
-            fake_device, live=True, inc_sleep_time=.01, max_retry_count=3,
-            supports_device_missing_error_code=supports_device_missing)
-        # We should have tried to detach from the persistent domain
-        self.domain.detachDeviceFlags.assert_called_once_with(
-            "</xml>", flags=(fakelibvirt.VIR_DOMAIN_AFFECT_CONFIG |
-                             fakelibvirt.VIR_DOMAIN_AFFECT_LIVE))
-        # During the retry detach, should detach from the live domain
-        self.domain.detachDeviceFlags.reset_mock()
-        retry_detach()
-        # We should have tried to detach from the live domain
-        self.domain.detachDeviceFlags.assert_called_once_with(
-            "</xml>", flags=fakelibvirt.VIR_DOMAIN_AFFECT_LIVE)
-
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_first_detach_operation_failed(self):
-        self._test_detach_device_with_retry_first_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_OPERATION_FAILED,
-            error_message="operation failed: disk vdb not found",
-            supports_device_missing=False)
-
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_first_detach_internal_error(self):
-        self._test_detach_device_with_retry_first_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_INTERNAL_ERROR,
-            error_message="operation failed: disk vdb not found",
-            supports_device_missing=False)
-
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_first_detach_invalid_arg(self):
-        self._test_detach_device_with_retry_first_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_INVALID_ARG,
-            error_message="invalid argument: no target device vdb",
-            supports_device_missing=False)
-
-    def test_detach_device_with_retry_first_detach_device_missing(self):
-        self._test_detach_device_with_retry_first_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_DEVICE_MISSING,
-            error_message="device not found: disk vdb not found",
-            supports_device_missing=True)
-
     def test_get_xml_desc(self):
         self.guest.get_xml_desc()
         self.domain.XMLDesc.assert_called_once_with(flags=0)
@@ -578,6 +371,61 @@ class GuestTestCase(test.NoDBTestCase):
         self.assertIsNotNone(
             self.guest.get_interface_by_cfg(cfg))
         self.assertIsNone(self.guest.get_interface_by_cfg(None))
+        self.domain.XMLDesc.assert_has_calls([mock.call(0)] * 6)
+
+        # now check if the persistent config can be queried too
+        self.domain.XMLDesc.reset_mock()
+        devs = self.guest.get_all_devices(
+            devtype=None, from_persistent_config=True)
+        self.domain.XMLDesc.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_XML_INACTIVE)
+
+    def test_get_interface_by_cfg_persistent_domain(self):
+        self.domain.XMLDesc.return_value = """<domain>
+  <devices>
+    <interface type="bridge">
+      <mac address="fa:16:3e:f9:af:ae"/>
+      <model type="virtio"/>
+      <driver name="qemu"/>
+      <source bridge="qbr84008d03-11"/>
+      <target dev="tap84008d03-11"/>
+    </interface>
+  </devices>
+</domain>"""
+        cfg = vconfig.LibvirtConfigGuestInterface()
+        cfg.parse_str("""
+            <interface type="bridge">
+              <mac address="fa:16:3e:f9:af:ae"/>
+              <model type="virtio"/>
+              <driver name="qemu"/>
+              <source bridge="qbr84008d03-11"/>
+              <target dev="tap84008d03-11"/>
+              </interface>""")
+        self.assertIsNotNone(
+            self.guest.get_interface_by_cfg(
+                cfg, from_persistent_config=True))
+        cfg = vconfig.LibvirtConfigGuestInterface()
+        # NOTE(sean-k-mooney): a default constructed object is not valid
+        # to pass to get_interface_by_cfg as so we just modify the xml to
+        # make it not match
+        cfg.parse_str("""
+            <interface type="wont_match">
+              <mac address="fa:16:3e:f9:af:ae"/>
+              <model type="virtio"/>
+              <driver name="qemu"/>
+              <source bridge="qbr84008d03-11"/>
+              <target dev="tap84008d03-11"/>
+              </interface>""")
+        self.assertIsNone(
+            self.guest.get_interface_by_cfg(
+                cfg,
+                from_persistent_config=True))
+        self.domain.XMLDesc.assert_has_calls(
+            [
+                mock.call(fakelibvirt.VIR_DOMAIN_XML_INACTIVE),
+                mock.call(fakelibvirt.VIR_DOMAIN_XML_INACTIVE),
+            ]
+        )
 
     def test_get_interface_by_cfg_vhostuser(self):
         self.domain.XMLDesc.return_value = """<domain>
@@ -751,13 +599,123 @@ class GuestTestCase(test.NoDBTestCase):
         self.guest.migrate_configure_max_downtime(1000)
         self.domain.migrateSetMaxDowntime.assert_called_once_with(1000)
 
+    def test_set_metadata(self):
+        meta = mock.Mock(spec=vconfig.LibvirtConfigGuestMetaNovaInstance)
+        meta.to_xml.return_value = "</xml>"
+        self.guest.set_metadata(meta)
+        self.domain.setMetadata.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_METADATA_ELEMENT, "</xml>", "instance",
+            vconfig.NOVA_NS, flags=0)
+
+    def test_set_metadata_persistent(self):
+        meta = mock.Mock(spec=vconfig.LibvirtConfigGuestMetaNovaInstance)
+        meta.to_xml.return_value = "</xml>"
+        self.guest.set_metadata(meta, persistent=True)
+        self.domain.setMetadata.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_METADATA_ELEMENT, "</xml>", "instance",
+            vconfig.NOVA_NS, flags=fakelibvirt.VIR_DOMAIN_AFFECT_CONFIG)
+
+    def test_set_metadata_device_live(self):
+        meta = mock.Mock(spec=vconfig.LibvirtConfigGuestMetaNovaInstance)
+        meta.to_xml.return_value = "</xml>"
+        self.guest.set_metadata(meta, live=True)
+        self.domain.setMetadata.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_METADATA_ELEMENT, "</xml>", "instance",
+            vconfig.NOVA_NS, flags=fakelibvirt.VIR_DOMAIN_AFFECT_LIVE)
+
+    def test_set_metadata_persistent_live(self):
+        meta = mock.Mock(spec=vconfig.LibvirtConfigGuestMetaNovaInstance)
+        meta.to_xml.return_value = "</xml>"
+        self.guest.set_metadata(meta, persistent=True, live=True)
+        self.domain.setMetadata.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_METADATA_ELEMENT, "</xml>", "instance",
+            vconfig.NOVA_NS, flags=fakelibvirt.VIR_DOMAIN_AFFECT_LIVE |
+                                   fakelibvirt.VIR_DOMAIN_AFFECT_CONFIG)
+
+    def test_get_disk_xml(self):
+        dom_xml = """
+              <domain type="kvm">
+                <devices>
+                  <disk type="file">
+                     <source file="disk1_file"/>
+                     <target dev="vda" bus="virtio"/>
+                     <serial>0e38683e-f0af-418f-a3f1-6b67ea0f919d</serial>
+                  </disk>
+                  <disk type="block">
+                    <source dev="/path/to/dev/1"/>
+                    <target dev="vdb" bus="virtio" serial="1234"/>
+                  </disk>
+                </devices>
+              </domain>
+              """
+
+        diska_xml = """<disk type="file" device="disk">
+  <source file="disk1_file"/>
+  <target bus="virtio" dev="vda"/>
+  <serial>0e38683e-f0af-418f-a3f1-6b67ea0f919d</serial>
+</disk>"""
+
+        diskb_xml = """<disk type="block" device="disk">
+  <source dev="/path/to/dev/1"/>
+  <target bus="virtio" dev="vdb"/>
+</disk>"""
+
+        dom = mock.MagicMock()
+        dom.XMLDesc.return_value = dom_xml
+        guest = libvirt_guest.Guest(dom)
+
+        # NOTE(gcb): etree.tostring(node) returns an extra line with
+        # some white spaces, need to strip it.
+        actual_diska_xml = guest.get_disk('vda').to_xml()
+        self.assertXmlEqual(diska_xml, actual_diska_xml)
+
+        actual_diskb_xml = guest.get_disk('vdb').to_xml()
+        self.assertXmlEqual(diskb_xml, actual_diskb_xml)
+
+        self.assertIsNone(guest.get_disk('vdc'))
+
+        dom.XMLDesc.assert_has_calls([mock.call(0)] * 3)
+
+    def test_get_disk_xml_from_persistent_config(self):
+        dom_xml = """
+              <domain type="kvm">
+                <devices>
+                  <disk type="file">
+                     <source file="disk1_file"/>
+                     <target dev="vda" bus="virtio"/>
+                     <serial>0e38683e-f0af-418f-a3f1-6b67ea0f919d</serial>
+                  </disk>
+                  <disk type="block">
+                    <source dev="/path/to/dev/1"/>
+                    <target dev="vdb" bus="virtio" serial="1234"/>
+                  </disk>
+                </devices>
+              </domain>
+              """
+
+        diska_xml = """<disk type="file" device="disk">
+  <source file="disk1_file"/>
+  <target bus="virtio" dev="vda"/>
+  <serial>0e38683e-f0af-418f-a3f1-6b67ea0f919d</serial>
+</disk>"""
+
+        dom = mock.MagicMock()
+        dom.XMLDesc.return_value = dom_xml
+        guest = libvirt_guest.Guest(dom)
+
+        actual_diska_xml = guest.get_disk(
+            'vda', from_persistent_config=True).to_xml()
+        self.assertXmlEqual(diska_xml, actual_diska_xml)
+        dom.XMLDesc.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_XML_INACTIVE)
+
 
 class GuestBlockTestCase(test.NoDBTestCase):
 
     def setUp(self):
         super(GuestBlockTestCase, self).setUp()
 
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
         self.host = host.Host("qemu:///system")
         self.context = context.get_admin_context()
 
@@ -907,7 +865,7 @@ class JobInfoTestCase(test.NoDBTestCase):
     def setUp(self):
         super(JobInfoTestCase, self).setUp()
 
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
 
         self.conn = fakelibvirt.openAuth("qemu:///system",
                                          [[], lambda: True])
@@ -1095,3 +1053,25 @@ class JobInfoTestCase(test.NoDBTestCase):
 
         mock_stats.assert_called_once_with()
         mock_info.assert_called_once_with()
+
+    @mock.patch.object(fakelibvirt.virDomain, "jobInfo")
+    @mock.patch.object(fakelibvirt.virDomain, "jobStats")
+    def test_job_stats_no_ram(self, mock_stats, mock_info):
+        mock_stats.side_effect = fakelibvirt.make_libvirtError(
+            fakelibvirt.libvirtError,
+            "internal error: migration was active, but no RAM info was set",
+            error_code=fakelibvirt.VIR_ERR_INTERNAL_ERROR,
+            error_message="migration was active, but no RAM info was set")
+
+        info = self.guest.get_job_info()
+
+        self.assertIsInstance(info, libvirt_guest.JobInfo)
+        self.assertEqual(fakelibvirt.VIR_DOMAIN_JOB_NONE, info.type)
+        self.assertEqual(0, info.time_elapsed)
+        self.assertEqual(0, info.time_remaining)
+        self.assertEqual(0, info.memory_total)
+        self.assertEqual(0, info.memory_processed)
+        self.assertEqual(0, info.memory_remaining)
+
+        mock_stats.assert_called_once_with()
+        self.assertFalse(mock_info.called)
