@@ -18,15 +18,13 @@ import grp
 import os
 import pwd
 import tempfile
+from unittest import mock
 
 import ddt
-import mock
 import os_traits
-from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_utils import fileutils
 from oslo_utils.fixture import uuidsentinel as uuids
-import six
 
 from nova.compute import utils as compute_utils
 from nova import context
@@ -38,7 +36,6 @@ import nova.privsep.qemu
 from nova import test
 from nova.tests import fixtures as nova_fixtures
 from nova.tests.unit import fake_instance
-from nova.tests.unit.virt.libvirt import fakelibvirt
 from nova.virt import images
 from nova.virt.libvirt import guest as libvirt_guest
 from nova.virt.libvirt import utils as libvirt_utils
@@ -106,33 +103,61 @@ class LibvirtUtilsTestCase(test.NoDBTestCase):
     def test_valid_hostname_bad(self):
         self.assertFalse(libvirt_utils.is_valid_hostname("foo/?com=/bin/sh"))
 
-    @mock.patch('oslo_concurrency.processutils.execute')
-    def test_create_image(self, mock_execute):
-        libvirt_utils.create_image('raw', '/some/path', '10G')
-        libvirt_utils.create_image('qcow2', '/some/stuff', '1234567891234')
-        expected_args = [(('qemu-img', 'create', '-f', 'raw',
-                           '/some/path', '10G'),),
-                         (('qemu-img', 'create', '-f', 'qcow2',
-                           '/some/stuff', '1234567891234'),)]
-        self.assertEqual(expected_args, mock_execute.call_args_list)
-
-    @mock.patch('os.path.exists', return_value=True)
     @mock.patch('oslo_concurrency.processutils.execute')
     @mock.patch('nova.virt.images.qemu_img_info')
-    def test_create_cow_image(self, mock_info, mock_execute, mock_exists):
-        mock_execute.return_value = ('stdout', None)
+    def _test_create_image(
+        self, path, disk_format, disk_size, mock_info, mock_execute,
+        backing_file=None
+    ):
         mock_info.return_value = mock.Mock(
             file_format=mock.sentinel.backing_fmt,
-            cluster_size=mock.sentinel.cluster_size)
-        libvirt_utils.create_cow_image(mock.sentinel.backing_path,
-                                       mock.sentinel.new_path)
-        mock_info.assert_called_once_with(mock.sentinel.backing_path)
-        mock_execute.assert_has_calls([mock.call(
-            'qemu-img', 'create', '-f', 'qcow2', '-o',
-            'backing_file=%s,backing_fmt=%s,cluster_size=%s' % (
-                mock.sentinel.backing_path, mock.sentinel.backing_fmt,
-                mock.sentinel.cluster_size),
-             mock.sentinel.new_path)])
+            cluster_size=mock.sentinel.cluster_size,
+        )
+
+        libvirt_utils.create_image(
+            path, disk_format, disk_size, backing_file=backing_file)
+
+        cow_opts = []
+
+        if backing_file is None:
+            mock_info.assert_not_called()
+        else:
+            mock_info.assert_called_once_with(backing_file)
+            cow_opts = [
+                '-o',
+                f'backing_file={mock.sentinel.backing_file},'
+                f'backing_fmt={mock.sentinel.backing_fmt},'
+                f'cluster_size={mock.sentinel.cluster_size}',
+            ]
+
+        expected_args = (
+            'env', 'LC_ALL=C', 'LANG=C', 'qemu-img', 'create', '-f',
+            disk_format, *cow_opts, path,
+        )
+        if disk_size is not None:
+            expected_args += (disk_size,)
+
+        self.assertEqual([(expected_args,)], mock_execute.call_args_list)
+
+    def test_create_image_raw(self):
+        self._test_create_image('/some/path', 'raw', '10G')
+
+    def test_create_image_qcow2(self):
+        self._test_create_image(
+            '/some/stuff', 'qcow2', '1234567891234',
+        )
+
+    def test_create_image_backing_file(self):
+        self._test_create_image(
+            '/some/stuff', 'qcow2', '1234567891234',
+            backing_file=mock.sentinel.backing_file,
+        )
+
+    def test_create_image_size_none(self):
+        self._test_create_image(
+            '/some/stuff', 'qcow2', None,
+            backing_file=mock.sentinel.backing_file,
+        )
 
     @ddt.unpack
     @ddt.data({'fs_type': 'some_fs_type',
@@ -159,77 +184,6 @@ class LibvirtUtilsTestCase(test.NoDBTestCase):
                 mock.call('5G', 'expanded', expected_fs_type,
                           '/some/path/root.hds')])
 
-    def test_pick_disk_driver_name(self):
-        type_map = {'kvm': ([True, 'qemu'], [False, 'qemu'], [None, 'qemu']),
-                    'qemu': ([True, 'qemu'], [False, 'qemu'], [None, 'qemu']),
-                    'uml': ([True, None], [False, None], [None, None]),
-                    'lxc': ([True, None], [False, None], [None, None])}
-        # NOTE(aloga): Xen is tested in test_pick_disk_driver_name_xen
-
-        version = 1005001
-        for (virt_type, checks) in type_map.items():
-            self.flags(virt_type=virt_type, group='libvirt')
-            for (is_block_dev, expected_result) in checks:
-                result = libvirt_utils.pick_disk_driver_name(version,
-                                                             is_block_dev)
-                self.assertEqual(result, expected_result)
-
-    @mock.patch('nova.privsep.libvirt.xend_probe')
-    @mock.patch('oslo_concurrency.processutils.execute')
-    def test_pick_disk_driver_name_xen(self, mock_execute, mock_xend_probe):
-
-        def execute_side_effect(*args, **kwargs):
-            if args == ('tap-ctl', 'check'):
-                if mock_execute.blktap is True:
-                    return ('ok\n', '')
-                elif mock_execute.blktap is False:
-                    return ('some error\n', '')
-                else:
-                    raise OSError(2, "No such file or directory")
-            raise Exception('Unexpected call')
-        mock_execute.side_effect = execute_side_effect
-
-        def xend_probe_side_effect():
-            if mock_execute.xend is True:
-                return ('', '')
-            elif mock_execute.xend is False:
-                raise processutils.ProcessExecutionError("error")
-            else:
-                raise OSError(2, "No such file or directory")
-        mock_xend_probe.side_effect = xend_probe_side_effect
-
-        self.flags(virt_type="xen", group='libvirt')
-        versions = [4000000, 4001000, 4002000, 4003000, 4005000]
-        for version in versions:
-            # block dev
-            result = libvirt_utils.pick_disk_driver_name(version, True)
-            self.assertEqual(result, "phy")
-            self.assertFalse(mock_execute.called)
-            mock_execute.reset_mock()
-            # file dev
-            for blktap in True, False, None:
-                mock_execute.blktap = blktap
-                for xend in True, False, None:
-                    mock_execute.xend = xend
-                    result = libvirt_utils.pick_disk_driver_name(version,
-                                                                 False)
-                    # qemu backend supported only by libxl which is
-                    # production since xen 4.2. libvirt use libxl if
-                    # xend service not started.
-                    if version >= 4002000 and xend is not True:
-                        self.assertEqual(result, 'qemu')
-                    elif blktap:
-                        if version == 4000000:
-                            self.assertEqual(result, 'tap')
-                        else:
-                            self.assertEqual(result, 'tap2')
-                    else:
-                        self.assertEqual(result, 'file')
-                    # default is_block_dev False
-                    self.assertEqual(result,
-                        libvirt_utils.pick_disk_driver_name(version))
-                    mock_execute.reset_mock()
-
     def test_copy_image(self):
         dst_fd, dst_path = tempfile.mkstemp()
         try:
@@ -549,23 +503,23 @@ sunrpc /var/lib/nfs/rpc_pipefs rpc_pipefs rw,relatime 0 0
 
             # Source is given, and matches source in /proc/mounts
             proc_mnt = mock.mock_open(read_data=proc_with_mnt)
-            with mock.patch.object(six.moves.builtins, "open", proc_mnt):
+            with mock.patch('builtins.open', proc_mnt):
                 self.assertTrue(libvirt_utils.is_mounted(mount_path, source))
 
             # Source is given, and doesn't match source in /proc/mounts
             proc_mnt = mock.mock_open(read_data=proc_wrong_mnt)
-            with mock.patch.object(six.moves.builtins, "open", proc_mnt):
+            with mock.patch('builtins.open', proc_mnt):
                 self.assertFalse(libvirt_utils.is_mounted(mount_path, source))
 
             # Source is given, and mountpoint isn't present in /proc/mounts
             # Note that this shouldn't occur, as os.path.ismount should have
             # previously returned False in this case.
             proc_umnt = mock.mock_open(read_data=proc_without_mnt)
-            with mock.patch.object(six.moves.builtins, "open", proc_umnt):
+            with mock.patch('builtins.open', proc_umnt):
                 self.assertFalse(libvirt_utils.is_mounted(mount_path, source))
 
     def test_find_disk_file_device(self):
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
         xml = """
           <domain type='kvm'>
             <os>
@@ -587,7 +541,7 @@ sunrpc /var/lib/nfs/rpc_pipefs rpc_pipefs rw,relatime 0 0
         self.assertEqual('qcow2', format)
 
     def test_find_disk_block_device(self):
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
         xml = """
           <domain type='kvm'>
             <os>
@@ -609,7 +563,7 @@ sunrpc /var/lib/nfs/rpc_pipefs rpc_pipefs rw,relatime 0 0
         self.assertEqual('raw', format)
 
     def test_find_disk_rbd(self):
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
         xml = """
           <domain type='kvm'>
             <os>
@@ -633,7 +587,7 @@ sunrpc /var/lib/nfs/rpc_pipefs rpc_pipefs rw,relatime 0 0
         self.assertEqual('raw', format)
 
     def test_find_disk_lxc(self):
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
         xml = """
           <domain type='lxc'>
             <os>
@@ -654,7 +608,7 @@ sunrpc /var/lib/nfs/rpc_pipefs rpc_pipefs rw,relatime 0 0
         self.assertIsNone(format)
 
     def test_find_disk_parallels(self):
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
         xml = """
           <domain type='parallels'>
             <os>
