@@ -15,9 +15,9 @@
 from collections import deque
 import sys
 import traceback
+from unittest import mock
 
 import fixtures
-import mock
 import netaddr
 import os_resource_classes as orc
 import os_vif
@@ -25,7 +25,6 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import importutils
 from oslo_utils import timeutils
-import six
 
 from nova.compute import manager
 from nova import conf
@@ -35,9 +34,9 @@ from nova import exception
 from nova import objects
 from nova import test
 from nova.tests import fixtures as nova_fixtures
+from nova.tests.fixtures import libvirt as fakelibvirt
 from nova.tests.unit import fake_block_device
 from nova.tests.unit import utils as test_utils
-from nova.tests.unit.virt.libvirt import fakelibvirt
 from nova.virt import block_device as driver_block_device
 from nova.virt import event as virtevent
 from nova.virt import fake
@@ -55,6 +54,7 @@ def catch_notimplementederror(f):
     If a particular call makes a driver raise NotImplementedError, we
     log it so that we can extract this information afterwards as needed.
     """
+
     def wrapped_func(self, *args, **kwargs):
         try:
             return f(self, *args, **kwargs)
@@ -83,17 +83,9 @@ class _FakeDriverBackendTestCase(object):
         else:
             self.saved_libvirt = None
 
-        from nova.tests.unit.virt.libvirt import fake_imagebackend
-        from nova.tests.unit.virt.libvirt import fakelibvirt
-
-        from nova.tests.unit.virt.libvirt import fake_os_brick_connector
-
-        self.useFixture(fake_imagebackend.ImageBackendFixture())
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
-
-        self.useFixture(fixtures.MonkeyPatch(
-            'nova.virt.libvirt.driver.connector',
-            fake_os_brick_connector))
+        self.useFixture(nova_fixtures.OSBrickFixture())
+        self.useFixture(nova_fixtures.LibvirtImageBackendFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
 
         self.useFixture(fixtures.MonkeyPatch(
             'nova.virt.libvirt.host.Host._conn_event_thread',
@@ -176,7 +168,7 @@ class VirtDriverLoaderTestCase(_FakeDriverBackendTestCase, test.TestCase):
                 self.fail("Couldn't load driver %s - %s" % (cls, e))
 
             self.assertEqual(cm.driver.__class__.__name__, driver,
-                             "Could't load driver %s" % cls)
+                             "Couldn't load driver %s" % cls)
 
     @mock.patch.object(sys, 'exit', side_effect=test.TestingException())
     def test_fail_to_load_new_drivers(self, mock_exit):
@@ -250,12 +242,6 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
         self.connection.snapshot(self.ctxt, instance_ref, img_ref['id'],
                                  lambda *args, **kwargs: None)
 
-    @catch_notimplementederror
-    def test_post_interrupted_snapshot_cleanup(self):
-        instance_ref, network_info = self._get_running_instance()
-        self.connection.post_interrupted_snapshot_cleanup(self.ctxt,
-                instance_ref)
-
     @catch_notimplementederror
     def test_reboot(self):
         reboot_type = "SOFT"
@@ -423,6 +409,7 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
         self.assertEqual(storage_ip, result['ip'])
 
     @catch_notimplementederror
+    @mock.patch('threading.Event.wait', new=mock.Mock())
     @mock.patch.object(libvirt.driver.LibvirtDriver, '_build_device_metadata',
                        return_value=objects.InstanceDeviceMetadata())
     def test_attach_detach_volume(self, _):
@@ -459,6 +446,7 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
                                         '/dev/sda', 2))
 
     @catch_notimplementederror
+    @mock.patch('threading.Event.wait', new=mock.Mock())
     @mock.patch.object(libvirt.driver.LibvirtDriver, '_build_device_metadata',
                        return_value=objects.InstanceDeviceMetadata())
     def test_attach_detach_different_power_states(self, _):
@@ -538,7 +526,7 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
         instance_ref, network_info = self._get_running_instance()
         console_output = self.connection.get_console_output(self.ctxt,
             instance_ref)
-        self.assertIsInstance(console_output, six.string_types)
+        self.assertIsInstance(console_output, str)
 
     @catch_notimplementederror
     def test_get_vnc_console(self):
@@ -574,16 +562,8 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
                                                       instance_ref)
         self.assertIsInstance(mks_console, ctype.ConsoleMKS)
 
-    @catch_notimplementederror
-    def test_get_console_pool_info(self):
-        instance_ref, network_info = self._get_running_instance()
-        console_pool = self.connection.get_console_pool_info(instance_ref)
-        self.assertIn('address', console_pool)
-        self.assertIn('username', console_pool)
-        self.assertIn('password', console_pool)
-
     @mock.patch(
-        'nova.tests.unit.virt.libvirt.fakelibvirt.Domain.jobStats',
+        'nova.tests.fixtures.libvirt.Domain.jobStats',
         new=mock.Mock(return_value={
             'type': fakelibvirt.VIR_DOMAIN_JOB_COMPLETED}))
     def test_live_migration(self):
@@ -662,14 +642,6 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
     def test_host_power_action_startup(self):
         self.connection.host_power_action('startup')
 
-    @catch_notimplementederror
-    def test_add_to_aggregate(self):
-        self.connection.add_to_aggregate(self.ctxt, 'aggregate', 'host')
-
-    @catch_notimplementederror
-    def test_remove_from_aggregate(self):
-        self.connection.remove_from_aggregate(self.ctxt, 'aggregate', 'host')
-
     def test_events(self):
         got_events = []
 
@@ -748,10 +720,6 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
             self.connection.emit_event(started_event)
         callback.assert_called_once_with(started_event)
 
-    def test_set_bootable(self):
-        self.assertRaises(NotImplementedError, self.connection.set_bootable,
-                          'instance', True)
-
     @catch_notimplementederror
     def test_get_instance_disk_info(self):
         # This should be implemented by any driver that supports live migrate.
@@ -778,13 +746,13 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
         self.flags(cpu_allocation_ratio=16.1)
         self.flags(ram_allocation_ratio=1.6)
         self.flags(disk_allocation_ratio=1.1)
-        expeced_ratios = {
+        expected_ratios = {
             orc.VCPU: CONF.cpu_allocation_ratio,
             orc.MEMORY_MB: CONF.ram_allocation_ratio,
             orc.DISK_GB: CONF.disk_allocation_ratio
         }
         # If conf is set, return conf
-        self.assertEqual(expeced_ratios,
+        self.assertEqual(expected_ratios,
                          self.connection._get_allocation_ratios(inv))
 
         self.flags(cpu_allocation_ratio=None)
@@ -793,25 +761,25 @@ class _VirtDriverTestCase(_FakeDriverBackendTestCase):
         self.flags(initial_cpu_allocation_ratio=15.9)
         self.flags(initial_ram_allocation_ratio=1.4)
         self.flags(initial_disk_allocation_ratio=0.9)
-        expeced_ratios = {
+        expected_ratios = {
             orc.VCPU: CONF.initial_cpu_allocation_ratio,
             orc.MEMORY_MB: CONF.initial_ram_allocation_ratio,
             orc.DISK_GB: CONF.initial_disk_allocation_ratio
         }
         # if conf is unset and inv doesn't exists, return init conf
-        self.assertEqual(expeced_ratios,
+        self.assertEqual(expected_ratios,
                          self.connection._get_allocation_ratios(inv))
 
         inv = {orc.VCPU: {'allocation_ratio': 3.0},
                orc.MEMORY_MB: {'allocation_ratio': 3.1},
                orc.DISK_GB: {'allocation_ratio': 3.2}}
-        expeced_ratios = {
+        expected_ratios = {
             orc.VCPU: inv[orc.VCPU]['allocation_ratio'],
             orc.MEMORY_MB: inv[orc.MEMORY_MB]['allocation_ratio'],
             orc.DISK_GB: inv[orc.DISK_GB]['allocation_ratio']
         }
         # if conf is unset and inv exists, return inv
-        self.assertEqual(expeced_ratios,
+        self.assertEqual(expected_ratios,
                          self.connection._get_allocation_ratios(inv))
 
 
@@ -864,6 +832,7 @@ class LibvirtConnTestCase(_VirtDriverTestCase, test.TestCase):
         # This is needed for the live migration tests which spawn off the
         # operation for monitoring.
         self.useFixture(nova_fixtures.SpawnIsSynchronousFixture())
+        self.useFixture(nova_fixtures.CGroupsFixture())
         # When destroying an instance, os-vif will try to execute some commands
         # which hang tests so let's just stub out the unplug call to os-vif
         # since we don't care about it.
