@@ -23,11 +23,11 @@ import collections
 import sys
 
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
 from oslo_utils import units
 from oslo_utils import uuidutils
 from oslo_vmware import exceptions as vexc
 from oslo_vmware.objects import datastore as ds_obj
+from oslo_vmware import vim_util
 
 from nova import exception
 from nova.virt.vmwareapi import constants
@@ -76,23 +76,34 @@ def cleanup():
             _db_content[c] = {}
 
 
-def _create_object(table, table_obj):
+def _create_object(table_obj):
     """Create an object in the db."""
-    _db_content.setdefault(table, {})
-    _db_content[table][table_obj.obj] = table_obj
+    _db_content.setdefault(table_obj.obj._type, {})
+    update_object(table_obj)
 
 
-def _get_object(obj_ref):
+def get_object(obj_ref):
     """Get object for the give reference."""
-    return _db_content[obj_ref.type][obj_ref]
+    return _db_content[obj_ref.type][obj_ref.value]
 
 
-def _get_objects(obj_type):
+def get_objects(obj_type):
     """Get objects of the type."""
-    lst_objs = FakeRetrieveResult()
-    for key in _db_content[obj_type]:
-        lst_objs.add_object(_db_content[obj_type][key])
-    return lst_objs
+    return _db_content[obj_type].values()
+
+
+def get_first_object(obj_type):
+    """Get the first object of an object type"""
+    return next(iter(_db_content[obj_type].values()))
+
+
+def get_first_object_ref(obj_type):
+    """Get the first reference of an object type"""
+    return get_first_object(obj_type).obj
+
+
+def _no_objects_of_type(obj_type):
+    return not _db_content.get(obj_type)
 
 
 def _convert_to_array_of_mor(mors):
@@ -135,21 +146,19 @@ class FakeRetrieveResult(object):
         if token is not None:
             self.token = token
 
-    def add_object(self, object):
-        self.objects.append(object)
+    def add_object(self, obj):
+        self.objects.append(obj)
 
 
-def _get_object_refs(obj_type):
-    """Get object References of the type."""
-    lst_objs = []
-    for key in _db_content[obj_type]:
-        lst_objs.append(key)
-    return lst_objs
+def get_object_refs(obj_type):
+    """Get iterator over object References of the type."""
+    for obj in _db_content[obj_type].values():
+        yield obj.obj
 
 
-def _update_object(table, table_obj):
+def update_object(table_obj):
     """Update objects of the type."""
-    _db_content[table][table_obj.obj] = table_obj
+    _db_content[table_obj.obj._type][table_obj.obj.value] = table_obj
 
 
 class Prop(object):
@@ -177,6 +186,14 @@ class ManagedObjectReference(object):
         self.type = name
         self._type = name
 
+    def __repr__(self):
+        return f'{self._type}:{self.value}'
+
+    def __eq__(self, other):
+        return (other is not None and
+                vim_util.get_moref_value(other) == self.value and
+                vim_util.get_moref_type(other) == self.type)
+
 
 class ObjectContent(object):
     """ObjectContent array holds dynamic properties."""
@@ -262,8 +279,11 @@ class ManagedObject(object):
         return prefix + "-" + str(self.__class__._counter)
 
     def __repr__(self):
-        return jsonutils.dumps({elem.name: elem.val
-                                for elem in self.propSet})
+        # We can't just dump the managed-object, because it may be circular
+        return "{}:{}({})".format(self.obj._type, self.obj.value,
+            ", ".join(
+                "{}={}".format(p.name, p.val if p.name == "name" else "<>")
+                for p in self.propSet))
 
 
 class DataObject(object):
@@ -366,6 +386,7 @@ class VirtualIDEController(DataObject):
 
 class VirtualLsiLogicController(DataObject):
     """VirtualLsiLogicController class."""
+
     def __init__(self, key=0, scsiCtlrUnitNumber=0, busNumber=0):
         self.key = key
         self.busNumber = busNumber
@@ -592,8 +613,7 @@ class ResourcePool(ManagedObject):
 class DatastoreHostMount(DataObject):
     def __init__(self, value='host-100'):
         super(DatastoreHostMount, self).__init__()
-        host_ref = (_db_content["HostSystem"]
-                    [list(_db_content["HostSystem"].keys())[0]].obj)
+        host_ref = get_first_object_ref("HostSystem")
         host_system = DataObject()
         host_system.ManagedObjectReference = [host_ref]
         host_system.value = value
@@ -620,9 +640,15 @@ class ClusterComputeResource(ManagedObject):
         summary.effectiveCpu = 10000
         self.set("summary", summary)
 
+        vm_list = DataObject()
+        vm_list.ManagedObjectReference = []
+        self.set("vm", vm_list)
+
     def _add_root_resource_pool(self, r_pool):
         if r_pool:
             self.set("resourcePool", r_pool)
+            pool = get_object(r_pool)
+            self.set("vm", pool.get("vm"))
 
     def _add_host(self, host_sys):
         if host_sys:
@@ -658,7 +684,7 @@ class ClusterComputeResource(ManagedObject):
         # Compute the aggregate stats
         summary.numHosts = len(hosts.ManagedObjectReference)
         for host_ref in hosts.ManagedObjectReference:
-            host_sys = _get_object(host_ref)
+            host_sys = get_object(host_ref)
             connected = host_sys.get("connected")
             host_summary = host_sys.get("summary")
             summary.numCpuCores += host_summary.hardware.numCpuCores
@@ -716,14 +742,17 @@ class HostSystem(ManagedObject):
                  maintenance_mode=False):
         super(HostSystem, self).__init__("host")
         self.set("name", name)
-        if _db_content.get("HostNetworkSystem", None) is None:
+        if _no_objects_of_type("HostNetworkSystem"):
             create_host_network_system()
-        if not _get_object_refs('HostStorageSystem'):
+
+        if _no_objects_of_type("HostStorageSystem"):
             create_host_storage_system()
-        host_net_key = list(_db_content["HostNetworkSystem"].keys())[0]
-        host_net_sys = _db_content["HostNetworkSystem"][host_net_key].obj
-        self.set("configManager.networkSystem", host_net_sys)
-        host_storage_sys_key = _get_object_refs('HostStorageSystem')[0]
+
+        host_net_obj = get_first_object("HostNetworkSystem")
+        host_net_ref = host_net_obj.obj
+        self.set("configManager.networkSystem", host_net_ref)
+
+        host_storage_sys_key = get_first_object_ref('HostStorageSystem')
         self.set("configManager.storageSystem", host_storage_sys_key)
 
         if not ds_ref:
@@ -778,10 +807,9 @@ class HostSystem(ManagedObject):
         self.set("config.network.pnic", net_info_pnic)
         self.set("connected", connected)
 
-        if _db_content.get("Network", None) is None:
+        if _no_objects_of_type("Network"):
             create_network()
-        net_ref = _db_content["Network"][
-            list(_db_content["Network"].keys())[0]].obj
+        net_ref = get_first_object_ref("Network")
         network_do = DataObject()
         network_do.ManagedObjectReference = [net_ref]
         self.set("network", network_do)
@@ -791,9 +819,9 @@ class HostSystem(ManagedObject):
         vswitch_do.name = "vSwitch0"
         vswitch_do.portgroup = ["PortGroup-vmnet0"]
 
-        net_swicth = DataObject()
-        net_swicth.HostVirtualSwitch = [vswitch_do]
-        self.set("config.network.vswitch", net_swicth)
+        net_switch = DataObject()
+        net_switch.HostVirtualSwitch = [vswitch_do]
+        self.set("config.network.vswitch", net_switch)
 
         host_pg_do = DataObject()
         host_pg_do.key = "PortGroup-vmnet0"
@@ -820,7 +848,7 @@ class HostSystem(ManagedObject):
         self.set("config.storageDevice.hostBusAdapter", host_bus_adapter_array)
 
         # Set the same on the storage system managed object
-        host_storage_sys = _get_object(host_storage_sys_key)
+        host_storage_sys = get_object(host_storage_sys_key)
         host_storage_sys.set('storageDeviceInfo.hostBusAdapter',
                              host_bus_adapter_array)
 
@@ -881,17 +909,15 @@ class Datacenter(ManagedObject):
     def __init__(self, name="ha-datacenter", ds_ref=None):
         super(Datacenter, self).__init__("dc")
         self.set("name", name)
-        if _db_content.get("Folder", None) is None:
+        if _no_objects_of_type("Folder"):
             create_folder()
-        folder_ref = _db_content["Folder"][
-            list(_db_content["Folder"].keys())[0]].obj
+        folder_ref = get_first_object_ref("Folder")
         folder_do = DataObject()
         folder_do.ManagedObjectReference = [folder_ref]
         self.set("vmFolder", folder_ref)
-        if _db_content.get("Network", None) is None:
+        if _no_objects_of_type("Network"):
             create_network()
-        net_ref = _db_content["Network"][
-            list(_db_content["Network"].keys())[0]].obj
+        net_ref = get_first_object_ref("Network")
         network_do = DataObject()
         network_do.ManagedObjectReference = [net_ref]
         self.set("network", network_do)
@@ -926,54 +952,56 @@ class Task(ManagedObject):
 
 def create_host_network_system():
     host_net_system = HostNetworkSystem()
-    _create_object("HostNetworkSystem", host_net_system)
+    _create_object(host_net_system)
 
 
 def create_host_storage_system():
     host_storage_system = HostStorageSystem()
-    _create_object("HostStorageSystem", host_storage_system)
+    _create_object(host_storage_system)
 
 
 def create_host(ds_ref=None):
     host_system = HostSystem(ds_ref=ds_ref)
-    _create_object('HostSystem', host_system)
+    _create_object(host_system)
 
 
 def create_datacenter(name, ds_ref=None):
     data_center = Datacenter(name, ds_ref)
-    _create_object('Datacenter', data_center)
+    _create_object(data_center)
 
 
 def create_datastore(name, capacity, free):
     data_store = Datastore(name, capacity, free)
-    _create_object('Datastore', data_store)
+    _create_object(data_store)
     return data_store.obj
 
 
 def create_res_pool():
     res_pool = ResourcePool()
-    _create_object('ResourcePool', res_pool)
+    _create_object(res_pool)
     return res_pool.obj
 
 
 def create_folder():
     folder = Folder()
-    _create_object('Folder', folder)
+    _create_object(folder)
     return folder.obj
 
 
 def create_network():
     network = Network()
-    _create_object('Network', network)
+    _create_object(network)
 
 
 def create_cluster(name, ds_ref):
     cluster = ClusterComputeResource(name=name)
-    cluster._add_host(_get_object_refs("HostSystem")[0])
-    cluster._add_host(_get_object_refs("HostSystem")[1])
+    for i, host in enumerate(get_object_refs("HostSystem")):
+        cluster._add_host(host)
+        if i >= 1:
+            break
     cluster._add_datastore(ds_ref)
     cluster._add_root_resource_pool(create_res_pool())
-    _create_object('ClusterComputeResource', cluster)
+    _create_object(cluster)
     return cluster
 
 
@@ -992,16 +1020,15 @@ def create_vm(uuid=None, name=None,
         devices = []
 
     if vmPathName is None:
-        vm_path = ds_obj.DatastorePath(
-            list(_db_content['Datastore'].values())[0])
+        vm_path = ds_obj.DatastorePath(get_first_object("Datastore"))
     else:
         vm_path = ds_obj.DatastorePath.parse(vmPathName)
 
     if res_pool_ref is None:
-        res_pool_ref = list(_db_content['ResourcePool'].keys())[0]
+        res_pool_ref = get_first_object_ref("ResourcePool")
 
     if host_ref is None:
-        host_ref = list(_db_content["HostSystem"].keys())[0]
+        host_ref = get_first_object_ref("HostSystem")
 
     # Fill in the default path to the vmx file if we were only given a
     # datastore. Note that if you create a VM with vmPathName '[foo]', when you
@@ -1010,9 +1037,9 @@ def create_vm(uuid=None, name=None,
     if vm_path.rel_path == '':
         vm_path = vm_path.join(name, name + '.vmx')
 
-    for key, value in _db_content["Datastore"].items():
+    for value in get_objects("Datastore"):
         if value.get('summary.name') == vm_path.datastore:
-            ds = key
+            ds = value.obj
             break
     else:
         ds = create_datastore(vm_path.datastore, 1024, 500)
@@ -1029,9 +1056,9 @@ def create_vm(uuid=None, name=None,
               "instanceUuid": uuid,
               "version": version}
     vm = VirtualMachine(**vm_dict)
-    _create_object("VirtualMachine", vm)
+    _create_object(vm)
 
-    res_pool = _get_object(res_pool_ref)
+    res_pool = get_object(res_pool_ref)
     res_pool.vm.ManagedObjectReference.append(vm.obj)
 
     return vm.obj
@@ -1039,7 +1066,7 @@ def create_vm(uuid=None, name=None,
 
 def create_task(task_name, state="running", result=None, error_fault=None):
     task = Task(task_name, state, result, error_fault)
-    _create_object("Task", task)
+    _create_object(task)
     return task
 
 
@@ -1102,12 +1129,14 @@ def fake_fetch_image(context, instance, host, port, dc_name, ds_name,
 
 def _get_vm_mdo(vm_ref):
     """Gets the Virtual Machine with the ref from the db."""
-    if _db_content.get("VirtualMachine", None) is None:
+    vms = _db_content.get("VirtualMachine")
+    if not vms:
         raise exception.NotFound("There is no VM registered")
-    if vm_ref not in _db_content.get("VirtualMachine"):
+    try:
+        return vms[vm_ref.value]
+    except KeyError:
         raise exception.NotFound("Virtual Machine with ref %s is not "
-                                 "there" % vm_ref)
-    return _db_content.get("VirtualMachine")[vm_ref]
+                                 "there" % vm_ref.value)
 
 
 def _merge_extraconfig(existing, changes):
@@ -1353,11 +1382,10 @@ class FakeVim(object):
     def _find_all_by_uuid(self, *args, **kwargs):
         uuid = kwargs.get('uuid')
         vm_refs = []
-        for vm_ref in _db_content.get("VirtualMachine"):
-            vm = _get_object(vm_ref)
+        for vm in get_objects("VirtualMachine"):
             vm_uuid = vm.get("summary.config.instanceUuid")
             if vm_uuid == uuid:
-                vm_refs.append(vm_ref)
+                vm_refs.append(vm.obj)
         return vm_refs
 
     def _delete_snapshot(self, method, *args, **kwargs):
@@ -1411,7 +1439,7 @@ class FakeVim(object):
                 vm_dict["extra_config"] = extraConfigs
 
         virtual_machine = VirtualMachine(**vm_dict)
-        _create_object("VirtualMachine", virtual_machine)
+        _create_object(virtual_machine)
         task_mdo = create_task(method, "success")
         return task_mdo.obj
 
@@ -1419,7 +1447,7 @@ class FakeVim(object):
         """Unregisters a VM from the Host System."""
         vm_ref = args[0]
         _get_vm_mdo(vm_ref)
-        del _db_content["VirtualMachine"][vm_ref]
+        del _db_content["VirtualMachine"][vm_ref.value]
         task_mdo = create_task(method, "success")
         return task_mdo.obj
 
@@ -1490,13 +1518,7 @@ class FakeVim(object):
 
     def _set_power_state(self, method, vm_ref, pwr_state="poweredOn"):
         """Sets power state for the VM."""
-        if _db_content.get("VirtualMachine", None) is None:
-            raise exception.NotFound("No Virtual Machine has been "
-                                     "registered yet")
-        if vm_ref not in _db_content.get("VirtualMachine"):
-            raise exception.NotFound("Virtual Machine with ref %s is not "
-                                     "there" % vm_ref)
-        vm_mdo = _db_content.get("VirtualMachine").get(vm_ref)
+        vm_mdo = _get_vm_mdo(vm_ref)
         vm_mdo.set("runtime.powerState", pwr_state)
         task_mdo = create_task(method, "success")
         return task_mdo.obj
@@ -1525,7 +1547,7 @@ class FakeVim(object):
                     # This means that we are retrieving props for all managed
                     # data objects of the specified 'type' in the entire
                     # inventory. This gets invoked by vim_util.get_objects.
-                    mdo_refs = _db_content[spec_type]
+                    mdo_refs = list(get_object_refs(spec_type))
                 elif obj_ref.type != spec_type:
                     # This means that we are retrieving props for the managed
                     # data objects in the parent object's 'path' property.
@@ -1535,7 +1557,7 @@ class FakeVim(object):
                     #     path = 'datastore'
                     # the above will retrieve all datastores in the given
                     # cluster.
-                    parent_mdo = _db_content[obj_ref.type][obj_ref]
+                    parent_mdo = get_object(obj_ref)
                     path = obj.selectSet[0].path
                     mdo_refs = parent_mdo.get(path).ManagedObjectReference
                 else:
@@ -1544,12 +1566,13 @@ class FakeVim(object):
                     # vim_util.get_properties_for_a_collection_of_objects.
                     mdo_refs = [obj_ref]
 
+                mdo_list = _db_content[spec_type]
                 for mdo_ref in mdo_refs:
-                    mdo = _db_content[spec_type][mdo_ref]
-                    prop_list = []
-                    for prop_name in properties:
-                        prop = Prop(prop_name, mdo.get(prop_name))
-                        prop_list.append(prop)
+                    mdo = mdo_list[mdo_ref.value]
+                    prop_list = [
+                        Prop(prop_name, mdo.get(prop_name))
+                        for prop_name in properties
+                    ]
                     obj_content = ObjectContent(mdo.obj, prop_list)
                     lst_ret_objs.add_object(obj_content)
             except Exception:
@@ -1559,14 +1582,13 @@ class FakeVim(object):
 
     def _add_port_group(self, method, *args, **kwargs):
         """Adds a port group to the host system."""
-        _host_sk = list(_db_content["HostSystem"].keys())[0]
-        host_mdo = _db_content["HostSystem"][_host_sk]
+        host_mdo = get_first_object("HostSystem")
         host_mdo._add_port_group(kwargs.get("portgrp"))
 
     def _add_iscsi_send_tgt(self, method, *args, **kwargs):
         """Adds a iscsi send target to the hba."""
         send_targets = kwargs.get('targets')
-        host_storage_sys = _get_objects('HostStorageSystem').objects[0]
+        host_storage_sys = get_first_object('HostStorageSystem')
         iscsi_hba_array = host_storage_sys.get('storageDeviceInfo'
                                                '.hostBusAdapter')
         iscsi_hba = iscsi_hba_array.HostHostBusAdapter[0]
