@@ -13,13 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 from cinderclient import api_versions as cinder_api_versions
 from cinderclient import exceptions as cinder_exception
 from cinderclient.v3 import limits as cinder_limits
 from keystoneauth1 import loading as ks_loading
 from keystoneauth1 import session
 from keystoneclient import exceptions as keystone_exception
-import mock
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import timeutils
 
@@ -1078,6 +1079,17 @@ class CinderApiTestCase(test.NoDBTestCase):
         mock_volumes.get_encryption_metadata.assert_called_once_with(
             {'encryption_key_id': 'fake_key'})
 
+    @mock.patch('nova.volume.cinder.cinderclient')
+    def test_volume_reimage(self, mock_cinderclient):
+        mock_reimage = mock.MagicMock()
+        mock_volumes = mock.MagicMock(reimage=mock_reimage)
+        mock_cinderclient.return_value = mock.MagicMock(volumes=mock_volumes)
+        self.api.reimage_volume(
+            self.ctx, uuids.volume_id, uuids.image_id,
+            reimage_reserved=True)
+        mock_cinderclient.assert_called_once_with(self.ctx, '3.68')
+        mock_reimage.assert_called_with(uuids.volume_id, uuids.image_id, True)
+
     def test_translate_cinder_exception_no_error(self):
         my_func = mock.Mock()
         my_func.__name__ = 'my_func'
@@ -1264,3 +1276,14 @@ class CinderClientTestCase(test.NoDBTestCase):
         admin_ctx = context.get_admin_context()
         params = cinder._get_cinderclient_parameters(admin_ctx)
         self.assertEqual(params[0], mock_admin_auth)
+
+    @mock.patch('nova.service_auth._SERVICE_AUTH')
+    @mock.patch('nova.volume.cinder._ADMIN_AUTH')
+    def test_admin_context_without_user_token_but_with_service_token(
+        self, mock_admin_auth, mock_service_auth
+    ):
+        self.flags(send_service_user_token=True, group='service_user')
+        admin_ctx = context.get_admin_context()
+        params = cinder._get_cinderclient_parameters(admin_ctx)
+        self.assertEqual(mock_admin_auth, params[0].user_auth)
+        self.assertEqual(mock_service_auth, params[0].service_auth)
