@@ -16,6 +16,7 @@ import functools
 import itertools
 
 from os_brick import encryptors
+from os_brick.initiator import utils as brick_utils
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
@@ -24,7 +25,6 @@ from oslo_utils import excutils
 from nova import block_device
 import nova.conf
 from nova import exception
-from nova import utils
 
 CONF = nova.conf.CONF
 
@@ -83,9 +83,8 @@ class DriverBlockDevice(dict):
 
     Uses block device objects internally to do the database access.
 
-    _fields and _legacy_fields class attributes present a set of fields that
-    are expected on a certain DriverBlockDevice type. We may have more legacy
-    versions in the future.
+    The _fields class attribute present a set of fields that
+    are expected on a certain DriverBlockDevice type.
 
     If an attribute access is attempted for a name that is found in the
     _proxy_as_attr set, it will be proxied to the underlying object. This
@@ -103,9 +102,8 @@ class DriverBlockDevice(dict):
     """
 
     _fields = set()
-    _legacy_fields = set()
 
-    _proxy_as_attr_inherited = set(['uuid'])
+    _proxy_as_attr_inherited = set(['uuid', 'is_volume'])
     _update_on_save = {'disk_bus': None,
                        'device_name': None,
                        'device_type': None}
@@ -144,26 +142,40 @@ class DriverBlockDevice(dict):
     def __getattr__(self, name):
         if name in self._proxy_as_attr:
             return getattr(self._bdm_obj, name)
+        elif name in self._fields:
+            return self[name]
         else:
-            super(DriverBlockDevice, self).__getattr__(name)
+            return super(DriverBlockDevice, self).__getattr__(name)
 
     def __setattr__(self, name, value):
         if name in self._proxy_as_attr:
-            return setattr(self._bdm_obj, name, value)
+            setattr(self._bdm_obj, name, value)
+        elif name in self._fields:
+            self[name] = value
         else:
             super(DriverBlockDevice, self).__setattr__(name, value)
 
+    def __getitem__(self, name):
+        if name in self._proxy_as_attr:
+            return getattr(self._bdm_obj, name)
+        return super(DriverBlockDevice, self).__getitem__(name)
+
+    def __setitem__(self, name, value):
+        if name in self._proxy_as_attr:
+            setattr(self._bdm_obj, name, value)
+        super(DriverBlockDevice, self).__setitem__(name, value)
+
     def _transform(self):
         """Transform bdm to the format that is passed to drivers."""
         raise NotImplementedError()
 
-    def legacy(self):
-        """Basic legacy transformation.
-
-        Basic method will just drop the fields that are not in
-        _legacy_fields set. Override this in subclass if needed.
-        """
-        return {key: self.get(key) for key in self._legacy_fields}
+    def get(self, name, default=None):
+        if name in self._proxy_as_attr:
+            return getattr(self._bdm_obj, name)
+        elif name in self._fields:
+            return self[name]
+        else:
+            return super(DriverBlockDevice, self).get(name, default)
 
     def attach(self, **kwargs):
         """Make the device available to be used by VMs.
@@ -201,7 +213,6 @@ class DriverBlockDevice(dict):
 
 class DriverSwapBlockDevice(DriverBlockDevice):
     _fields = set(['device_name', 'swap_size', 'disk_bus'])
-    _legacy_fields = _fields - set(['disk_bus'])
 
     _update_on_save = {'disk_bus': None,
                        'device_name': None}
@@ -216,11 +227,70 @@ class DriverSwapBlockDevice(DriverBlockDevice):
         })
 
 
+class DriverImageBlockDevice(DriverBlockDevice):
+    _valid_source = 'image'
+    _proxy_as_attr_inherited = set(['image_id'])
+    _new_only_fields = set([
+        'disk_bus',
+        'device_type',
+        'guest_format',
+        'boot_index',
+        'encrypted',
+        'encryption_secret_uuid',
+        'encryption_format',
+        'encryption_options'
+    ])
+    _fields = set([
+        'device_name',
+        'size']) | _new_only_fields
+    _legacy_fields = (
+        _fields - _new_only_fields | set(['num', 'virtual_name']))
+    _update_on_save = {
+        'disk_bus': None,
+        'device_name': None,
+        'device_type': None,
+        'encryption_secret_uuid': None,
+        'encryption_format': None,
+        'encryption_options': None,
+    }
+
+    def _transform(self):
+        if (not self._bdm_obj.get('source_type') == 'image' or
+            not self._bdm_obj.get('destination_type') == 'local'):
+            raise _InvalidType
+        self.update({
+            'device_name': self._bdm_obj.device_name,
+            'size': self._bdm_obj.volume_size or 0,
+            'disk_bus': self._bdm_obj.disk_bus,
+            'device_type': self._bdm_obj.device_type,
+            'guest_format': self._bdm_obj.guest_format,
+            'image_id': self._bdm_obj.image_id,
+            'boot_index': 0,
+            'encrypted': self._bdm_obj.encrypted,
+            'encryption_secret_uuid': self._bdm_obj.encryption_secret_uuid,
+            'encryption_format': self._bdm_obj.encryption_format,
+            'encryption_options': self._bdm_obj.encryption_options
+        })
+
+
 class DriverEphemeralBlockDevice(DriverBlockDevice):
-    _new_only_fields = set(['disk_bus', 'device_type', 'guest_format'])
+    _new_only_fields = set([
+        'disk_bus',
+        'device_type',
+        'guest_format',
+        'encrypted',
+        'encryption_secret_uuid',
+        'encryption_format',
+        'encryption_options'])
     _fields = set(['device_name', 'size']) | _new_only_fields
-    _legacy_fields = (_fields - _new_only_fields |
-                      set(['num', 'virtual_name']))
+    _update_on_save = {
+        'disk_bus': None,
+        'device_name': None,
+        'device_type': None,
+        'encryption_secret_uuid': None,
+        'encryption_format': None,
+        'encryption_options': None,
+    }
 
     def _transform(self):
         if not block_device.new_format_is_ephemeral(self._bdm_obj):
@@ -230,36 +300,40 @@ class DriverEphemeralBlockDevice(DriverBlockDevice):
             'size': self._bdm_obj.volume_size or 0,
             'disk_bus': self._bdm_obj.disk_bus,
             'device_type': self._bdm_obj.device_type,
-            'guest_format': self._bdm_obj.guest_format
+            'guest_format': self._bdm_obj.guest_format,
+            'encrypted': self._bdm_obj.encrypted,
+            'encryption_secret_uuid': self._bdm_obj.encryption_secret_uuid,
+            'encryption_format': self._bdm_obj.encryption_format,
+            'encryption_options': self._bdm_obj.encryption_options
         })
 
-    def legacy(self, num=0):
-        legacy_bdm = super(DriverEphemeralBlockDevice, self).legacy()
-        legacy_bdm['num'] = num
-        legacy_bdm['virtual_name'] = 'ephemeral' + str(num)
-        return legacy_bdm
-
 
 class DriverVolumeBlockDevice(DriverBlockDevice):
-    _legacy_fields = set(['connection_info', 'mount_device',
-                          'delete_on_termination'])
-    _new_fields = set(['guest_format', 'device_type',
-                       'disk_bus', 'boot_index',
-                       'attachment_id'])
-    _fields = _legacy_fields | _new_fields
+    _new_fields = set([
+        'guest_format',
+        'device_type',
+        'disk_bus',
+        'boot_index',
+        'attachment_id'])
+    _fields = set([
+        'connection_info',
+        'mount_device',
+        'delete_on_termination'
+    ]) | _new_fields
 
     _valid_source = 'volume'
     _valid_destination = 'volume'
 
-    _proxy_as_attr_inherited = set(['volume_size', 'volume_id'])
+    _proxy_as_attr_inherited = set(['volume_size', 'volume_id', 'volume_type'])
     _update_on_save = {'disk_bus': None,
                        'device_name': 'mount_device',
-                       'device_type': None}
+                       'device_type': None,
+                       # needed for boot from volume for blank/image/snapshot
+                       'attachment_id': None}
 
     def _transform(self):
-        if (not self._bdm_obj.source_type == self._valid_source
-                or not self._bdm_obj.destination_type ==
-                self._valid_destination):
+        if (not self._bdm_obj.source_type == self._valid_source or
+                not self._bdm_obj.destination_type == self._valid_destination):
             raise _InvalidType
 
         self.update(
@@ -267,11 +341,17 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
              if k in self._new_fields | set(['delete_on_termination'])}
         )
         self['mount_device'] = self._bdm_obj.device_name
+        # connection_info might not be set so default to an empty dict so that
+        # it can be serialized to an empty JSON object.
         try:
             self['connection_info'] = jsonutils.loads(
                 self._bdm_obj.connection_info)
         except TypeError:
-            self['connection_info'] = None
+            self['connection_info'] = {}
+        # volume_type might not be set on the internal bdm object so default
+        # to None if not set
+        self['volume_type'] = (
+            self.volume_type if 'volume_type' in self._bdm_obj else None)
 
     def _preserve_multipath_id(self, connection_info):
         if self['connection_info'] and 'data' in self['connection_info']:
@@ -304,10 +384,11 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
                         '%(mp)s : %(err)s',
                         {'volume_id': volume_id, 'mp': mp,
                          'err': err}, instance=instance)
-        except exception.DeviceDetachFailed as err:
+        except exception.DeviceDetachFailed:
             with excutils.save_and_reraise_exception():
                 LOG.warning('Guest refused to detach volume %(vol)s',
                             {'vol': volume_id}, instance=instance)
+                volume_api.roll_detaching(context, volume_id)
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception('Failed to detach volume '
@@ -326,6 +407,43 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
         except exception.CinderAPIVersionNotAvailable:
             return volume_api.get(context, volume_id)
 
+    def _create_volume(self, context, instance, volume_api, size,
+                       wait_func=None, **create_kwargs):
+        """Create a volume and attachment record.
+
+        :param context: nova auth RequestContext
+        :param instance: Instance object to which the created volume will be
+            attached.
+        :param volume_api: nova.volume.cinder.API instance
+        :param size: The size of the volume to create in GiB.
+        :param wait_func: Optional callback function to wait for the volume to
+            reach some status before continuing with a signature of::
+
+                wait_func(context, volume_id)
+        :param create_kwargs: Additional optional parameters used to create the
+            volume. See nova.volume.cinder.API.create for keys.
+        :return: A two-item tuple of volume ID and attachment ID.
+        """
+        av_zone = _get_volume_create_az_value(instance)
+        name = create_kwargs.pop('name', '')
+        description = create_kwargs.pop('description', '')
+        vol = volume_api.create(
+            context, size, name, description, volume_type=self.volume_type,
+            availability_zone=av_zone, **create_kwargs)
+
+        if wait_func:
+            self._call_wait_func(context, wait_func, volume_api, vol['id'])
+
+        # Unconditionally create an attachment record for the volume so the
+        # attach/detach flows use the "new style" introduced in Queens. Note
+        # that nova required the Cinder Queens level APIs (3.44+) starting in
+        # Train.
+        attachment_id = (
+            volume_api.attachment_create(
+                context, vol['id'], instance.uuid)['id'])
+
+        return vol['id'], attachment_id
+
     def _do_detach(self, context, instance, volume_api, virt_driver,
                    attachment_id=None, destroy_bdm=False):
         """Private method that actually does the detach.
@@ -397,24 +515,26 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
             volume_api.detach(context.elevated(), volume_id, instance.uuid,
                               attachment_id)
         else:
-            volume_api.attachment_delete(context, self['attachment_id'])
+            try:
+                volume_api.attachment_delete(context, self['attachment_id'])
+            except exception.VolumeAttachmentNotFound:
+                LOG.info(
+                    "Ignoring a volume attachment deletion failure as the "
+                    "volume %(volume_id)s or the volume attachment "
+                    "%(attachment_id)s disappeared during the request.",
+                    {
+                        'volume_id': volume_id,
+                        'attachment_id': self['attachment_id']
+                    }
+                )
 
     def detach(self, context, instance, volume_api, virt_driver,
                attachment_id=None, destroy_bdm=False):
         volume = self._get_volume(context, volume_api, self.volume_id)
-        # Check to see if we need to lock based on the shared_targets value.
-        # Default to False if the volume does not expose that value to maintain
-        # legacy behavior.
-        if volume.get('shared_targets', False):
-            # Lock the detach call using the provided service_uuid.
-            @utils.synchronized(volume['service_uuid'])
-            def _do_locked_detach(*args, **_kwargs):
-                self._do_detach(*args, **_kwargs)
-
-            _do_locked_detach(context, instance, volume_api, virt_driver,
-                              attachment_id, destroy_bdm)
-        else:
-            # We don't need to (or don't know if we need to) lock.
+        # Let OS-Brick handle high level locking that covers the local os-brick
+        # detach and the Cinder call to call unmap the volume.  Not all volume
+        # backends or hosts require locking.
+        with brick_utils.guard_connection(volume):
             self._do_detach(context, instance, volume_api, virt_driver,
                             attachment_id, destroy_bdm)
 
@@ -470,7 +590,8 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
                 with excutils.save_and_reraise_exception():
                     if do_driver_attach:
                         try:
-                            virt_driver.detach_volume(connection_info,
+                            virt_driver.detach_volume(context,
+                                                      connection_info,
                                                       instance,
                                                       self['mount_device'],
                                                       encryption=encryption)
@@ -500,7 +621,8 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
             self.volume_size = volume.get('size')
 
         vol_multiattach = volume.get('multiattach', False)
-        virt_multiattach = virt_driver.capabilities['supports_multiattach']
+        virt_multiattach = virt_driver.capabilities.get(
+            'supports_multiattach', False)
 
         if vol_multiattach and not virt_multiattach:
             raise exception.MultiattachNotSupportedByVirtDriver(
@@ -566,7 +688,8 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
                 if do_driver_attach:
                     # Disconnect the volume from the host.
                     try:
-                        virt_driver.detach_volume(connection_info,
+                        virt_driver.detach_volume(context,
+                                                  connection_info,
                                                   instance,
                                                   self['mount_device'],
                                                   encryption=encryption)
@@ -604,19 +727,10 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
         volume = self._get_volume(context, volume_api, self.volume_id)
         volume_api.check_availability_zone(context, volume,
                                            instance=instance)
-        # Check to see if we need to lock based on the shared_targets value.
-        # Default to False if the volume does not expose that value to maintain
-        # legacy behavior.
-        if volume.get('shared_targets', False):
-            # Lock the attach call using the provided service_uuid.
-            @utils.synchronized(volume['service_uuid'])
-            def _do_locked_attach(*args, **_kwargs):
-                self._do_attach(*args, **_kwargs)
-
-            _do_locked_attach(context, instance, volume, volume_api,
-                              virt_driver, do_driver_attach)
-        else:
-            # We don't need to (or don't know if we need to) lock.
+        # Let OS-Brick handle high level locking that covers the call to
+        # Cinder that exports & maps the volume, and for the local os-brick
+        # attach.  Not all volume backends or hosts require locking.
+        with brick_utils.guard_connection(volume):
             self._do_attach(context, instance, volume, volume_api,
                             virt_driver, do_driver_attach)
 
@@ -679,7 +793,7 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
                             {'volume_id': volume_id, 'exc': exc})
 
 
-class DriverSnapshotBlockDevice(DriverVolumeBlockDevice):
+class DriverVolSnapshotBlockDevice(DriverVolumeBlockDevice):
 
     _valid_source = 'snapshot'
     _proxy_as_attr_inherited = set(['snapshot_id'])
@@ -688,25 +802,25 @@ class DriverSnapshotBlockDevice(DriverVolumeBlockDevice):
                virt_driver, wait_func=None):
 
         if not self.volume_id:
-            av_zone = _get_volume_create_az_value(instance)
             snapshot = volume_api.get_snapshot(context,
                                                self.snapshot_id)
-            vol = volume_api.create(context, self.volume_size, '', '',
-                                    snapshot, availability_zone=av_zone)
-            if wait_func:
-                self._call_wait_func(context, wait_func, volume_api, vol['id'])
+            # NOTE(lyarwood): Try to use the original volume type if one isn't
+            # set against the bdm but is on the original volume.
+            if not self.volume_type and snapshot.get('volume_id'):
+                snap_volume_id = snapshot.get('volume_id')
+                orig_volume = volume_api.get(context, snap_volume_id)
+                self.volume_type = orig_volume.get('volume_type_id')
 
-            self.volume_id = vol['id']
-
-            # TODO(mriedem): Create an attachment to reserve the volume and
-            # make us go down the new-style attach flow.
+            self.volume_id, self.attachment_id = self._create_volume(
+                context, instance, volume_api, self.volume_size,
+                wait_func=wait_func, snapshot=snapshot)
 
         # Call the volume attach now
-        super(DriverSnapshotBlockDevice, self).attach(
+        super(DriverVolSnapshotBlockDevice, self).attach(
             context, instance, volume_api, virt_driver)
 
 
-class DriverImageBlockDevice(DriverVolumeBlockDevice):
+class DriverVolImageBlockDevice(DriverVolumeBlockDevice):
 
     _valid_source = 'image'
     _proxy_as_attr_inherited = set(['image_id'])
@@ -714,23 +828,15 @@ class DriverImageBlockDevice(DriverVolumeBlockDevice):
     def attach(self, context, instance, volume_api,
                virt_driver, wait_func=None):
         if not self.volume_id:
-            av_zone = _get_volume_create_az_value(instance)
-            vol = volume_api.create(context, self.volume_size,
-                                    '', '', image_id=self.image_id,
-                                    availability_zone=av_zone)
-            if wait_func:
-                self._call_wait_func(context, wait_func, volume_api, vol['id'])
+            self.volume_id, self.attachment_id = self._create_volume(
+                context, instance, volume_api, self.volume_size,
+                wait_func=wait_func, image_id=self.image_id)
 
-            self.volume_id = vol['id']
-
-            # TODO(mriedem): Create an attachment to reserve the volume and
-            # make us go down the new-style attach flow.
-
-        super(DriverImageBlockDevice, self).attach(
+        super(DriverVolImageBlockDevice, self).attach(
             context, instance, volume_api, virt_driver)
 
 
-class DriverBlankBlockDevice(DriverVolumeBlockDevice):
+class DriverVolBlankBlockDevice(DriverVolumeBlockDevice):
 
     _valid_source = 'blank'
     _proxy_as_attr_inherited = set(['image_id'])
@@ -739,18 +845,11 @@ class DriverBlankBlockDevice(DriverVolumeBlockDevice):
                virt_driver, wait_func=None):
         if not self.volume_id:
             vol_name = instance.uuid + '-blank-vol'
-            av_zone = _get_volume_create_az_value(instance)
-            vol = volume_api.create(context, self.volume_size, vol_name, '',
-                                    availability_zone=av_zone)
-            if wait_func:
-                self._call_wait_func(context, wait_func, volume_api, vol['id'])
+            self.volume_id, self.attachment_id = self._create_volume(
+                context, instance, volume_api, self.volume_size,
+                wait_func=wait_func, name=vol_name)
 
-            self.volume_id = vol['id']
-
-            # TODO(mriedem): Create an attachment to reserve the volume and
-            # make us go down the new-style attach flow.
-
-        super(DriverBlankBlockDevice, self).attach(
+        super(DriverVolBlankBlockDevice, self).attach(
             context, instance, volume_api, virt_driver)
 
 
@@ -768,23 +867,23 @@ def _convert_block_devices(device_type, block_device_mapping):
 convert_swap = functools.partial(_convert_block_devices,
                                  DriverSwapBlockDevice)
 
+convert_local_images = functools.partial(_convert_block_devices,
+                                         DriverImageBlockDevice)
 
 convert_ephemerals = functools.partial(_convert_block_devices,
                                       DriverEphemeralBlockDevice)
 
-
 convert_volumes = functools.partial(_convert_block_devices,
                                    DriverVolumeBlockDevice)
 
-
 convert_snapshots = functools.partial(_convert_block_devices,
-                                     DriverSnapshotBlockDevice)
+                                     DriverVolSnapshotBlockDevice)
 
 convert_images = functools.partial(_convert_block_devices,
-                                     DriverImageBlockDevice)
+                                     DriverVolImageBlockDevice)
 
 convert_blanks = functools.partial(_convert_block_devices,
-                                   DriverBlankBlockDevice)
+                                   DriverVolBlankBlockDevice)
 
 
 def convert_all_volumes(*volume_bdms):
@@ -847,19 +946,6 @@ def refresh_conn_infos(block_device_mapping, *refresh_args, **refresh_kwargs):
     return block_device_mapping
 
 
-def legacy_block_devices(block_device_mapping):
-    bdms = [bdm.legacy() for bdm in block_device_mapping]
-
-    # Re-enumerate ephemeral devices
-    if all(isinstance(bdm, DriverEphemeralBlockDevice)
-           for bdm in block_device_mapping):
-        for i, dev in enumerate(bdms):
-            dev['virtual_name'] = dev['virtual_name'][:-1] + str(i)
-            dev['num'] = i
-
-    return bdms
-
-
 def get_swap(transformed_list):
     """Get the swap device out of the list context.
 
@@ -876,9 +962,15 @@ def get_swap(transformed_list):
         return None
 
 
-_IMPLEMENTED_CLASSES = (DriverSwapBlockDevice, DriverEphemeralBlockDevice,
-                        DriverVolumeBlockDevice, DriverSnapshotBlockDevice,
-                        DriverImageBlockDevice, DriverBlankBlockDevice)
+_IMPLEMENTED_CLASSES = (
+    DriverSwapBlockDevice,
+    DriverEphemeralBlockDevice,
+    DriverVolumeBlockDevice,
+    DriverVolSnapshotBlockDevice,
+    DriverVolImageBlockDevice,
+    DriverVolBlankBlockDevice,
+    DriverImageBlockDevice
+)
 
 
 def is_implemented(bdm):
@@ -891,10 +983,14 @@ def is_implemented(bdm):
     return False
 
 
+def is_local_image(bdm):
+    return bdm.source_type == 'image' and bdm.destination_type == 'local'
+
+
 def is_block_device_mapping(bdm):
-    return (bdm.source_type in ('image', 'volume', 'snapshot', 'blank')
-            and bdm.destination_type == 'volume'
-            and is_implemented(bdm))
+    return (bdm.source_type in ('image', 'volume', 'snapshot', 'blank') and
+            bdm.destination_type == 'volume' and
+            is_implemented(bdm))
 
 
 def get_volume_id(connection_info):
