@@ -20,13 +20,14 @@ Driver base-classes:
     types that support that contract
 """
 
+import itertools
 import sys
+import typing as ty
 
 import os_resource_classes as orc
 import os_traits
 from oslo_log import log as logging
 from oslo_utils import importutils
-import six
 
 import nova.conf
 from nova import context as nova_context
@@ -45,6 +46,7 @@ def get_block_device_info(instance, block_device_mapping):
     of a dict containing the following keys:
 
     - root_device_name: device name of the root disk
+    - image: An instance of DriverImageBlockDevice or None
     - ephemerals: a (potentially empty) list of DriverEphemeralBlockDevice
                   instances
     - swap: An instance of DriverSwapBlockDevice or None
@@ -53,18 +55,18 @@ def get_block_device_info(instance, block_device_mapping):
                             specialized subclasses.
     """
     from nova.virt import block_device as virt_block_device
-
-    block_device_info = {
+    return {
         'root_device_name': instance.root_device_name,
+        'image': virt_block_device.convert_local_images(
+            block_device_mapping),
         'ephemerals': virt_block_device.convert_ephemerals(
             block_device_mapping),
         'block_device_mapping':
-            virt_block_device.convert_all_volumes(*block_device_mapping)
+            virt_block_device.convert_all_volumes(*block_device_mapping),
+        'swap':
+            virt_block_device.get_swap(
+                virt_block_device.convert_swap(block_device_mapping))
     }
-    swap_list = virt_block_device.convert_swap(block_device_mapping)
-    block_device_info['swap'] = virt_block_device.get_swap(swap_list)
-
-    return block_device_info
 
 
 def block_device_info_get_root_device(block_device_info):
@@ -82,6 +84,14 @@ def swap_is_usable(swap):
     return swap and swap['device_name'] and swap['swap_size'] > 0
 
 
+def block_device_info_get_image(block_device_info):
+    block_device_info = block_device_info or {}
+    # get_disk_mapping() supports block_device_info=None and thus requires that
+    # we return a list here.
+    image = block_device_info.get('image') or []
+    return image
+
+
 def block_device_info_get_ephemerals(block_device_info):
     block_device_info = block_device_info or {}
     ephemerals = block_device_info.get('ephemerals') or []
@@ -94,6 +104,19 @@ def block_device_info_get_mapping(block_device_info):
     return block_device_mapping
 
 
+def block_device_info_get_encrypted_disks(
+    block_device_info: ty.Mapping[str, ty.Any],
+) -> ty.List['nova.virt.block_device.DriverBlockDevice']:
+    block_device_info = block_device_info or {}
+    return [
+        driver_bdm for driver_bdm in itertools.chain(
+            block_device_info.get('image', []),
+            block_device_info.get('ephemerals', []),
+        )
+        if driver_bdm.get('encrypted')
+    ]
+
+
 # NOTE(aspiers): When adding new capabilities, ensure they are
 # mirrored in ComputeDriver.capabilities, and that the corresponding
 # values should always be standard traits in os_traits.  If something
@@ -102,7 +125,6 @@ def block_device_info_get_mapping(block_device_info):
 # trait, it needs to be (made) standard, and must be prefixed with
 # "COMPUTE_".
 CAPABILITY_TRAITS_MAP = {
-    # Added in os-traits 0.7.0.
     "supports_attach_interface": os_traits.COMPUTE_NET_ATTACH_INTERFACE,
     "supports_device_tagging": os_traits.COMPUTE_DEVICE_TAGGING,
     "supports_tagged_attach_interface":
@@ -110,11 +132,8 @@ CAPABILITY_TRAITS_MAP = {
     "supports_tagged_attach_volume": os_traits.COMPUTE_VOLUME_ATTACH_WITH_TAG,
     "supports_extend_volume": os_traits.COMPUTE_VOLUME_EXTEND,
     "supports_multiattach": os_traits.COMPUTE_VOLUME_MULTI_ATTACH,
-    # Added in os-traits 0.8.0.
     "supports_trusted_certs": os_traits.COMPUTE_TRUSTED_CERTS,
     "supports_accelerators": os_traits.COMPUTE_ACCELERATORS,
-
-    # Image type support flags, added in os-traits 0.12.0
     "supports_image_type_aki": os_traits.COMPUTE_IMAGE_TYPE_AKI,
     "supports_image_type_ami": os_traits.COMPUTE_IMAGE_TYPE_AMI,
     "supports_image_type_ari": os_traits.COMPUTE_IMAGE_TYPE_ARI,
@@ -125,15 +144,35 @@ CAPABILITY_TRAITS_MAP = {
     "supports_image_type_vhd": os_traits.COMPUTE_IMAGE_TYPE_VHD,
     "supports_image_type_vhdx": os_traits.COMPUTE_IMAGE_TYPE_VHDX,
     "supports_image_type_vmdk": os_traits.COMPUTE_IMAGE_TYPE_VMDK,
-    # Added in os-traits 2.0.0
     "supports_image_type_ploop": os_traits.COMPUTE_IMAGE_TYPE_PLOOP,
-    # Added in os-traits 2.1.0.
     "supports_migrate_to_same_host": os_traits.COMPUTE_SAME_HOST_COLD_MIGRATE,
-    # Added in os-traits 2.2.0.
     "supports_bfv_rescue": os_traits.COMPUTE_RESCUE_BFV,
+    "supports_secure_boot": os_traits.COMPUTE_SECURITY_UEFI_SECURE_BOOT,
+    "supports_socket_pci_numa_affinity":
+        os_traits.COMPUTE_SOCKET_PCI_NUMA_AFFINITY,
+    "supports_remote_managed_ports": os_traits.COMPUTE_REMOTE_MANAGED_PORTS,
+    "supports_ephemeral_encryption": os_traits.COMPUTE_EPHEMERAL_ENCRYPTION,
+    "supports_ephemeral_encryption_luks":
+        os_traits.COMPUTE_EPHEMERAL_ENCRYPTION_LUKS,
+    "supports_ephemeral_encryption_plain":
+        os_traits.COMPUTE_EPHEMERAL_ENCRYPTION_PLAIN,
 }
 
 
+def _check_image_type_exclude_list(capability, supported):
+    """Enforce the exclusion list on image_type capabilities.
+
+    :param capability: The supports_image_type_foo capability being checked
+    :param supported: The flag indicating whether the virt driver *can*
+                      support the given image type.
+    :returns: True if the virt driver *can* support the image type and
+              if it is not listed in the config to be excluded.
+    """
+    image_type = capability.replace('supports_image_type_', '')
+    return (supported and
+            image_type not in CONF.compute.image_type_exclude_list)
+
+
 class ComputeDriver(object):
     """Base class for compute drivers.
 
@@ -183,6 +222,14 @@ class ComputeDriver(object):
         "supports_accelerators": False,
         "supports_bfv_rescue": False,
         "supports_vtpm": False,
+        "supports_secure_boot": False,
+        "supports_socket_pci_numa_affinity": False,
+        "supports_remote_managed_ports": False,
+
+        # Ephemeral encryption support flags
+        "supports_ephemeral_encryption": False,
+        "supports_ephemeral_encryption_luks": False,
+        "supports_ephemeral_encryption_plain": False,
 
         # Image type support flags
         "supports_image_type_aki": False,
@@ -286,7 +333,8 @@ class ComputeDriver(object):
                 admin_password, allocations, bdms, detach_block_devices,
                 attach_block_devices, network_info=None,
                 evacuate=False, block_device_info=None,
-                preserve_ephemeral=False, accel_uuids=None):
+                preserve_ephemeral=False, accel_uuids=None,
+                reimage_boot_volume=False):
         """Destroy and re-make this instance.
 
         A 'rebuild' effectively purges all existing data from the system and
@@ -324,6 +372,7 @@ class ComputeDriver(object):
         :param preserve_ephemeral: True if the default ephemeral storage
                                    partition must be preserved on rebuild
         :param accel_uuids: Accelerator UUIDs.
+        :param reimage_boot_volume: Re-image the volume backed instance.
         """
         raise NotImplementedError()
 
@@ -404,7 +453,7 @@ class ComputeDriver(object):
         raise NotImplementedError()
 
     def destroy(self, context, instance, network_info, block_device_info=None,
-                destroy_disks=True):
+                destroy_disks=True, destroy_secrets=True):
         """Destroy the specified instance from the Hypervisor.
 
         If the instance is not found (for example if networking failed), this
@@ -417,11 +466,13 @@ class ComputeDriver(object):
         :param block_device_info: Information about block devices that should
                                   be detached from the instance.
         :param destroy_disks: Indicates if disks should be destroyed
+        :param destroy_secrets: Indicates if secrets should be destroyed
         """
         raise NotImplementedError()
 
     def cleanup(self, context, instance, network_info, block_device_info=None,
-                destroy_disks=True, migrate_data=None, destroy_vifs=True):
+                destroy_disks=True, migrate_data=None, destroy_vifs=True,
+                destroy_secrets=True):
         """Cleanup the instance resources .
 
         Instance should have been destroyed from the Hypervisor before calling
@@ -434,6 +485,8 @@ class ComputeDriver(object):
                                   be detached from the instance.
         :param destroy_disks: Indicates if disks should be destroyed
         :param migrate_data: implementation specific params
+        :param destroy_vifs: Indicates if vifs should be unplugged
+        :param destroy_secrets: Indicates if secrets should be destroyed
         """
         raise NotImplementedError()
 
@@ -459,12 +512,6 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def get_console_pool_info(self, console_type):
-        # TODO(Vek): Need to pass context in for access to auth_token
-        raise NotImplementedError()
-
     def get_console_output(self, context, instance):
         """Get console output for an instance
 
@@ -548,16 +595,6 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def get_all_bw_counters(self, instances):
-        """Return bandwidth usage counters for each interface on each
-           running VM.
-
-        :param instances: nova.objects.instance.InstanceList
-        """
-        raise NotImplementedError()
-
     def get_all_volume_usage(self, context, compute_host_bdms):
         """Return usage info for volumes attached to vms on
            a given host.-
@@ -730,16 +767,6 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def post_interrupted_snapshot_cleanup(self, context, instance):
-        """Cleans up any resources left after an interrupted snapshot.
-
-        :param context: security context
-        :param instance: nova.objects.instance.Instance
-        """
-        pass
-
     def finish_migration(self, context, migration, instance, disk_info,
                          network_info, image_meta, resize_instance,
                          allocations, block_device_info=None, power_on=True):
@@ -882,15 +909,6 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def set_bootable(self, instance, is_bootable):
-        """Set the ability to power on/off an instance.
-
-        :param instance: nova.objects.instance.Instance
-        """
-        raise NotImplementedError()
-
     def unrescue(
         self,
         context: nova_context.RequestContext,
@@ -1113,6 +1131,10 @@ class ComputeDriver(object):
         """
         traits = {}
         for capability, supported in self.capabilities.items():
+            if capability.startswith('supports_image_type_'):
+                supported = _check_image_type_exclude_list(capability,
+                                                           supported)
+
             if capability in CAPABILITY_TRAITS_MAP:
                 traits[CAPABILITY_TRAITS_MAP[capability]] = supported
 
@@ -1154,7 +1176,7 @@ class ComputeDriver(object):
 
         :param context: security context
         :param instance:
-            nova.db.sqlalchemy.models.Instance object
+            nova.db.main.models.Instance object
             instance object that is migrated.
         :param dest: destination host
         :param post_method:
@@ -1284,7 +1306,7 @@ class ComputeDriver(object):
         back to the source host to check the results.
 
         :param context: security context
-        :param instance: nova.db.sqlalchemy.models.Instance
+        :param instance: nova.db.main.models.Instance
         :param src_compute_info: Info about the sending machine
         :param dst_compute_info: Info about the receiving machine
         :param block_migration: if true, prepare for block migration
@@ -1326,7 +1348,7 @@ class ComputeDriver(object):
         results from check_can_live_migrate_destination.
 
         :param context: security context
-        :param instance: nova.db.sqlalchemy.models.Instance
+        :param instance: nova.db.main.models.Instance
         :param dest_check_data: result of check_can_live_migrate_destination
         :param block_device_info: result of _get_instance_block_device_info
         :returns: a LiveMigrateData object
@@ -1354,13 +1376,6 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def reset_network(self, instance):
-        """reset networking for specified instance."""
-        # TODO(Vek): Need to pass context in for access to auth_token
-        pass
-
     def set_admin_password(self, instance, new_pass):
         """Set the root password on the specified instance.
 
@@ -1369,21 +1384,6 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def change_instance_metadata(self, context, instance, diff):
-        """Applies a diff to the instance metadata.
-
-        This is an optional driver method which is used to publish
-        changes to the instance's metadata to the hypervisor.  If the
-        hypervisor has no means of publishing the instance metadata to
-        the instance, then this method should not be implemented.
-
-        :param context: security context
-        :param instance: nova.objects.instance.Instance
-        """
-        pass
-
     def inject_network_info(self, instance, nw_info):
         """inject network info for specified instance."""
         # TODO(Vek): Need to pass context in for access to auth_token
@@ -1569,54 +1569,6 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def add_to_aggregate(self, context, aggregate, host, **kwargs):
-        """Add a compute host to an aggregate.
-
-        The counter action to this is :func:`remove_from_aggregate`
-
-        :param nova.context.RequestContext context:
-            The security context.
-        :param nova.objects.aggregate.Aggregate aggregate:
-            The aggregate which should add the given `host`
-        :param str host:
-            The name of the host to add to the given `aggregate`.
-        :param dict kwargs:
-            A free-form thingy...
-
-        :return: None
-        """
-        # NOTE(jogo) Currently only used for XenAPI-Pool
-        raise NotImplementedError()
-
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def remove_from_aggregate(self, context, aggregate, host, **kwargs):
-        """Remove a compute host from an aggregate.
-
-        The counter action to this is :func:`add_to_aggregate`
-
-        :param nova.context.RequestContext context:
-            The security context.
-        :param nova.objects.aggregate.Aggregate aggregate:
-            The aggregate which should remove the given `host`
-        :param str host:
-            The name of the host to remove from the given `aggregate`.
-        :param dict kwargs:
-            A free-form thingy...
-
-        :return: None
-        """
-        raise NotImplementedError()
-
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def undo_aggregate_operation(self, context, op, aggregate,
-                                  host, set_error=True):
-        """Undo for Resource Pools."""
-        raise NotImplementedError()
-
     def get_volume_connector(self, instance):
         """Get connector information for the instance for attaching to volumes.
 
@@ -1650,15 +1602,6 @@ class ComputeDriver(object):
         # Refresh and check again.
         return nodename in self.get_available_nodes(refresh=True)
 
-    # TODO(stephenfin): This was only implemented (properly) for XenAPI and
-    # should be removed.
-    def get_per_instance_usage(self):
-        """Get information about instance resource usage.
-
-        :returns: dict of  nova uuid => dict of usage info
-        """
-        return {}
-
     def instance_on_disk(self, instance):
         """Checks access of instance files on the host.
 
@@ -1693,7 +1636,7 @@ class ComputeDriver(object):
         """
 
         if not self._compute_event_callback:
-            LOG.debug("Discarding event %s", six.text_type(event))
+            LOG.debug("Discarding event %s", str(event))
             return
 
         if not isinstance(event, virtevent.Event):
@@ -1701,7 +1644,7 @@ class ComputeDriver(object):
                 _("Event must be an instance of nova.virt.event.Event"))
 
         try:
-            LOG.debug("Emitting event %s", six.text_type(event))
+            LOG.debug("Emitting event %s", str(event))
             self._compute_event_callback(event)
         except Exception as ex:
             LOG.error("Exception dispatching event %(event)s: %(ex)s",
@@ -1715,15 +1658,6 @@ class ComputeDriver(object):
         """
         return True
 
-    @property
-    def need_legacy_block_device_info(self):
-        """Tell the caller if the driver requires legacy block device info.
-
-        Tell the caller whether we expect the legacy format of block
-        device info to be passed in to methods that expect it.
-        """
-        return True
-
     def volume_snapshot_create(self, context, instance, volume_id,
                                create_info):
         """Snapshots volumes attached to a specified instance.
