@@ -116,6 +116,7 @@ class FakeDriver(driver.ComputeDriver):
         "supports_trusted_certs": True,
         "supports_pcpus": False,
         "supports_accelerators": True,
+        "supports_remote_managed_ports": True,
 
         # Supported image types
         "supports_image_type_raw": True,
@@ -308,7 +309,7 @@ class FakeDriver(driver.ComputeDriver):
         pass
 
     def destroy(self, context, instance, network_info, block_device_info=None,
-                destroy_disks=True):
+                destroy_disks=True, destroy_secrets=True):
         key = instance.uuid
         if key in self.instances:
             flavor = instance.flavor
@@ -323,7 +324,8 @@ class FakeDriver(driver.ComputeDriver):
                          'inst': self.instances}, instance=instance)
 
     def cleanup(self, context, instance, network_info, block_device_info=None,
-                destroy_disks=True, migrate_data=None, destroy_vifs=True):
+                destroy_disks=True, migrate_data=None, destroy_vifs=True,
+                destroy_secrets=True):
         # cleanup() should not be called when the guest has not been destroyed.
         if instance.uuid in self.instances:
             raise exception.InstanceExists(
@@ -420,18 +422,6 @@ class FakeDriver(driver.ComputeDriver):
             maximum=524288, used=0)
         return diags
 
-    def get_all_bw_counters(self, instances):
-        """Return bandwidth usage counters for each interface on each
-           running VM.
-        """
-        bw = []
-        for instance in instances:
-            bw.append({'uuid': instance.uuid,
-                       'mac_address': 'fa:16:3e:4c:2c:30',
-                       'bw_in': 0,
-                       'bw_out': 0})
-        return bw
-
     def get_all_volume_usage(self, context, compute_host_bdms):
         """Return usage info for volumes attached to vms on
            a given host.
@@ -488,11 +478,6 @@ class FakeDriver(driver.ComputeDriver):
                                 host='fakemksconsole.com',
                                 port=6969)
 
-    def get_console_pool_info(self, console_type):
-        return {'address': '127.0.0.1',
-                'username': 'fakeuser',
-                'password': 'fakepassword'}
-
     def get_available_resource(self, nodename):
         """Updates compute manager resource info on ComputeNode table.
 
@@ -773,6 +758,7 @@ class PredictableNodeUUIDDriver(SmallFakeDriver):
     """SmallFakeDriver variant that reports a predictable node uuid in
     get_available_resource, like IronicDriver.
     """
+
     def get_available_resource(self, nodename):
         resources = super(
             PredictableNodeUUIDDriver, self).get_available_resource(nodename)
@@ -814,6 +800,7 @@ class FakeBuildAbortDriver(FakeDriver):
     """FakeDriver derivative that always fails on spawn() with a
     BuildAbortException so no reschedule is attempted.
     """
+
     def spawn(self, context, instance, image_meta, injected_files,
               admin_password, allocations, network_info=None,
               block_device_info=None, power_on=True, accel_info=None):
@@ -830,6 +817,7 @@ class FakeUnshelveSpawnFailDriver(FakeDriver):
     """FakeDriver derivative that always fails on spawn() with a
     VirtualInterfaceCreateException when unshelving an offloaded instance.
     """
+
     def spawn(self, context, instance, image_meta, injected_files,
               admin_password, allocations, network_info=None,
               block_device_info=None, power_on=True, accel_info=None):
@@ -903,6 +891,36 @@ class FakeLiveMigrateDriverWithNestedCustomResources(
 
 
 class FakeDriverWithPciResources(SmallFakeDriver):
+    """NOTE: this driver provides symmetric compute nodes. Each compute will
+    have the same resources with the same addresses. It is dangerous as using
+    this driver can hide issues when in an asymmetric environment nova fails to
+    update entities according to the host specific addresses (e.g. pci_slot of
+    the neutron port bindings).
+
+    The current non virt driver specific functional test environment has many
+    shortcomings making it really hard to simulate host specific virt drivers.
+
+    1) The virt driver is instantiated by the service logic from the name of
+    the driver class. This makes passing input to the driver instance from the
+    test at init time pretty impossible. This could be solved with some
+    fixtures around nova.virt.driver.load_compute_driver()
+
+    2) The compute service access the hypervisor not only via the virt
+    interface but also reads the sysfs of the host. So simply providing a fake
+    virt driver instance is not enough to isolate simulated compute services
+    that are running on the same host. Also these low level sysfs reads are not
+    having host specific information in the call params. So simply mocking the
+    low level call does not give a way to provide host specific return values.
+
+    3) CONF is global, and it is read dynamically by the driver. So
+    providing host specific CONF to driver instances without race conditions
+    between the drivers are extremely hard especially if periodic tasks are
+    enabled.
+
+    The libvirt based functional test env under nova.tests.functional.libvirt
+    has better support to create asymmetric environments. So please consider
+    using that if possible instead.
+    """
 
     PCI_ADDR_PF1 = '0000:01:00.0'
     PCI_ADDR_PF1_VF1 = '0000:01:00.1'
@@ -918,7 +936,7 @@ class FakeDriverWithPciResources(SmallFakeDriver):
         def setUp(self):
             super(FakeDriverWithPciResources.
                   FakeDriverWithPciResourcesConfigFixture, self).setUp()
-            # Set passthrough_whitelist before the compute node starts to match
+            # Set device_spec before the compute node starts to match
             # with the PCI devices reported by this fake driver.
 
             # NOTE(gibi): 0000:01:00 is tagged to physnet1 and therefore not a
@@ -933,7 +951,7 @@ class FakeDriverWithPciResources(SmallFakeDriver):
             # Having two PFs on the same physnet will allow us to test the
             # placement allocation - physical allocation matching based on the
             # bandwidth allocation in the future.
-            CONF.set_override('passthrough_whitelist', override=[
+            CONF.set_override('device_spec', override=[
                 jsonutils.dumps(
                     {
                         "address": {
@@ -967,6 +985,19 @@ class FakeDriverWithPciResources(SmallFakeDriver):
             ],
                              group='pci')
 
+            # These mocks should be removed after bug
+            # https://bugs.launchpad.net/nova/+bug/1961587 has been fixed and
+            # every SRIOV device related information is transferred through the
+            # virt driver and the PciDevice object instead of queried with
+            # sysfs calls by the network.neutron.API code.
+            self.useFixture(fixtures.MockPatch(
+                'nova.pci.utils.get_mac_by_pci_address',
+                return_value='52:54:00:1e:59:c6'))
+
+            self.useFixture(fixtures.MockPatch(
+                'nova.pci.utils.get_vf_num_by_pci_address',
+                return_value=1))
+
     def get_available_resource(self, nodename):
         host_status = super(
             FakeDriverWithPciResources, self).get_available_resource(nodename)
@@ -1068,3 +1099,23 @@ class FakeDriverWithCaching(FakeDriver):
         else:
             self.cached_images.add(image_id)
             return True
+
+
+class EphEncryptionDriver(MediumFakeDriver):
+    capabilities = dict(
+        FakeDriver.capabilities,
+        supports_ephemeral_encryption=True)
+
+
+class EphEncryptionDriverLUKS(MediumFakeDriver):
+    capabilities = dict(
+        FakeDriver.capabilities,
+        supports_ephemeral_encryption=True,
+        supports_ephemeral_encryption_luks=True)
+
+
+class EphEncryptionDriverPLAIN(MediumFakeDriver):
+    capabilities = dict(
+        FakeDriver.capabilities,
+        supports_ephemeral_encryption=True,
+        supports_ephemeral_encryption_plain=True)
