@@ -27,25 +27,32 @@ higher level APIs around the raw libvirt API. These APIs are
 then used by all the other libvirt related classes
 """
 
+import time
+import typing as ty
+
 from lxml import etree
 from oslo_log import log as logging
-from oslo_service import loopingcall
 from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import importutils
-import time
 
 from nova.compute import power_state
 from nova import exception
 from nova.i18n import _
-from nova.i18n import _LE
-from nova.i18n import _LW
-from nova import utils
 from nova.virt import hardware
-from nova.virt.libvirt import compat
 from nova.virt.libvirt import config as vconfig
 
-libvirt = None
+
+if ty.TYPE_CHECKING:
+    import libvirt
+else:
+    libvirt = None
+
+try:
+    import libvirtmod_qemu
+except ImportError:
+    libvirtmod_qemu = None
+
 
 LOG = logging.getLogger(__name__)
 
@@ -76,6 +83,25 @@ LIBVIRT_POWER_STATE = {
     VIR_DOMAIN_PMSUSPENDED: power_state.SUSPENDED,
 }
 
+# https://libvirt.org/html/libvirt-libvirt-domain.html#virDomainBlockJobType
+VIR_DOMAIN_BLOCK_JOB_TYPE_UNKNOWN = 0
+VIR_DOMAIN_BLOCK_JOB_TYPE_PULL = 1
+VIR_DOMAIN_BLOCK_JOB_TYPE_COPY = 2
+VIR_DOMAIN_BLOCK_JOB_TYPE_COMMIT = 3
+VIR_DOMAIN_BLOCK_JOB_TYPE_ACTIVE_COMMIT = 4
+VIR_DOMAIN_BLOCK_JOB_TYPE_BACKUP = 5
+VIR_DOMAIN_BLOCK_JOB_TYPE_LAST = 6
+
+LIBVIRT_BLOCK_JOB_TYPE = {
+    VIR_DOMAIN_BLOCK_JOB_TYPE_UNKNOWN: 'UNKNOWN',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_PULL: 'PULL',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_COPY: 'COPY',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_COMMIT: 'COMMIT',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_ACTIVE_COMMIT: 'ACTIVE_COMMIT',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_BACKUP: 'BACKUP',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_LAST: 'LAST',
+}
+
 
 class Guest(object):
 
@@ -120,14 +146,14 @@ class Guest(object):
         :returns guest.Guest: Guest ready to be launched
         """
         try:
-            # TODO(sahid): Host.write_instance_config should return
-            # an instance of Guest
-            domain = host.write_instance_config(xml)
+            if isinstance(xml, bytes):
+                xml = xml.decode('utf-8')
+            guest = host.write_instance_config(xml)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Error defining a domain with XML: %s'),
+                LOG.error('Error defining a guest with XML: %s',
                           encodeutils.safe_decode(xml))
-        return cls(domain)
+        return guest
 
     def launch(self, pause=False):
         """Starts a created guest.
@@ -139,15 +165,14 @@ class Guest(object):
             return self._domain.createWithFlags(flags)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Error launching a defined domain '
-                              'with XML: %s'),
-                          self._encoded_xml, errors='ignore')
+                LOG.exception('Error launching a defined domain with XML: %s',
+                              self._encoded_xml, errors='ignore')
 
     def poweroff(self):
         """Stops a running guest."""
         self._domain.destroy()
 
-    def _sync_guest_time(self):
+    def sync_guest_time(self):
         """Try to set VM time to the current value.  This is typically useful
         when clock wasn't running on the VM for some time (e.g. during
         suspension or migration), especially if the time delay exceeds NTP
@@ -168,14 +193,14 @@ class Guest(object):
             if code == libvirt.VIR_ERR_AGENT_UNRESPONSIVE:
                 LOG.debug('Failed to set time: QEMU agent unresponsive',
                           instance_uuid=self.uuid)
-            elif code == libvirt.VIR_ERR_NO_SUPPORT:
+            elif code == libvirt.VIR_ERR_OPERATION_UNSUPPORTED:
                 LOG.debug('Failed to set time: not supported',
                           instance_uuid=self.uuid)
             elif code == libvirt.VIR_ERR_ARGUMENT_UNSUPPORTED:
                 LOG.debug('Failed to set time: agent not configured',
                           instance_uuid=self.uuid)
             else:
-                LOG.warning(_LW('Failed to set time: %(reason)s'),
+                LOG.warning('Failed to set time: %(reason)s',
                             {'reason': e}, instance_uuid=self.uuid)
         except Exception as ex:
             # The highest priority is not to let this method crash and thus
@@ -192,25 +217,8 @@ class Guest(object):
         self._domain.injectNMI()
 
     def resume(self):
-        """Resumes a suspended guest."""
+        """Resumes a paused guest."""
         self._domain.resume()
-        self._sync_guest_time()
-
-    def enable_hairpin(self):
-        """Enables hairpin mode for this guest."""
-        interfaces = self.get_interfaces()
-        try:
-            for interface in interfaces:
-                utils.execute(
-                    'tee',
-                    '/sys/class/net/%s/brport/hairpin_mode' % interface,
-                    process_input='1',
-                    run_as_root=True,
-                    check_exit_code=[0, 1])
-        except Exception:
-            with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Error enabling hairpin mode with XML: %s'),
-                          self._encoded_xml, errors='ignore')
 
     def get_interfaces(self):
         """Returns a list of all network interfaces for this domain."""
@@ -229,21 +237,53 @@ class Guest(object):
 
         return interfaces
 
-    def get_interface_by_mac(self, mac):
-        """Lookup a LibvirtConfigGuestInterface by the MAC address.
+    def get_interface_by_cfg(
+        self,
+        cfg: vconfig.LibvirtConfigGuestDevice,
+        from_persistent_config: bool = False
+    ) -> ty.Optional[vconfig.LibvirtConfigGuestDevice]:
+        """Lookup a full LibvirtConfigGuestDevice with
+        LibvirtConfigGuesDevice generated
+        by nova.virt.libvirt.vif.get_config.
 
-        :param mac: MAC address of the guest interface.
-        :type mac: str
-        :returns: nova.virt.libvirt.config.LibvirtConfigGuestInterface instance
+        :param cfg: config object that represents the guest interface.
+        :param from_persistent_config: query the device from the persistent
+            domain instead of the live domain configuration
+        :returns: nova.virt.libvirt.config.LibvirtConfigGuestDevice instance
             if found, else None
         """
 
-        if mac:
+        if cfg:
+            LOG.debug(f'looking for interface given config: {cfg}')
             interfaces = self.get_all_devices(
-                vconfig.LibvirtConfigGuestInterface)
+                type(cfg), from_persistent_config)
+            if not interfaces:
+                LOG.debug(f'No interface of type: {type(cfg)} found in domain')
+                return None
+            # FIXME(sean-k-mooney): we should be able to print the list of
+            # interfaces however some tests use incomplete objects that cant
+            # be printed due to incomplete mocks or defects in the libvirt
+            # fixture. Lets address this later.
+            # LOG.debug(f'within interfaces: {list(interfaces)}')
             for interface in interfaces:
-                if interface.mac_addr == mac:
+                # NOTE(leehom) LibvirtConfigGuest get from domain and
+                # LibvirtConfigGuest generated by
+                # nova.virt.libvirt.vif.get_config must be identical.
+                # NOTE(gibi): LibvirtConfigGuest subtypes does a custom
+                # equality check based on available information on nova side
+                if cfg == interface:
                     return interface
+            else:
+                # NOTE(sean-k-mooney): {list(interfaces)} could be used
+                # instead of self._domain.XMLDesc(0) once all tests have
+                # printable interfaces see the comment above ^.
+                # While the XML is more verbose it should always work
+                # for our current test suite and in production code.
+                LOG.debug(
+                    f'interface for config: {cfg}'
+                    f'not found in domain: {self._domain.XMLDesc(0)}'
+                )
+        return None
 
     def get_vcpus_info(self):
         """Returns virtual cpus information of guest.
@@ -251,19 +291,20 @@ class Guest(object):
         :returns: guest.VCPUInfo
         """
         vcpus = self._domain.vcpus()
-        if vcpus is not None:
-            for vcpu in vcpus[0]:
-                yield VCPUInfo(
-                    id=vcpu[0], cpu=vcpu[3], state=vcpu[1], time=vcpu[2])
+        for vcpu in vcpus[0]:
+            yield VCPUInfo(
+                id=vcpu[0], cpu=vcpu[3], state=vcpu[1], time=vcpu[2])
 
-    def delete_configuration(self):
+    def delete_configuration(self, support_uefi=False):
         """Undefines a domain from hypervisor."""
         try:
-            self._domain.undefineFlags(
-                libvirt.VIR_DOMAIN_UNDEFINE_MANAGED_SAVE)
+            flags = libvirt.VIR_DOMAIN_UNDEFINE_MANAGED_SAVE
+            if support_uefi:
+                flags |= libvirt.VIR_DOMAIN_UNDEFINE_NVRAM
+            self._domain.undefineFlags(flags)
         except libvirt.libvirtError:
-            LOG.debug("Error from libvirt during undefineFlags. %d"
-                      "Retrying with undefine", self.id)
+            LOG.debug("Error from libvirt during undefineFlags for guest "
+                      "%d. Retrying with undefine", self.id)
             self._domain.undefine()
         except AttributeError:
             # Older versions of libvirt don't support undefine flags,
@@ -290,25 +331,81 @@ class Guest(object):
         """
         flags = persistent and libvirt.VIR_DOMAIN_AFFECT_CONFIG or 0
         flags |= live and libvirt.VIR_DOMAIN_AFFECT_LIVE or 0
+
         device_xml = conf.to_xml()
+        if isinstance(device_xml, bytes):
+            device_xml = device_xml.decode('utf-8')
+
         LOG.debug("attach device xml: %s", device_xml)
         self._domain.attachDeviceFlags(device_xml, flags=flags)
 
-    def get_disk(self, device):
+    def set_metadata(self, metadata, persistent=False, live=False):
+        """Set metadata to the guest.
+
+        Please note that this function completely replaces the existing
+        metadata. The scope of the replacement is limited to the Nova-specific
+        XML Namespace.
+
+        :param metadata: A LibvirtConfigGuestMetaNovaInstance
+        :param persistent: A bool to indicate whether the change is
+                           persistent or not
+        :param live: A bool to indicate whether it affect the guest
+                     in running state
+        """
+        flags = persistent and libvirt.VIR_DOMAIN_AFFECT_CONFIG or 0
+        flags |= live and libvirt.VIR_DOMAIN_AFFECT_LIVE or 0
+
+        metadata_xml = metadata.to_xml()
+        LOG.debug("set metadata xml: %s", metadata_xml)
+        self._domain.setMetadata(libvirt.VIR_DOMAIN_METADATA_ELEMENT,
+                                 metadata_xml, "instance",
+                                 vconfig.NOVA_NS, flags=flags)
+
+    def get_config(self):
+        """Returns the config instance for a guest
+
+        :returns: LibvirtConfigGuest instance
+        """
+        config = vconfig.LibvirtConfigGuest()
+        config.parse_str(self._domain.XMLDesc(0))
+        return config
+
+    def get_disk(
+        self,
+        device: str,
+        from_persistent_config: bool = False
+    ) -> ty.Optional[vconfig.LibvirtConfigGuestDisk]:
         """Returns the disk mounted at device
 
-        :returns LivirtConfigGuestDisk: mounted at device or None
+        :param device: the name of either the source or the target device
+        :param from_persistent_config: query the device from the persistent
+            domain (i.e. inactive XML configuration that'll be used on next
+            start of the domain) instead of the live domain configuration
+        :returns LibvirtConfigGuestDisk: mounted at device or None
         """
+        flags = 0
+        if from_persistent_config:
+            flags |= libvirt.VIR_DOMAIN_XML_INACTIVE
         try:
-            doc = etree.fromstring(self._domain.XMLDesc(0))
+            doc = etree.fromstring(self._domain.XMLDesc(flags))
         except Exception:
             return None
+
+        # FIXME(lyarwood): Workaround for the device being either a target dev
+        # when called via swap_volume or source file when called via
+        # live_snapshot. This should be removed once both are refactored to use
+        # only the target dev of the device.
         node = doc.find("./devices/disk/target[@dev='%s'].." % device)
+        if node is None:
+            node = doc.find("./devices/disk/source[@file='%s'].." % device)
+
         if node is not None:
             conf = vconfig.LibvirtConfigGuestDisk()
             conf.parse_dom(node)
             return conf
 
+        return None
+
     def get_all_disks(self):
         """Returns all the disks for a guest
 
@@ -317,18 +414,28 @@ class Guest(object):
 
         return self.get_all_devices(vconfig.LibvirtConfigGuestDisk)
 
-    def get_all_devices(self, devtype=None):
+    def get_all_devices(
+        self,
+        devtype: vconfig.LibvirtConfigGuestDevice = None,
+        from_persistent_config: bool = False
+    ) -> ty.List[vconfig.LibvirtConfigGuestDevice]:
         """Returns all devices for a guest
 
         :param devtype: a LibvirtConfigGuestDevice subclass class
-
+        :param from_persistent_config: query the device from the persistent
+            domain (i.e. inactive XML configuration that'll be used on next
+            start of the domain) instead of the live domain configuration
         :returns: a list of LibvirtConfigGuestDevice instances
         """
 
+        flags = 0
+        if from_persistent_config:
+            flags |= libvirt.VIR_DOMAIN_XML_INACTIVE
+
         try:
             config = vconfig.LibvirtConfigGuest()
             config.parse_str(
-                self._domain.XMLDesc(0))
+                self._domain.XMLDesc(flags))
         except Exception:
             return []
 
@@ -339,55 +446,6 @@ class Guest(object):
                 devs.append(dev)
         return devs
 
-    def detach_device_with_retry(self, get_device_conf_func, device,
-                                 persistent, live, max_retry_count=7,
-                                 inc_sleep_time=2,
-                                 max_sleep_time=30):
-        """Detaches a device from the guest. After the initial detach request,
-        a function is returned which can be used to ensure the device is
-        successfully removed from the guest domain (retrying the removal as
-        necessary).
-
-        :param get_device_conf_func: function which takes device as a parameter
-                                     and returns the configuration for device
-        :param device: device to detach
-        :param persistent: bool to indicate whether the change is
-                           persistent or not
-        :param live: bool to indicate whether it affects the guest in running
-                     state
-        :param max_retry_count: number of times the returned function will
-                                retry a detach before failing
-        :param inc_sleep_time: incremental time to sleep in seconds between
-                               detach retries
-        :param max_sleep_time: max sleep time in seconds beyond which the sleep
-                               time will not be incremented using param
-                               inc_sleep_time. On reaching this threshold,
-                               max_sleep_time will be used as the sleep time.
-        """
-
-        conf = get_device_conf_func(device)
-        if conf is None:
-            raise exception.DeviceNotFound(device=device)
-
-        self.detach_device(conf, persistent, live)
-
-        @loopingcall.RetryDecorator(max_retry_count=max_retry_count,
-                                    inc_sleep_time=inc_sleep_time,
-                                    max_sleep_time=max_sleep_time,
-                                    exceptions=exception.DeviceDetachFailed)
-        def _do_wait_and_retry_detach():
-            config = get_device_conf_func(device)
-            if config is not None:
-                # Device is already detached from persistent domain
-                # and only transient domain needs update
-                self.detach_device(config, persistent=False, live=live)
-                # Raise error since the device still existed on the guest
-                reason = _("Unable to detach from guest transient domain.")
-                raise exception.DeviceDetachFailed(device=device,
-                                                   reason=reason)
-
-        return _do_wait_and_retry_detach
-
     def detach_device(self, conf, persistent=False, live=False):
         """Detaches device to the guest.
 
@@ -399,7 +457,11 @@ class Guest(object):
         """
         flags = persistent and libvirt.VIR_DOMAIN_AFFECT_CONFIG or 0
         flags |= live and libvirt.VIR_DOMAIN_AFFECT_LIVE or 0
+
         device_xml = conf.to_xml()
+        if isinstance(device_xml, bytes):
+            device_xml = device_xml.decode('utf-8')
+
         LOG.debug("detach device xml: %s", device_xml)
         self._domain.detachDeviceFlags(device_xml, flags=flags)
 
@@ -433,16 +495,12 @@ class Guest(object):
         """Configures a new user password."""
         self._domain.setUserPassword(user, new_pass, 0)
 
-    def _get_domain_info(self, host):
-        """Returns information on Guest
-
-        :param host: a host.Host object with current
-                     connection. Unfortunately we need to pass it
-                     because of a workaround with < version 1.2..11
+    def _get_domain_info(self):
+        """Returns information on Guest.
 
         :returns list: [state, maxMem, memory, nrVirtCpu, cpuTime]
         """
-        return compat.get_domain_info(libvirt, host, self._domain)
+        return self._domain.info()
 
     def get_info(self, host):
         """Retrieve information from libvirt for a specific instance name.
@@ -454,7 +512,7 @@ class Guest(object):
         :returns hardware.InstanceInfo:
         """
         try:
-            dom_info = self._get_domain_info(host)
+            dom_info = self._get_domain_info()
         except libvirt.libvirtError as ex:
             error_code = ex.get_error_code()
             if error_code == libvirt.VIR_ERR_NO_DOMAIN:
@@ -465,15 +523,11 @@ class Guest(object):
                    {'instance_name': self.name,
                     'error_code': error_code,
                     'ex': ex})
-            raise exception.NovaException(msg)
+            raise exception.InternalError(msg)
 
         return hardware.InstanceInfo(
             state=LIBVIRT_POWER_STATE[dom_info[0]],
-            max_mem_kb=dom_info[1],
-            mem_kb=dom_info[2],
-            num_cpu=dom_info[3],
-            cpu_time_ns=dom_info[4],
-            id=self.id)
+            internal_id=self.id)
 
     def get_power_state(self, host):
         return self.get_info(host).state
@@ -498,16 +552,21 @@ class Guest(object):
         :param no_metadata: Make snapshot without remembering it
         :param disk_only: Disk snapshot, no system checkpoint
         :param reuse_ext: Reuse any existing external files
-        :param quiesce: Use QGA to quiece all mounted file systems
+        :param quiesce: Use QGA to quiesce all mounted file systems
         """
-        flags = no_metadata and (libvirt.VIR_DOMAIN_SNAPSHOT_CREATE_NO_METADATA
-                                 or 0)
-        flags |= disk_only and (libvirt.VIR_DOMAIN_SNAPSHOT_CREATE_DISK_ONLY
-                                or 0)
-        flags |= reuse_ext and (libvirt.VIR_DOMAIN_SNAPSHOT_CREATE_REUSE_EXT
-                                or 0)
+        flags = no_metadata and (
+            libvirt.VIR_DOMAIN_SNAPSHOT_CREATE_NO_METADATA or 0)
+        flags |= disk_only and (
+            libvirt.VIR_DOMAIN_SNAPSHOT_CREATE_DISK_ONLY or 0)
+        flags |= reuse_ext and (
+            libvirt.VIR_DOMAIN_SNAPSHOT_CREATE_REUSE_EXT or 0)
         flags |= quiesce and libvirt.VIR_DOMAIN_SNAPSHOT_CREATE_QUIESCE or 0
-        self._domain.snapshotCreateXML(conf.to_xml(), flags=flags)
+
+        device_xml = conf.to_xml()
+        if isinstance(device_xml, bytes):
+            device_xml = device_xml.decode('utf-8')
+
+        self._domain.snapshotCreateXML(device_xml, flags=flags)
 
     def shutdown(self):
         """Shutdown guest"""
@@ -524,11 +583,14 @@ class Guest(object):
         """
         self._domain.suspend()
 
-    def migrate(self, destination, params=None, flags=0, domain_xml=None,
-                bandwidth=0):
+    def migrate(self, destination, migrate_uri=None, migrate_disks=None,
+                destination_xml=None, flags=0, bandwidth=0):
         """Migrate guest object from its current host to the destination
 
         :param destination: URI of host destination where guest will be migrate
+        :param migrate_uri: URI for invoking the migration
+        :param migrate_disks: List of disks to be migrated
+        :param destination_xml: The guest XML to be used on the target host
         :param flags: May be one of more of the following:
            VIR_MIGRATE_LIVE Do not pause the VM during migration
            VIR_MIGRATE_PEER2PEER Direct connection between source &
@@ -541,35 +603,44 @@ class Guest(object):
            VIR_MIGRATE_UNDEFINE_SOURCE If the migration is successful,
                                        undefine the domain on the
                                        source host.
-           VIR_MIGRATE_PAUSED Leave the domain suspended on the remote
-                              side.
-           VIR_MIGRATE_NON_SHARED_DISK Migration with non-shared
-                                       storage with full disk copy
            VIR_MIGRATE_NON_SHARED_INC Migration with non-shared
                                       storage with incremental disk
                                       copy
-           VIR_MIGRATE_CHANGE_PROTECTION Protect against domain
-                                         configuration changes during
-                                         the migration process (set
-                                         automatically when
-                                         supported).
-           VIR_MIGRATE_UNSAFE Force migration even if it is considered
-                              unsafe.
-           VIR_MIGRATE_OFFLINE Migrate offline
-        :param domain_xml: Changing guest configuration during migration
-        :param bandwidth: The maximun bandwidth in MiB/s
+           VIR_MIGRATE_AUTO_CONVERGE Slow down domain to make sure it does
+                                     not change its memory faster than a
+                                     hypervisor can transfer the changed
+                                     memory to the destination host
+           VIR_MIGRATE_POSTCOPY Tell libvirt to enable post-copy migration
+           VIR_MIGRATE_TLS Use QEMU-native TLS
+        :param bandwidth: The maximum bandwidth in MiB/s
         """
-        if domain_xml is None:
-            self._domain.migrateToURI(
-                destination, flags=flags, bandwidth=bandwidth)
-        else:
-            if params:
-                self._domain.migrateToURI3(
-                    destination, params=params, flags=flags)
-            else:
-                self._domain.migrateToURI2(
-                    destination, dxml=domain_xml,
-                    flags=flags, bandwidth=bandwidth)
+        params = {}
+        # In migrateToURI3 these parameters are extracted from the
+        # `params` dict
+        params['bandwidth'] = bandwidth
+
+        if destination_xml:
+            params['destination_xml'] = destination_xml
+            params['persistent_xml'] = destination_xml
+        if migrate_disks:
+            params['migrate_disks'] = migrate_disks
+        if migrate_uri:
+            params['migrate_uri'] = migrate_uri
+
+        # Due to a quirk in the libvirt python bindings,
+        # VIR_MIGRATE_NON_SHARED_INC with an empty migrate_disks is
+        # interpreted as "block migrate all writable disks" rather than
+        # "don't block migrate any disks". This includes attached
+        # volumes, which will potentially corrupt data on those
+        # volumes. Consequently we need to explicitly unset
+        # VIR_MIGRATE_NON_SHARED_INC if there are no disks to be block
+        # migrated.
+        if (flags & libvirt.VIR_MIGRATE_NON_SHARED_INC != 0 and
+                not params.get('migrate_disks')):
+            flags &= ~libvirt.VIR_MIGRATE_NON_SHARED_INC
+
+        self._domain.migrateToURI3(
+            destination, params=params, flags=flags)
 
     def abort_job(self):
         """Requests to abort current background job"""
@@ -586,6 +657,10 @@ class Guest(object):
         """Switch running live migration to post-copy mode"""
         self._domain.migrateStartPostCopy()
 
+    def announce_self(self):
+        libvirtmod_qemu.virDomainQemuMonitorCommand(
+            self._domain._o, 'announce_self', 1)
+
     def get_job_info(self):
         """Get job info for the domain
 
@@ -599,6 +674,7 @@ class Guest(object):
                 stats = self._domain.jobStats()
                 return JobInfo(**stats)
             except libvirt.libvirtError as ex:
+                errmsg = ex.get_error_message()
                 if ex.get_error_code() == libvirt.VIR_ERR_NO_SUPPORT:
                     # Remote libvirt doesn't support new API
                     LOG.debug("Missing remote virDomainGetJobStats: %s", ex)
@@ -611,6 +687,12 @@ class Guest(object):
                     # away completclsely
                     LOG.debug("Domain has shutdown/gone away: %s", ex)
                     return JobInfo(type=libvirt.VIR_DOMAIN_JOB_COMPLETED)
+                elif (ex.get_error_code() == libvirt.VIR_ERR_INTERNAL_ERROR and
+                      errmsg and "migration was active, "
+                      "but no RAM info was set" in errmsg):
+                    LOG.debug("Migration is active or completed but "
+                              "virDomainGetJobStats is missing ram: %s", ex)
+                    return JobInfo(type=libvirt.VIR_DOMAIN_JOB_NONE)
                 else:
                     LOG.debug("Failed to get job stats: %s", ex)
                     raise
@@ -633,51 +715,88 @@ class BlockDevice(object):
         self._guest = guest
         self._disk = disk
 
-    def abort_job(self, async=False, pivot=False):
-        """Request to cancel any job currently running on the block.
+    def abort_job(self, async_=False, pivot=False):
+        """Request to cancel a live block device job
 
-        :param async: Request only, do not wait for completion
-        :param pivot: Pivot to new file when ending a copy or
-                      active commit job
+        :param async_: Cancel the block device job (e.g. 'copy' or
+                       'commit'), and return as soon as possible, without
+                       waiting for job completion
+        :param pivot: Pivot to the destination image when ending a
+                      'copy' or "active commit" (meaning: merging the
+                      contents of current active disk into its backing
+                      file) job
         """
-        flags = async and libvirt.VIR_DOMAIN_BLOCK_JOB_ABORT_ASYNC or 0
+        flags = async_ and libvirt.VIR_DOMAIN_BLOCK_JOB_ABORT_ASYNC or 0
         flags |= pivot and libvirt.VIR_DOMAIN_BLOCK_JOB_ABORT_PIVOT or 0
         self._guest._domain.blockJobAbort(self._disk, flags=flags)
 
     def get_job_info(self):
         """Returns information about job currently running
 
-        :returns: BlockDeviceJobInfo or None
+        :returns: BlockDeviceJobInfo, or None if no job exists
+        :raises: libvirt.libvirtError on error fetching block job info
         """
+
+        # libvirt's blockJobInfo() raises libvirt.libvirtError if there was an
+        # error. It returns {} if the job no longer exists, or a fully
+        # populated dict if the job exists.
         status = self._guest._domain.blockJobInfo(self._disk, flags=0)
-        if status != -1:
-            return BlockDeviceJobInfo(
-                job=status.get("type", 0),
-                bandwidth=status.get("bandwidth", 0),
-                cur=status.get("cur", 0),
-                end=status.get("end", 0))
+
+        # The job no longer exists
+        if not status:
+            return None
+
+        return BlockDeviceJobInfo(
+            job=status['type'],
+            bandwidth=status['bandwidth'],
+            cur=status['cur'],
+            end=status['end'])
+
+    def copy(self, dest_xml, shallow=False, reuse_ext=False, transient=False):
+        """Copy the guest-visible contents into a new disk
+
+        http://libvirt.org/html/libvirt-libvirt-domain.html#virDomainBlockCopy
+
+        :param: dest_xml: XML describing the destination disk to copy to
+        :param: shallow: Limit copy to top of source backing chain
+        :param: reuse_ext: Reuse existing external file for a copy
+        :param: transient: Don't force usage of recoverable job for the copy
+                           operation
+         """
+        flags = shallow and libvirt.VIR_DOMAIN_BLOCK_COPY_SHALLOW or 0
+        flags |= reuse_ext and libvirt.VIR_DOMAIN_BLOCK_COPY_REUSE_EXT or 0
+        flags |= transient and libvirt.VIR_DOMAIN_BLOCK_COPY_TRANSIENT_JOB or 0
+        return self._guest._domain.blockCopy(self._disk, dest_xml, flags=flags)
 
     def rebase(self, base, shallow=False, reuse_ext=False,
-               copy=False, relative=False):
-        """Rebases block to new base
+               copy=False, relative=False, copy_dev=False):
+        """Copy data from backing chain into a new disk
 
-        :param shallow: Limit copy to top of source backing chain
-        :param reuse_ext: Reuse existing external file of a copy
+        This copies data from backing file(s) into overlay(s), giving
+        control over several aspects like what part of a disk image
+        chain to be copied, whether to reuse an existing destination
+        file, etc.  And updates the backing file to the new disk
+
+        :param shallow: Limit copy to top of the source backing chain
+        :param reuse_ext: Reuse an existing external file that was
+                          pre-created
         :param copy: Start a copy job
         :param relative: Keep backing chain referenced using relative names
+        :param copy_dev: Treat the destination as type="block"
         """
         flags = shallow and libvirt.VIR_DOMAIN_BLOCK_REBASE_SHALLOW or 0
         flags |= reuse_ext and libvirt.VIR_DOMAIN_BLOCK_REBASE_REUSE_EXT or 0
         flags |= copy and libvirt.VIR_DOMAIN_BLOCK_REBASE_COPY or 0
+        flags |= copy_dev and libvirt.VIR_DOMAIN_BLOCK_REBASE_COPY_DEV or 0
         flags |= relative and libvirt.VIR_DOMAIN_BLOCK_REBASE_RELATIVE or 0
         return self._guest._domain.blockRebase(
             self._disk, base, self.REBASE_DEFAULT_BANDWIDTH, flags=flags)
 
     def commit(self, base, top, relative=False):
-        """Commit on block device
+        """Merge data from overlays into backing file
 
-        For performance during live snapshot it will reduces the disk chain
-        to a single disk.
+        This live merges (or "commits") contents from backing files into
+        overlays, thus reducing the length of a disk image chain.
 
         :param relative: Keep backing chain referenced using relative names
         """
@@ -685,43 +804,61 @@ class BlockDevice(object):
         return self._guest._domain.blockCommit(
             self._disk, base, top, self.COMMIT_DEFAULT_BANDWIDTH, flags=flags)
 
-    def resize(self, size_kb):
-        """Resizes block device to Kib size."""
-        self._guest._domain.blockResize(self._disk, size_kb)
+    def resize(self, size):
+        """Resize block device to the given size in bytes.
 
-    def wait_for_job(self, abort_on_error=False, wait_for_job_clean=False):
-        """Wait for libvirt block job to complete.
+        This resizes the block device within the instance to the given size.
 
-        Libvirt may return either cur==end or an empty dict when
-        the job is complete, depending on whether the job has been
-        cleaned up by libvirt yet, or not.
-        It can also return end=0 if qemu has not yet started the block
-        operation.
-
-        :param abort_on_error: Whether to stop process and raise NovaException
-                               on error (default: False)
-        :param wait_for_job_clean: Whether to force wait to ensure job is
-                                   finished (see bug: RH Bugzilla#1119173)
-
-        :returns: True if still in progress
-                  False if completed
+        :param size: The size to resize the device to in bytes.
         """
+        flags = libvirt.VIR_DOMAIN_BLOCK_RESIZE_BYTES
+        self._guest._domain.blockResize(self._disk, size, flags=flags)
+
+    def is_job_complete(self):
+        """Return True if the job is complete, False otherwise
+
+        :returns: True if the job is complete, False otherwise
+        :raises: libvirt.libvirtError on error fetching block job info
+        """
+        # NOTE(mdbooth): This method polls for block job completion. It returns
+        # true if either we get a status which indicates completion, or there
+        # is no longer a record of the job. Ideally this method and its
+        # callers would be rewritten to consume libvirt events from the job.
+        # This would provide a couple of advantages. Firstly, as it would no
+        # longer be polling it would notice completion immediately rather than
+        # at the next 0.5s check, and would also consume fewer resources.
+        # Secondly, with the current method we only know that 'no job'
+        # indicates completion. It does not necessarily indicate successful
+        # completion: the job could have failed, or been cancelled. When
+        # polling for block job info we have no way to detect this, so we
+        # assume success.
+
         status = self.get_job_info()
-        if not status:
-            if abort_on_error:
-                msg = _('libvirt error while requesting blockjob info.')
-                raise exception.NovaException(msg)
-            return False
 
-        if wait_for_job_clean:
-            job_ended = status.job == 0
-        else:
-            # NOTE(slaweq): because of bug in libvirt, which is described in
-            # http://www.redhat.com/archives/libvir-list/2016-September/msg00017.html
-            # if status.end == 0 job is not started yet so it is not finished
-            job_ended = status.end != 0 and status.cur == status.end
+        # If the job no longer exists, it is because it has completed
+        # NOTE(mdbooth): See comment above: it may not have succeeded.
+        if status is None:
+            return True
 
-        return not job_ended
+        # Track blockjob progress in DEBUG, helpful when reviewing failures.
+        job_type = LIBVIRT_BLOCK_JOB_TYPE.get(
+            status.job, f"Unknown to Nova ({status.job})")
+        LOG.debug("%(job_type)s block job progress, current cursor: %(cur)s "
+                  "final cursor: %(end)s",
+                  {'job_type': job_type, 'cur': status.cur, 'end': status.end})
+
+        # NOTE(lyarwood): Use the mirror element to determine if we can pivot
+        # to the new disk once blockjobinfo reports progress as complete.
+        if status.cur == status.end:
+            disk = self._guest.get_disk(self._disk)
+            if disk and disk.mirror:
+                return disk.mirror.ready == 'yes'
+
+        return False
+
+    def blockStats(self):
+        """Extracts block device statistics for a domain"""
+        return self._guest._domain.blockStats(self._disk)
 
 
 class VCPUInfo(object):
