@@ -19,6 +19,7 @@
 """VIF drivers for libvirt."""
 
 import os
+import typing as ty
 
 import os_vif
 from os_vif import exception as osv_exception
@@ -38,8 +39,10 @@ from nova.pci import utils as pci_utils
 import nova.privsep.linux_net
 from nova import profiler
 from nova import utils
+from nova.virt import hardware
 from nova.virt.libvirt import config as vconfig
 from nova.virt.libvirt import designer
+from nova.virt.libvirt import host as libvirt_host
 from nova.virt import osinfo
 
 
@@ -57,7 +60,8 @@ SUPPORTED_VIF_MODELS = {
         network_model.VIF_MODEL_E1000E,
         network_model.VIF_MODEL_LAN9118,
         network_model.VIF_MODEL_SPAPR_VLAN,
-        network_model.VIF_MODEL_VMXNET3],
+        network_model.VIF_MODEL_VMXNET3,
+    ],
     'kvm': [
         network_model.VIF_MODEL_VIRTIO,
         network_model.VIF_MODEL_NE2K_PCI,
@@ -66,19 +70,14 @@ SUPPORTED_VIF_MODELS = {
         network_model.VIF_MODEL_E1000,
         network_model.VIF_MODEL_E1000E,
         network_model.VIF_MODEL_SPAPR_VLAN,
-        network_model.VIF_MODEL_VMXNET3],
-    'xen': [
-        network_model.VIF_MODEL_NETFRONT,
-        network_model.VIF_MODEL_NE2K_PCI,
-        network_model.VIF_MODEL_PCNET,
-        network_model.VIF_MODEL_RTL8139,
-        network_model.VIF_MODEL_E1000],
+        network_model.VIF_MODEL_VMXNET3,
+    ],
     'lxc': [],
-    'uml': [],
     'parallels': [
         network_model.VIF_MODEL_VIRTIO,
         network_model.VIF_MODEL_RTL8139,
-        network_model.VIF_MODEL_E1000],
+        network_model.VIF_MODEL_E1000,
+    ],
 }
 
 
@@ -152,6 +151,10 @@ def ensure_vlan(vlan_num, bridge_interface, mac_address=None, mtu=None,
 class LibvirtGenericVIFDriver(object):
     """Generic VIF driver for libvirt networking."""
 
+    def __init__(self, host: libvirt_host.Host = None):
+        super().__init__()
+        self.host = host
+
     def get_vif_devname(self, vif):
         if 'devname' in vif:
             return vif['devname']
@@ -175,8 +178,9 @@ class LibvirtGenericVIFDriver(object):
 
         return model
 
-    def get_base_config(self, instance, mac, image_meta,
-                        inst_type, virt_type, vnic_type):
+    def get_base_config(
+        self, instance, mac, image_meta, flavor, virt_type, vnic_type,
+    ):
         # TODO(sahid): We should rewrite it. This method handles too
         # many unrelated things. We probably need to have a specific
         # virtio, vhost, vhostuser functions.
@@ -216,8 +220,8 @@ class LibvirtGenericVIFDriver(object):
             driver = 'qemu'
 
         if virt_type in ('kvm', 'parallels'):
-            vhost_drv, vhost_queues = self._get_virtio_mq_settings(image_meta,
-                                                                   inst_type)
+            vhost_drv, vhost_queues = self._get_virtio_mq_settings(
+                image_meta, flavor)
             # TODO(sahid): It seems that we return driver 'vhost' even
             # for vhostuser interface where for vhostuser interface
             # the driver should be 'vhost-user'. That currently does
@@ -229,7 +233,7 @@ class LibvirtGenericVIFDriver(object):
 
         if driver == 'vhost' or driver is None:
             # vhost backend only supports update of RX queue size
-            if CONF.libvirt.rx_queue_size:
+            if rx_queue_size:
                 # TODO(sahid): Specifically force driver to be vhost
                 # that because if None we don't generate the XML
                 # driver element needed to set the queue size
@@ -253,9 +257,12 @@ class LibvirtGenericVIFDriver(object):
         """A methods to set the number of virtio queues,
            if it has been requested in extra specs.
         """
+        if not isinstance(image_meta, objects.ImageMeta):
+            image_meta = objects.ImageMeta.from_dict(image_meta)
+
         driver = None
         vhost_queues = None
-        if self._requests_multiqueue(image_meta):
+        if hardware.get_vif_multiqueue_constraint(flavor, image_meta):
             driver = 'vhost'
             max_tap_queues = self._get_max_tap_queues()
             if max_tap_queues:
@@ -266,19 +273,6 @@ class LibvirtGenericVIFDriver(object):
 
         return (driver, vhost_queues)
 
-    def _requests_multiqueue(self, image_meta):
-        """Check if multiqueue property is set in the image metadata."""
-
-        if not isinstance(image_meta, objects.ImageMeta):
-            image_meta = objects.ImageMeta.from_dict(image_meta)
-
-        img_props = image_meta.properties
-
-        if img_props.get('hw_vif_multiqueue_enabled'):
-            return True
-
-        return False
-
     def _get_max_tap_queues(self):
         # Note(sean-k-mooney): some linux distros have backported
         # changes for newer kernels which make the kernel version
@@ -292,7 +286,7 @@ class LibvirtGenericVIFDriver(object):
         # In kernels 3.x, the number of queues on a tap interface
         # is limited to 8. From 4.0, the number is 256.
         # See: https://bugs.launchpad.net/nova/+bug/1570631
-        kernel_version = int(os.uname()[2].split(".")[0])
+        kernel_version = int(os.uname().release.split(".")[0])
         if kernel_version <= 2:
             return 1
         elif kernel_version == 3:
@@ -309,10 +303,10 @@ class LibvirtGenericVIFDriver(object):
         return (("qvb%s" % iface_id)[:network_model.NIC_NAME_LEN],
                 ("qvo%s" % iface_id)[:network_model.NIC_NAME_LEN])
 
-    def get_config_802qbg(self, instance, vif, image_meta,
-                          inst_type, virt_type):
-        conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'])
+    def get_config_802qbg(self, instance, vif, image_meta, flavor, virt_type):
+        conf = self.get_base_config(
+            instance, vif['address'], image_meta, flavor, virt_type,
+            vif['vnic_type'])
 
         params = vif["qbg_params"]
         designer.set_vif_host_backend_802qbg_config(
@@ -322,14 +316,14 @@ class LibvirtGenericVIFDriver(object):
             params['typeidversion'],
             params['instanceid'])
 
-        designer.set_vif_bandwidth_config(conf, inst_type)
+        designer.set_vif_bandwidth_config(conf, flavor)
 
         return conf
 
-    def get_config_802qbh(self, instance, vif, image_meta,
-                          inst_type, virt_type):
-        conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'])
+    def get_config_802qbh(self, instance, vif, image_meta, flavor, virt_type):
+        conf = self.get_base_config(
+            instance, vif['address'], image_meta, flavor, virt_type,
+            vif['vnic_type'])
 
         profile = vif["profile"]
         vif_details = vif["details"]
@@ -341,37 +335,39 @@ class LibvirtGenericVIFDriver(object):
             conf, net_type, profile['pci_slot'],
             vif_details[network_model.VIF_DETAILS_PROFILEID])
 
-        designer.set_vif_bandwidth_config(conf, inst_type)
+        designer.set_vif_bandwidth_config(conf, flavor)
 
         return conf
 
-    def get_config_hw_veb(self, instance, vif, image_meta,
-                          inst_type, virt_type):
-        conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'])
+    def get_config_hw_veb(self, instance, vif, image_meta, flavor, virt_type):
+        conf = self.get_base_config(
+            instance, vif['address'], image_meta, flavor, virt_type,
+            vif['vnic_type'])
 
         profile = vif["profile"]
         vif_details = vif["details"]
         net_type = 'direct'
-        if vif['vnic_type'] == network_model.VNIC_TYPE_DIRECT:
+        if vif['vnic_type'] in [network_model.VNIC_TYPE_DIRECT,
+            network_model.VNIC_TYPE_ACCELERATOR_DIRECT]:
             net_type = 'hostdev'
 
         designer.set_vif_host_backend_hw_veb(
             conf, net_type, profile['pci_slot'],
             vif_details[network_model.VIF_DETAILS_VLAN])
 
-        designer.set_vif_bandwidth_config(conf, inst_type)
+        designer.set_vif_bandwidth_config(conf, flavor)
 
         return conf
 
-    def get_config_hostdev_physical(self, instance, vif, image_meta,
-                                    inst_type, virt_type):
+    def get_config_hostdev_physical(
+        self, instance, vif, image_meta, flavor, virt_type,
+    ):
         return self.get_base_hostdev_pci_config(vif)
 
-    def get_config_macvtap(self, instance, vif, image_meta,
-                           inst_type, virt_type):
-        conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'])
+    def get_config_macvtap(self, instance, vif, image_meta, flavor, virt_type):
+        conf = self.get_base_config(
+            instance, vif['address'], image_meta, flavor, virt_type,
+            vif['vnic_type'])
 
         vif_details = vif['details']
         macvtap_src = vif_details.get(network_model.VIF_DETAILS_MACVTAP_SOURCE)
@@ -395,35 +391,36 @@ class LibvirtGenericVIFDriver(object):
         designer.set_vif_host_backend_direct_config(
             conf, macvtap_src, macvtap_mode)
 
-        designer.set_vif_bandwidth_config(conf, inst_type)
+        designer.set_vif_bandwidth_config(conf, flavor)
 
         return conf
 
-    def get_config_iovisor(self, instance, vif, image_meta,
-                           inst_type, virt_type):
-        conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'])
+    def get_config_iovisor(self, instance, vif, image_meta, flavor, virt_type):
+        conf = self.get_base_config(
+            instance, vif['address'], image_meta, flavor, virt_type,
+            vif['vnic_type'])
 
         dev = self.get_vif_devname(vif)
         designer.set_vif_host_backend_ethernet_config(conf, dev)
 
-        designer.set_vif_bandwidth_config(conf, inst_type)
+        designer.set_vif_bandwidth_config(conf, flavor)
 
         return conf
 
-    def get_config_midonet(self, instance, vif, image_meta,
-                           inst_type, virt_type):
-        conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'])
+    def get_config_midonet(self, instance, vif, image_meta, flavor, virt_type):
+        conf = self.get_base_config(
+            instance, vif['address'], image_meta, flavor, virt_type,
+            vif['vnic_type'])
 
         dev = self.get_vif_devname(vif)
         designer.set_vif_host_backend_ethernet_config(conf, dev)
 
         return conf
 
-    def get_config_tap(self, instance, vif, image_meta, inst_type, virt_type):
-        conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'])
+    def get_config_tap(self, instance, vif, image_meta, flavor, virt_type):
+        conf = self.get_base_config(
+            instance, vif['address'], image_meta, flavor, virt_type,
+            vif['vnic_type'])
 
         dev = self.get_vif_devname(vif)
         designer.set_vif_host_backend_ethernet_config(conf, dev)
@@ -434,8 +431,9 @@ class LibvirtGenericVIFDriver(object):
 
         return conf
 
-    def get_config_ib_hostdev(self, instance, vif, image_meta,
-                              inst_type, virt_type):
+    def get_config_ib_hostdev(
+        self, instance, vif, image_meta, flavor, virt_type,
+    ):
         return self.get_base_hostdev_pci_config(vif)
 
     def _set_config_VIFGeneric(self, instance, vif, conf):
@@ -448,10 +446,15 @@ class LibvirtGenericVIFDriver(object):
         conf.target_dev = vif.vif_name
 
     def _set_config_VIFOpenVSwitch(self, instance, vif, conf):
-        conf.net_type = "bridge"
-        conf.source_dev = vif.bridge_name
-        conf.target_dev = vif.vif_name
-        self._set_config_VIFPortProfile(instance, vif, conf)
+        # if delegating creation to os-vif, create an ethernet-type VIF and let
+        # os-vif do the actual wiring up
+        if 'create_port' in vif.port_profile and vif.port_profile.create_port:
+            self._set_config_VIFGeneric(instance, vif, conf)
+        else:
+            conf.net_type = "bridge"
+            conf.source_dev = vif.bridge_name
+            conf.target_dev = vif.vif_name
+            self._set_config_VIFPortProfile(instance, vif, conf)
 
     def _set_config_VIFVHostUser(self, instance, vif, conf):
         # TODO(sahid): We should never configure a driver backend for
@@ -494,14 +497,22 @@ class LibvirtGenericVIFDriver(object):
             raise exception.InternalError(
                 _('Unsupported VIF port profile type %s') % profile_name)
 
-    def _get_config_os_vif(self, instance, vif, image_meta, inst_type,
-                           virt_type, vnic_type):
+    def _get_vdpa_dev_path(self, pci_address: ty.Text) -> ty.Text:
+        if self.host is not None:
+            return self.host.get_vdpa_device_path(pci_address)
+        # TODO(sean-k-mooney) this should never be raised remove when host
+        # is not optional in __init__.
+        raise TypeError("self.host must set to use this function.")
+
+    def _get_config_os_vif(
+        self, instance, vif, image_meta, flavor, virt_type, vnic_type,
+    ):
         """Get the domain config for a VIF
 
         :param instance: nova.objects.Instance
         :param vif: os_vif.objects.vif.VIFBase subclass
         :param image_meta: nova.objects.ImageMeta
-        :param inst_type: nova.objects.Flavor
+        :param flavor: nova.objects.Flavor
         :param virt_type: virtualization type
         :param vnic_type: vnic type
 
@@ -509,8 +520,8 @@ class LibvirtGenericVIFDriver(object):
         """
 
         # Do the config that's common to all vif types
-        conf = self.get_base_config(instance, vif.address, image_meta,
-                                    inst_type, virt_type, vnic_type)
+        conf = self.get_base_config(
+            instance, vif.address, image_meta, flavor, virt_type, vnic_type)
 
         # Do the VIF type specific config
         if isinstance(vif, osv_vifs.VIFGeneric):
@@ -522,7 +533,13 @@ class LibvirtGenericVIFDriver(object):
         elif isinstance(vif, osv_vifs.VIFVHostUser):
             self._set_config_VIFVHostUser(instance, vif, conf)
         elif isinstance(vif, osv_vifs.VIFHostDevice):
-            self._set_config_VIFHostDevice(instance, vif, conf)
+            if vnic_type != network_model.VNIC_TYPE_VDPA:
+                self._set_config_VIFHostDevice(instance, vif, conf)
+            else:
+                dev_path = self._get_vdpa_dev_path(vif.dev_address)
+                designer.set_vif_host_backend_vdpa_config(
+                    conf, dev_path, CONF.libvirt.rx_queue_size,
+                    CONF.libvirt.tx_queue_size)
         else:
             raise exception.InternalError(
                 _("Unsupported VIF type %s") % vif.obj_name())
@@ -530,14 +547,14 @@ class LibvirtGenericVIFDriver(object):
         # not all VIF types support bandwidth configuration
         # https://github.com/libvirt/libvirt/blob/568a41722/src/conf/netdev_bandwidth_conf.h#L38
         if vif.obj_name() not in ('VIFVHostUser', 'VIFHostDevice'):
-            designer.set_vif_bandwidth_config(conf, inst_type)
+            designer.set_vif_bandwidth_config(conf, flavor)
 
         if 'network' in vif and 'mtu' in vif.network:
             designer.set_vif_mtu_config(conf, vif.network.mtu)
 
         return conf
 
-    def get_config(self, instance, vif, image_meta, inst_type, virt_type):
+    def get_config(self, instance, vif, image_meta, flavor, virt_type):
         vif_type = vif['type']
         vnic_type = vif['vnic_type']
 
@@ -556,11 +573,11 @@ class LibvirtGenericVIFDriver(object):
         # Try os-vif codepath first
         vif_obj = os_vif_util.nova_to_osvif_vif(vif)
         if vif_obj is not None:
-            return self._get_config_os_vif(instance, vif_obj, image_meta,
-                                           inst_type, virt_type, vnic_type)
+            return self._get_config_os_vif(
+                instance, vif_obj, image_meta, flavor, virt_type, vnic_type)
 
         # Legacy non-os-vif codepath
-        args = (instance, vif, image_meta, inst_type, virt_type)
+        args = (instance, vif, image_meta, flavor, virt_type)
         if vif_type == network_model.VIF_TYPE_IOVISOR:
             return self.get_config_iovisor(*args)
         elif vif_type == network_model.VIF_TYPE_802_QBG:
@@ -606,7 +623,7 @@ class LibvirtGenericVIFDriver(object):
         #  2. libvirt driver does not change mac address for macvtap VNICs
         #     or Alternatively does not rely on recreating libvirt's nodev
         #     name from the current mac address set on the netdevice.
-        #     See: virt.libvrit.driver.LibvirtDriver._get_pcinet_info
+        #     See: virt.libvirt.driver.LibvirtDriver._get_pcinet_info
         if vif['vnic_type'] == network_model.VNIC_TYPE_MACVTAP:
             set_vf_interface_vlan(
                 vif['profile']['pci_slot'],
@@ -666,9 +683,10 @@ class LibvirtGenericVIFDriver(object):
         vif_model = self.get_vif_model(image_meta=image_meta)
         # TODO(ganso): explore whether multiqueue works for other vif models
         # that go through this code path.
-        multiqueue = (instance.get_flavor().vcpus > 1 and
-                      self._requests_multiqueue(image_meta) and
-                      vif_model == network_model.VIF_MODEL_VIRTIO)
+        multiqueue = False
+        if vif_model == network_model.VIF_MODEL_VIRTIO:
+            multiqueue = hardware.get_vif_multiqueue_constraint(
+                instance.flavor, image_meta)
         nova.privsep.linux_net.create_tap_dev(dev, mac, multiqueue=multiqueue)
         network = vif.get('network')
         mtu = network.get_meta('mtu') if network else None
